/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import org.vono.narau.db.CommonDB;
import org.vono.narau.db.CommonDB.Database;
import org.vono.narau.download.DownloaderActivity;
import org.vono.narau.preferences.PreferencesActivity;

import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.NotificationCompat;
import android.support.v4.app.TaskStackBuilder;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.TextView;

/**
 * 
 * <p>
 * <b>Narau</b> 習う (ならう)<br/>
 * to take lessons in/to be taught/to learn (from a teacher)/to study (under a
 * teacher)/to get training in
 * </p>
 * The menu of this application.
 */
public class Narau extends FragmentActivity {

	/**
	 * Log tag
	 */
	private static final String TAG = Narau.class.getSimpleName();
	private static Narau instance = null;

	/** Called when the activity is created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		Narau.instance = this;

		if (DownloaderActivity.isActive()) {
			this.showDownload(DownloaderActivity.LauncherSource.FIRST_RUN);
		} else {
			QuitThread.stopQuitThread();
			this.doInit();
		}
	}

	private void doInit() {
		final Context context = this.getApplicationContext();
		try {
			Common.init(context);

			if (CommonDB.haveDB(Database.dictonary.getValue()) || CommonDB.haveDB(Database.kanji.getValue())) {
				this.showMenuView();
			} else {
				this.showDownload(DownloaderActivity.LauncherSource.FIRST_RUN);
			}

		} catch (AppException ae) {
			// show the init error view
			super.setContentView(R.layout.menu_init_error);
			TextView tv = (TextView) super.findViewById(R.id.initErrorTextView);
			tv.setText(ae.getMessage());
		}
	}

	@Override
	public void onBackPressed() {
		Common.clean();
		super.onBackPressed();
	}

	/**
	 * 
	 * @param v
	 *            the menu_init_error.xml quit button (initErrorButtonQuit).
	 */
	public void quitApplication(View v) {
		QuitThread.startQuitThread();
		super.finish();
	}

	/**
	 * Show the menu view
	 */
	public void showMenuView() {

		final ArrayList<MenuActivity> listActivities = new ArrayList<MenuActivity>(5);

		final Intent intent = new Intent(Intent.ACTION_MAIN, null);
		intent.setPackage(getPackageName());
		intent.addCategory("org.vono.narau.MENU");

		final PackageManager pm = getPackageManager();
		final List<ResolveInfo> infos = pm.queryIntentActivities(intent, 0);
		final int count = infos.size();
		for (int i = 0; i < count; i++) {
			final ResolveInfo info = infos.get(i);

			final CharSequence labelSeq = info.loadLabel(pm);
			final String label = labelSeq != null ? labelSeq.toString() : info.activityInfo.name;

			final Intent target = new Intent();
			target.setClassName(info.activityInfo.applicationInfo.packageName, info.activityInfo.name);

			// Check if there is the database for this item
			boolean enabled = true;
			try {
				final Class<?> aClass = super.getClassLoader().loadClass(info.activityInfo.name);
				final Method m = aClass.getMethod("haveDatabase", new Class<?>[0]);
				enabled = (Boolean) m.invoke(null, new Object[0]);

			} catch (ClassNotFoundException nfee) {
				Log.e(TAG, nfee.getMessage(), nfee);
			} catch (SecurityException se) {
				Log.e(TAG, se.getMessage(), se);
			} catch (NoSuchMethodException nsme) {
				Log.e(TAG, info.activityInfo.name + ": " + nsme.getMessage(), nsme);
			} catch (IllegalArgumentException iae) {
				Log.e(TAG, iae.getMessage(), iae);
			} catch (IllegalAccessException iae) {
				Log.e(TAG, iae.getMessage(), iae);
			} catch (InvocationTargetException ite) {
				Log.e(TAG, ite.getMessage(), ite);
			}

			Drawable dIcon = null;
			if (0 != info.activityInfo.icon) {
				dIcon = super.getResources().getDrawable(info.activityInfo.icon);
			}

			MenuActivity ma = new MenuActivity(label, target, dIcon);
			ma.enabled = enabled;
			listActivities.add(ma);
		}

		MainMenuFragment.setList(listActivities, super.getLayoutInflater());
		super.setContentView(R.layout.narau);
		// super.setListAdapter(new MenuAdapter(listActivities));

		this.showMissingDatabaseNotification();
	}

	private void showMissingDatabaseNotification() {
		final boolean haveDict = CommonDB.haveDB(Database.dictonary);
		final boolean haveKanji = CommonDB.haveDB(Database.kanji);
		if (!haveDict || !haveKanji) {

			final String strMissing = super.getString(R.string.notificationMissingDB);
			final StringBuilder sb = new StringBuilder();
			if (!haveDict) {
				sb.append(String.format(strMissing, super.getString(R.string.dbDictionary)));
				sb.append("\n");
			}

			if (!haveKanji) {
				sb.append(String.format(strMissing, super.getString(R.string.dbKanjiKana)));
				sb.append("\n");
			}

			final Intent intent = new Intent(this, DownloaderActivity.class);
			intent.putExtra(DownloaderActivity.UPDATE_PROCESS, DownloaderActivity.LauncherSource.NOTIFICATION);

			final TaskStackBuilder taskStackBuilder = TaskStackBuilder.from(this);
			taskStackBuilder.addParentStack(DownloaderActivity.class);
			taskStackBuilder.addNextIntent(intent);

			final PendingIntent pendingIntent = taskStackBuilder.getPendingIntent(0, 0);
			final String title = super.getString(R.string.app_name);
			final String ticker = super.getString(R.string.notificationDBNeedUpdate);
			final NotificationCompat.Builder builder = new NotificationCompat.Builder(this);
			builder.setTicker(ticker);
			builder.setSmallIcon(R.drawable.ic_stat_notify_db_update);
			builder.setContentTitle(title);
			builder.setContentInfo(ticker);
			builder.setContentText(sb);
			builder.setAutoCancel(true);
			builder.setContentIntent(pendingIntent);

			NotificationManager nm = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
			nm.notify(R.id.download_db_notification, builder.getNotification());
		}
	}

	public static void finishOnUpdateDB() {
		if (null != Narau.instance) {
			Narau.instance.finish();
			Narau.instance = null;
		}
	}

	private void showDownload(DownloaderActivity.LauncherSource source) {
		Intent intent = new Intent(super.getApplicationContext(), DownloaderActivity.class);
		intent.putExtra(DownloaderActivity.UPDATE_PROCESS, source);
		super.startActivity(intent);
		super.finish();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.main, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		boolean downloading = false;
		if (!downloading) {

			switch (item.getItemId()) {
			case R.id.menuMainQuit:
				QuitThread.startQuitThread();
				super.finish();
				break;
			case R.id.menuMainLicense:
				Intent mIntent = new Intent(this, LicenseActivity.class);
				super.startActivity(mIntent);
				break;
			case R.id.menuMainUpdateDB:
				this.showDownload(DownloaderActivity.LauncherSource.UPDATE);
				break;
			case R.id.menuMainPreferences:
				mIntent = new Intent(this.getApplicationContext(), PreferencesActivity.class);
				super.startActivity(mIntent);
				break;
			default:
				return super.onOptionsItemSelected(item);
			}
		}
		return true;
	}

	/**
	 * An activity displayed in the menu.
	 */
	static class MenuActivity {
		final String name;
		final Intent intent;
		final Drawable icon;
		boolean enabled;

		MenuActivity(String name, Intent intent, Drawable drawable) {
			this.name = name;
			this.intent = intent;
			this.icon = drawable;
			if (null != drawable) {
				this.icon.setBounds(0, 0, this.icon.getIntrinsicWidth(), this.icon.getIntrinsicHeight());
			}
			this.enabled = true;
		}
	}

	/**
	 * The list adapter.
	 */
	static class MenuAdapter extends BaseAdapter {
		private final List<MenuActivity> listActivities;
		private final LayoutInflater layoutInflater;

		public MenuAdapter(List<MenuActivity> items, LayoutInflater layoutInflater) {
			this.listActivities = items;
			this.layoutInflater = layoutInflater;
		}

		public int getCount() {
			return this.listActivities.size();
		}

		public Object getItem(int position) {
			return this.listActivities.get(position);
		}

		public long getItemId(int position) {
			return position;
		}

		public View getView(int position, View convertView, ViewGroup parent) {
			View cView = convertView;
			if (null == cView) {
				cView = this.layoutInflater.inflate(R.layout.main_menu_item, parent, false);
				cView.setTag(cView.findViewById(android.R.id.text1));
			}
			final TextView tv = (TextView) cView.getTag();
			final MenuActivity item = listActivities.get(position);
			tv.setText(item.name);
			if (null != item.icon) {
				tv.setCompoundDrawables(item.icon, null, null, null);
			}
			tv.setEnabled(item.enabled);

			return cView;
		}

	}
}