/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau;

import android.util.Log;

/**
 * Since android do not quit the app not so often, this thread is running to
 * stop the application after WAIT_TIME of the main Activity receive onDestroy()
 * and if this activity do not receive one onCreate() in the mean time.
 */
public class QuitThread extends Thread {
	/**
	 * Auto exit after this number of seconds
	 */
	private static final int WAIT_TIME_SECONDS = 5;
	private static final int WAIT_TIME = WAIT_TIME_SECONDS * 1000;

	private static final String TAG = QuitThread.class.getSimpleName();

	private boolean haveToExit;
	private int exitTime;

	private static QuitThread quitThread;

	/**
	 * Exit application after 1 minute.
	 */
	public static void startQuitThread() {
		QuitThread.quitThread = new QuitThread();
		QuitThread.quitThread.start();
	}

	/**
	 * 
	 * @param time
	 *            exit application after 'time' miliseconds
	 */
	public static void startQuitThread(int time) {
		QuitThread.quitThread = new QuitThread(time);
		QuitThread.quitThread.start();
	}

	/**
	 * Tell the this thread to do nothing.
	 */
	public static void stopQuitThread() {
		if (null != QuitThread.quitThread) {
			QuitThread.quitThread.finallyNotNow();
		}
	}

	/**
	 */
	private QuitThread() {
		this(WAIT_TIME);
	}

	/**
	 * @param main
	 *            The menu activity
	 */
	private QuitThread(int time) {
		super(TAG);
		this.haveToExit = true;
		this.exitTime = time;
	}

	@Override
	public void run() {
		Common.clean();

		synchronized (this) {
			try {
				this.wait(exitTime);
			} catch (InterruptedException ie) {
			}

			if (this.haveToExit) {
				Log.i(TAG, "Stop application after " + (exitTime / 1000) + "s of inactivity");
				System.exit(0);
			}
		}
	}

	private synchronized void finallyNotNow() {

		this.haveToExit = false;
		if (this.isAlive()) {
			this.notify();
		}
	}

}
