/*
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.dictionary;

import org.vono.narau.Common;
import org.vono.narau.Common.KanaType;
import org.vono.narau.R;
import org.vono.narau.db.CommonDB;
import org.vono.narau.db.CommonDB.Database;
import org.vono.narau.models.dictionary.ItemDisplay;
import org.vono.narau.models.dictionary.ItemDisplay.Item;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;

public class DictionaryActivity extends Activity {
	// private static final String TAG =
	// DictionaryActivity.class.getSimpleName();
	public static final String KEY_MESSAGE = "INFO";
	public static final String KEY_RESULT_AVAIL = "RA";

	private static final String KEY_TXT_SEARCH = "search";
	private static final String KEY_BUTTON_CONF = "conv";

	public static final char A_ROMANJI = 'a';
	public static final char A_HIRAGANA = 'あ';
	public static final char A_KATAKANA = 'ア';

	private static final String[][] TEXT_REPLACE = { { "'", "''" }, { ";", "" }, { "\"", "" }, { "/*", "" },
			{ "*/", "" }, { "--", "" } };

	private static ItemDisplay itemDisplay = null;

	private LinearLayout linearLayoutContent;
	private LinearLayout linearLayoutInput;
	private LinearLayout linearLayoutZoomControls;
	private Button buttonConvRomanji;
	private Button buttonSearch;
	private EditText editTextInput;
	private DictionaryHandler handler;
	private DictionaryThread thread;

	public DictionaryActivity() {
		this.handler = new DictionaryHandler();
		this.thread = null;
	}

	public static boolean haveDatabase() {
		// Method used by Narau.showMissingDatabaseNotification()
		return CommonDB.haveDB(Database.dictonary);
	}

	/**
	 * Called when the activity is first created.
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		if (null == DictionaryActivity.itemDisplay) {
			DictionaryActivity.itemDisplay = new ItemDisplay(this);
		}

		super.setContentView(R.layout.dictionary);
		this.linearLayoutContent = (LinearLayout) super.findViewById(R.id.dictionaryLinearLayoutContent);
		this.linearLayoutInput = (LinearLayout) super.findViewById(R.id.dictionaryLinearLayoutInput);
		this.linearLayoutZoomControls = (LinearLayout) super.findViewById(R.id.dictionaryLinearLayoutZoomControls);
		this.buttonConvRomanji = (Button) super.findViewById(R.id.dictionaryButtonConvRomanji);
		this.buttonSearch = (Button) super.findViewById(R.id.dictionaryButtonSearch);
		this.editTextInput = (EditText) super.findViewById(R.id.dictionaryEditTextInput);

		if (null == savedInstanceState) {
			this.onClickReset(null);
		}
	}

	@Override
	public void onBackPressed() {

		if (null != this.thread) {
			this.thread.requestStop();
			while (this.thread.isAlive()) {
				try {
					Thread.sleep(50);
				} catch (InterruptedException ie) {
				}
			}
			this.thread = null;
		}
		this.handler = null;

		this.linearLayoutContent.removeAllViews();
		DictionaryActivity.itemDisplay.clear();
		DictionaryActivity.itemDisplay = null;

		this.linearLayoutContent = null;
		this.linearLayoutInput = null;
		this.linearLayoutZoomControls = null;
		this.buttonConvRomanji = null;
		this.buttonSearch = null;
		this.editTextInput = null;

		System.runFinalization();
		System.gc();

		super.onBackPressed();
	}

	@Override
	protected void onPause() {
		super.onPause();

		// Need to remove items views before add them again
		// when calling ItemDisplay.setActivity()
		if (null != this.linearLayoutContent) {
			// check null, because onBackPressed() can be called before
			this.linearLayoutContent.removeAllViews();
		}
	}

	@Override
	protected void onResume() {
		super.onResume();

		// Restore the view:
		// - first activity start 
		// - device rotation
		// - EntryActivity finish
		this.linearLayoutContent.removeAllViews();
		this.linearLayoutContent.addView(this.linearLayoutInput);
		this.linearLayoutContent.addView(this.buttonSearch);

		DictionaryActivity.itemDisplay.setActivity(this, this.linearLayoutContent);
	}

	@Override
	protected void onSaveInstanceState(Bundle outState) {
		super.onSaveInstanceState(outState);
		outState.putChar(KEY_BUTTON_CONF, this.buttonConvRomanji.getText().charAt(0));
		outState.putCharSequence(KEY_TXT_SEARCH, this.editTextInput.getText());
	}

	@Override
	protected void onRestoreInstanceState(Bundle savedInstanceState) {
		super.onRestoreInstanceState(savedInstanceState);

		this.buttonConvRomanji.setText(Character.valueOf(savedInstanceState.getChar(KEY_BUTTON_CONF)).toString());
		this.editTextInput.setText(savedInstanceState.getCharSequence(KEY_TXT_SEARCH));
	}

	/**
	 * Click on the search button
	 * 
	 * @param button
	 *            The search button
	 */
	public void onClickSearch(View button) {

		final String searchText = DictionaryActivity.secureTextMatch(this.editTextInput.getText().toString());

		if (!searchText.isEmpty()) {
			this.linearLayoutContent.removeAllViews();
			this.linearLayoutContent.addView(this.linearLayoutInput);
			this.linearLayoutContent.addView(this.buttonSearch);

			this.buttonSearch.setEnabled(false);
			this.buttonConvRomanji.setEnabled(false);
			this.editTextInput.setEnabled(false);

			DictionaryActivity.itemDisplay.clearList();

			if (null != this.thread && this.thread.isAlive()) {
				this.thread.requestStop();
			}
			this.thread = new DictionaryThread(this.handler, searchText);
			this.thread.start();
		}
	}

	/**
	 * Click on the reset button
	 * 
	 * @param button
	 *            The reset button
	 */
	public void onClickReset(View button) {
		this.editTextInput.setText(Common.EMPTY_STRING);
		this.buttonConvRomanji.setText(Character.valueOf(A_HIRAGANA).toString());
		this.buttonSearch.setEnabled(true);
		this.buttonConvRomanji.setEnabled(true);
		this.editTextInput.setEnabled(true);
		this.linearLayoutContent.removeAllViews();
		this.linearLayoutContent.addView(this.linearLayoutInput);
		this.linearLayoutContent.addView(this.buttonSearch);

		DictionaryActivity.itemDisplay.clearList();

		if (null != this.thread && this.thread.isAlive()) {
			this.thread.requestStop();
			this.thread = null;
		}
	}

	/**
	 * Characters conversion: romanji->hiragana->katakana
	 * 
	 * @param button
	 */
	public void onClickConvCharacters(View button) {
		char currentTextFormat = this.buttonConvRomanji.getText().charAt(0);

		final String searchText = this.editTextInput.getText().toString();
		String newSearchText = searchText;

		switch (currentTextFormat) {
		case A_HIRAGANA:
			// romanji -> hiragana
			newSearchText = Common.convertKana(searchText, KanaType.ROMANJI, KanaType.HIRAGANA);
			this.buttonConvRomanji.setText(Character.valueOf(A_KATAKANA).toString());
			break;
		case A_KATAKANA:
			// hiragana -> katakana
			newSearchText = Common.convertKana(searchText, KanaType.HIRAGANA, KanaType.KATAKANA);
			this.buttonConvRomanji.setText(Character.valueOf(A_ROMANJI).toString());
			break;
		case A_ROMANJI:
			// katakana -> romanji
			newSearchText = Common.convertKana(searchText, KanaType.KATAKANA, KanaType.ROMANJI);
			this.buttonConvRomanji.setText(Character.valueOf(A_HIRAGANA).toString());
			break;
		}

		this.editTextInput.setText(newSearchText);
	}

	/**
	 * Change the text size.
	 * 
	 * @param buttonZoom
	 */
	@SuppressWarnings("static-method")
	public void onClickZoom(final View buttonZoom) {
		if (R.id.dictionaryZoomButtonOut == buttonZoom.getId()) {
			DictionaryActivity.itemDisplay.zoomOut();
		} else {
			DictionaryActivity.itemDisplay.zoomIn();
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.dictionary, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.menuDictionaryShowControls:
			if (View.VISIBLE == this.linearLayoutZoomControls.getVisibility()) {
				this.linearLayoutZoomControls.setVisibility(View.GONE);
			} else {
				this.linearLayoutZoomControls.setVisibility(View.VISIBLE);
			}
			break;
		}

		return true;
	}

	/**
	 * Prevent something like an SQL injection.
	 * 
	 * @param text
	 *            The text to put in the LIKE clause
	 * @return the secured text
	 */
	private static String secureTextMatch(String text) {
		int repArraySize = DictionaryActivity.TEXT_REPLACE.length;
		int txtLen = text.length();
		StringBuilder result = new StringBuilder(txtLen);

		for (int i = 0; i < txtLen; i++) {
			int j = 0;
			for (; j < repArraySize; j++) {
				String txtSearch = DictionaryActivity.TEXT_REPLACE[j][0];
				int txtSearchLen = txtSearch.length();
				// regionMatches -> startWith
				if (text.regionMatches(i, txtSearch, 0, txtSearchLen)) {
					result.append(DictionaryActivity.TEXT_REPLACE[j][1]);
					i += txtSearchLen - 1;
					break;
				}
			}
			if (j == repArraySize) {
				// no replacement found, add the original text character
				result.append(text.charAt(i));
			}

		}
		String secText = result.toString();

		return secText;
	}

	/**
	 * Glue to update the UI by the DictionaryThread.
	 */
	class DictionaryHandler extends Handler {
		static final String MSG_ADD_ITEM = "item";
		static final String MSG_END = "end";

		public void addResult(Item item) {
			Message message = Message.obtain(this);
			Bundle bundle = message.getData();
			bundle.clear();

			bundle.putSerializable(MSG_ADD_ITEM, item);
			super.sendMessage(message);
		}

		public void searchEnd(String text) {
			Message message = Message.obtain(this);
			Bundle bundle = message.getData();
			bundle.clear();
			bundle.putString(MSG_END, text);
			super.sendMessage(message);
		}

		@Override
		public void handleMessage(Message msg) {
			final Bundle data = msg.getData();
			if (data.containsKey(MSG_ADD_ITEM)) {
				Item item = (Item) msg.getData().getSerializable(MSG_ADD_ITEM);
				DictionaryActivity.itemDisplay.addItem(DictionaryActivity.this.linearLayoutContent, item);
			} else if (data.containsKey(MSG_END)) {
				DictionaryActivity.this.buttonSearch.setEnabled(true);
				DictionaryActivity.this.buttonConvRomanji.setEnabled(true);
				DictionaryActivity.this.editTextInput.setEnabled(true);

				DictionaryActivity.itemDisplay.showToast(data.getString(MSG_END));
			}
		}
	}

}
