/*
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.dictionary;

import java.util.ArrayList;

import org.vono.narau.Common;
import org.vono.narau.Common.KanaType;
import org.vono.narau.db.CommonDB.Database;
import org.vono.narau.db.DictionaryDB.DictionaryIterator;
import org.vono.narau.dictionary.DictionaryActivity.DictionaryHandler;
import org.vono.narau.models.dictionary.Entry;
import org.vono.narau.models.dictionary.ItemDisplay.Item;
import org.vono.narau.models.kanji.KanjiData;
import org.vono.narau.preferences.Preferences;

public class DictionaryThread extends Thread {
	@SuppressWarnings("unused")
	private static final String TAG = DictionaryThread.class.getSimpleName();

	private String searchText;
	private DictionaryHandler handler;
	private boolean requestStop;

	DictionaryThread(DictionaryHandler handler, String text) {
		this.handler = handler;
		this.searchText = text;

	}

	@Override
	public void run() {
		// Search type of text (Romanji, Kana, Kanji)
		KanaType kanaType = KanaType.ROMANJI;
		final int textLength = this.searchText.length();
		for (int i = 0; i < textLength; i++) {
			int c = this.searchText.codePointAt(i);
			if (Character.charCount(c) == 2)
				i++;

			if (KanjiData.isKana(c)) {
				kanaType = KanaType.HIRAGANA;
			} else if (KanjiData.isKanji(c)) {
				kanaType = KanaType.KANJI;
				i = textLength;
			}
		}

		final DictionaryIterator iterator = new DictionaryIterator();

		boolean haveResult = false;
		if (KanaType.ROMANJI == kanaType) {
			ArrayList<String> listLangs = Preferences.getLangsPriorities(Database.dictonary);
			for (String lLang : listLangs) {
				haveResult = iterator.searchToJpn(this.searchText, lLang);
				if (haveResult) {
					break;
				}
			}
		} else if (KanaType.HIRAGANA == kanaType) {
			haveResult = iterator.searchFromJpnKana(this.searchText);
		} else {
			haveResult = iterator.searchFromJpnKanji(this.searchText);
		}

		if (haveResult) {
			while (iterator.hasNext()) {

				Entry entry = iterator.next();
				String kanjiElem = Common.EMPTY_STRING;
				if (entry.k_ele != null && !entry.k_ele.isEmpty()) {
					kanjiElem = entry.k_ele.get(0).keb;
				}
				Item item = new Item(entry.sequence, kanjiElem, entry.r_ele.get(0).reb, entry.senses.get(0)
						.getFirstGloss());
				this.handler.addResult(item);

				synchronized (this) {
					if (this.requestStop) {
						iterator.close();
					}
				}
			}
		}

		iterator.close();

		// show the toast message, only if there is no user interruption.
		this.handler.searchEnd(this.requestStop ? null : this.searchText);
	}

	public synchronized void requestStop() {
		this.requestStop = true;
	}

}
