/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.exercise;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Random;

import org.vono.narau.Common;
import org.vono.narau.R;
import org.vono.narau.db.CommonDB.Database;
import org.vono.narau.db.KanjiDB;
import org.vono.narau.kanji.KanjiArrayAdapter;
import org.vono.narau.models.kanji.Kanji;
import org.vono.narau.models.kanji.KanjiData;
import org.vono.narau.preferences.Preferences;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.graphics.Typeface;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.TableLayout;
import android.widget.TableRow;
import android.widget.TableRow.LayoutParams;
import android.widget.TextView;

/**
 * Drill: Display a kanji/kana, and ask the user to find the right answer
 * between 6 possibilities.
 * 
 */
public abstract class DrillCharActivity extends Activity implements OnClickListener {
	private static final String TAG = DrillCharActivity.class.getSimpleName();

	private static final String ST_TT = "testType";
	private static final String ST_TG = "textViewGood";
	private static final String ST_TB = "textViewBad";
	private static final String ST_BT = "buttonsTexts";
	private static final String ST_TS = "testState";

	static final int TYPE_MASK = KanjiData.KANA_CHAR | KanjiData.KANA_STRING | KanjiData.KANJI_JLPT
			| KanjiData.KANJI_GRADE | KanjiData.KANJI_NAME | KanjiData.KANJI_OTHER;

	private enum TestState {
		ERROR, NEW, IN_PROGRESS, DONE
	}

	private enum TestType {
		UNKNOWN, KANA_ROMANJI, KANJI_ROMANJI, ROMANJI_KANA, ROMANJI_KANJI
	}

	private static final int NUM_ANSWERS = 6;

	private TextView textViewQuestion;
	private TextView textViewGood;
	private TextView textViewBad;
	private ArrayList<Button> listButtons;

	private Random randomKanji;
	private TestType testType;

	private static ProgressHandler progressHandler = null;
	private static ThreadLoadKanji threadLoadKanji = null;

	private TestState testState;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		Bundle extra = this.getIntent().getExtras();
		if (null == extra) {
			throw new RuntimeException("don't know what to display");
		}
		this.testType = TestType.UNKNOWN;
		this.randomKanji = new Random();
		if (null == DrillCharActivity.progressHandler) {
			DrillCharActivity.progressHandler = new ProgressHandler();
		}

		testState = TestState.ERROR;
		if (savedInstanceState == null) {
			// New start
			if (DrillCharActivity.threadLoadKanji != null) {
				DrillCharActivity.threadLoadKanji.prematureEnd();
				// Wait for the loading thread to end
				while (DrillCharActivity.threadLoadKanji != null) {
					try {
						// Log.d(TAG, "wait older loading thread to end");
						Thread.sleep(100);
					} catch (InterruptedException e) {
					}
				}
			}
			DrillCharActivity.progressHandler.clear();

			final Serializable sIC = extra.getSerializable(KanjiArrayAdapter.ITEM_CATEGORY);
			final String direction = extra.getString(DrillMenuActivity.DRILL_DIRECTION);
			if (initDrillQA(sIC, direction)) {
				testState = TestState.NEW;
			}
		} else {
			// Pass here when there is for example a screen rotation
			// so it just the current drill which continue.
			this.testState = TestState.valueOf(savedInstanceState.getString(ST_TS));
			this.testType = TestType.valueOf(savedInstanceState.getString(ST_TT));

			if (testState == TestState.IN_PROGRESS && DrillCharActivity.threadLoadKanji != null
					&& DrillCharActivity.progressHandler.activity != null) {
				DrillCharActivity.progressHandler.setActivity(this);
				DrillCharActivity.progressHandler.progressDialog = ProgressDialog.show(this, Common.EMPTY_STRING,
						super.getString(R.string.loading), true);
			}
		}

		switch (testState) {
		case DONE:
			this.initUIResult();
			break;
		case ERROR:
			Log.e(TAG, "No question to display");
			this.finish();
			break;
		case NEW:
			this.testState = TestState.IN_PROGRESS;
			//$FALL-THROUGH$
		case IN_PROGRESS:
			this.initUI(savedInstanceState);
			break;

		}
	}

	/**
	 * Load drill datas
	 * 
	 * @param itemId
	 *            The type of drill
	 * @return true if there is data to display
	 */
	private boolean initDrillQA(Serializable s, String direction) {
		KanjiData.WritingCategory wc = (KanjiData.WritingCategory) s;
		boolean haveData = false;

		final ArrayList<KanjiData.CharacterCatalog> alCC = wc.catalogList;
		final int numCC = alCC.size();
		final Common.KanaType charType = wc.type;
		switch (charType) {
		case HIRAGANA:
		case KATAKANA:
			if (DrillMenuActivity.DRILL_DIRECTION_FROM.equals(direction)) {
				this.testType = TestType.KANA_ROMANJI;
			} else {
				this.testType = TestType.ROMANJI_KANA;
			}

			break;
		case KANJI:
			if (DrillMenuActivity.DRILL_DIRECTION_FROM.equals(direction)) {
				this.testType = TestType.KANJI_ROMANJI;
			} else {
				this.testType = TestType.ROMANJI_KANJI;
			}
			break;
		case ROMANJI:
			String msg = "No drill for " + charType.toString();
			Log.e(TAG, msg);
			throw new RuntimeException(msg);
		}
		ArrayList<DrillQA> drillQA = DrillCharActivity.progressHandler.drillQA;
		StringBuilder sbSelection = new StringBuilder();
		ArrayList<String> selectionArgsList = new ArrayList<String>(numCC);
		for (int i = 0; i < numCC; i++) {
			final KanjiData.CharacterCatalog cc = wc.catalogList.get(i);
			if (cc.isSelected() && (cc.typeMask & TYPE_MASK) == cc.typeMask) {
				haveData = true;
				if (this.testType == TestType.KANA_ROMANJI || this.testType == TestType.ROMANJI_KANA) {
					final String[] charCatalog = cc.charCatalog;
					final int max = charCatalog.length;
					for (int j = 0; j < max; j++) {
						DrillQA dqa = new DrillQA();
						if (this.testType == TestType.KANA_ROMANJI) {
							dqa.question = charCatalog[j];
							dqa.answer = Common.convertKana(dqa.question, charType, Common.KanaType.ROMANJI);
						} else {
							dqa.answer = charCatalog[j];
							dqa.question = Common.convertKana(dqa.answer, charType, Common.KanaType.ROMANJI);
						}
						drillQA.add(dqa);
					}
				} else {
					if (sbSelection.length() > 0) {
						sbSelection.append(" or ");
					}
					sbSelection.append(cc.selection);
					selectionArgsList.addAll(cc.selectionArgs);
				}
			}
		}
		if (this.testType == TestType.KANJI_ROMANJI || this.testType == TestType.ROMANJI_KANJI) {
			// If kanji drill is selected,
			// we need to fetch data from the SQLite DB.
			// This is done in another thread, because it take some times
			// to be fully loaded

			// This thread is started after the ui is set
			// Because it need the progressBar instance to be set.
			final String selection = sbSelection.toString();
			final String[] selectionArgs = selectionArgsList.toArray(new String[0]);
			DrillCharActivity.threadLoadKanji = new ThreadLoadKanji(selection, selectionArgs, this.testType);
		} else {
			// All data already loaded:
			int count = DrillCharActivity.progressHandler.drillQA.size();
			DrillCharActivity.progressHandler.numKanji = count;
			DrillCharActivity.progressHandler.numLoadedKanji = count;
		}
		return haveData;
	}

	/**
	 * UI setup.
	 * 
	 * @param savedInstanceState
	 */
	private void initUI(Bundle savedInstanceState) {
		this.listButtons = new ArrayList<Button>(NUM_ANSWERS);
		if (testType == TestType.KANA_ROMANJI) {
			super.setContentView(R.layout.drill_char_kana);
		} else if (testType == TestType.KANJI_ROMANJI) {
			super.setContentView(R.layout.drill_char_kanji);
		} else {
			super.setContentView(R.layout.drill_char_romaji);
		}

		ProgressBar progressBar = (ProgressBar) super.findViewById(R.id.drillProgressBarLoadKanji);
		progressHandler.setProgressBar(progressBar);

		Button button = (Button) super.findViewById(R.id.drillButtonAnswer1);
		button.setOnClickListener(this);
		this.listButtons.add(button);
		button = (Button) super.findViewById(R.id.drillButtonAnswer2);
		button.setOnClickListener(this);
		this.listButtons.add(button);
		button = (Button) super.findViewById(R.id.drillButtonAnswer3);
		button.setOnClickListener(this);
		this.listButtons.add(button);
		button = (Button) super.findViewById(R.id.drillButtonAnswer4);
		button.setOnClickListener(this);
		this.listButtons.add(button);
		button = (Button) super.findViewById(R.id.drillButtonAnswer5);
		button.setOnClickListener(this);
		this.listButtons.add(button);
		button = (Button) super.findViewById(R.id.drillButtonAnswer6);
		button.setOnClickListener(this);
		this.listButtons.add(button);

		this.textViewQuestion = (TextView) super.findViewById(R.id.drillCharText);
		this.textViewGood = (TextView) super.findViewById(R.id.drillTextGoodAnswer);
		this.textViewBad = (TextView) super.findViewById(R.id.drillTextBadAnswer);

		if (testType == TestType.ROMANJI_KANA) {
			this.textViewQuestion.setTextAppearance(super.getApplicationContext(), android.R.attr.textAppearanceLarge);
			this.textViewQuestion.setTextSize(30.0f);
		}

		if (DrillCharActivity.threadLoadKanji != null && DrillCharActivity.threadLoadKanji.isAlive()
				&& DrillCharActivity.progressHandler.progressDialog != null) {
			progressBar.setVisibility(View.VISIBLE);
		} else if (savedInstanceState != null) {

			CharSequence textGood = savedInstanceState.getCharSequence(ST_TG);
			CharSequence textBad = savedInstanceState.getCharSequence(ST_TB);
			ArrayList<CharSequence> buttonsText = savedInstanceState.getCharSequenceArrayList(ST_BT);

			this.textViewGood.setText(textGood);
			this.textViewBad.setVisibility(View.VISIBLE);
			this.textViewBad.setText(textBad);
			this.textViewQuestion.setText(DrillCharActivity.progressHandler.currentTest.question);

			for (int i = 0; i < NUM_ANSWERS; i++) {
				button = this.listButtons.get(i);
				button.setText(buttonsText.get(i));
				button.setTag(DrillCharActivity.progressHandler.currentTest);
			}

			if (DrillCharActivity.threadLoadKanji != null) {
				progressBar.setVisibility(View.VISIBLE);
			}
			String title = super.getResources().getQuantityString(R.plurals.drillProgress,
					DrillCharActivity.progressHandler.numBadResponse,
					(DrillCharActivity.progressHandler.numTestDone + 1),
					DrillCharActivity.progressHandler.numLoadedKanji, DrillCharActivity.progressHandler.numBadResponse);
			super.setTitle(title);
		} else {
			if (null != DrillCharActivity.threadLoadKanji) {
				DrillCharActivity.progressHandler.setActivity(this);
				DrillCharActivity.progressHandler.progressDialog = ProgressDialog.show(this, Common.EMPTY_STRING,
						super.getString(R.string.loading), true);
				DrillCharActivity.threadLoadKanji.start();
			} else {
				this.nextQuestion();
			}
		}
	}

	/**
	 * Show this drill test results
	 */
	private void initUIResult() {
		super.setContentView(R.layout.drill_char_kana_result);
		super.setTitle(R.string.drillFromJap);

		TextView textViewResult = (TextView) super.findViewById(R.id.drillCharKanaResTextViewSummary);
		String text = super.getResources().getQuantityString(R.plurals.drillProgress,
				DrillCharActivity.progressHandler.numBadResponse, (DrillCharActivity.progressHandler.numTestDone + 1),
				DrillCharActivity.progressHandler.numLoadedKanji, DrillCharActivity.progressHandler.numBadResponse);
		textViewResult.setText(text);

		final TableLayout tableLayout = (TableLayout) super.findViewById(R.id.drillCharKanaResTable);
		final int max = DrillCharActivity.progressHandler.numKanji;
		final Context context = super.getApplicationContext();
		final ArrayList<DrillQA> drillQA = DrillCharActivity.progressHandler.drillQA;
		final TableRow.LayoutParams lpTv = new TableRow.LayoutParams(LayoutParams.MATCH_PARENT,
				LayoutParams.WRAP_CONTENT, 1 / 3.0f);

		lpTv.setMargins(0, 3, 0, 3);

		final int lightGreen = super.getResources().getColor(R.color.light_green);
		final int lightRed = super.getResources().getColor(R.color.light_red);
		final int white = super.getResources().getColor(android.R.color.white);
		int textSize = 0;
		switch (context.getResources().getDisplayMetrics().densityDpi) {
		case DisplayMetrics.DENSITY_LOW:
			textSize = android.R.style.TextAppearance_Small;
			break;
		case DisplayMetrics.DENSITY_MEDIUM:
			textSize = android.R.style.TextAppearance_Medium;
			break;
		case DisplayMetrics.DENSITY_HIGH:
		case DisplayMetrics.DENSITY_XHIGH:
			textSize = android.R.style.TextAppearance_Large;
			break;
		default:
			textSize = android.R.style.TextAppearance_Medium;
		}

		final int numPerLine = (this.testType == TestType.KANJI_ROMANJI || this.testType == TestType.ROMANJI_KANJI) ? 1
				: 5;
		for (int i = 0; i < max;) {
			TableRow row = new TableRow(context);
			final int lmax = i + numPerLine > max ? max : i + numPerLine;
			for (; i < lmax; i++) {
				row.addView(DrillCharActivity.getTVResult(context, drillQA, i, lightGreen, lightRed, white, textSize),
						lpTv);
			}

			tableLayout.addView(row);
		}

	}

	private static TextView getTVResult(Context context, ArrayList<DrillQA> drillQA, int i, int lightGreen,
			int lightRed, int white, int textSize) {
		final TextView tvRow = new TextView(context);
		final DrillQA dqa = drillQA.get(i);
		String text = dqa.question + ": " + dqa.answer;
		if (dqa.userAnswer != null) {
			text += "\n" + dqa.userAnswer;
			tvRow.setBackgroundColor(lightRed);
		} else {
			tvRow.setBackgroundColor(lightGreen);
		}
		tvRow.setText(text);
		tvRow.setGravity(Gravity.CENTER_HORIZONTAL);
		tvRow.setTypeface(null, Typeface.BOLD);
		tvRow.setTextColor(white);
		tvRow.setTextAppearance(context, textSize);
		return tvRow;
	}

	/**
	 * Handle a click on a response button.
	 */
	public void onClick(View v) {

		DrillQA dqa = (DrillQA) v.getTag();// this.drillQA.get(this.done[currentTestCount]);
		Button button = (Button) v;
		if (dqa.answer != button.getText().toString()) {
			// Bad response
			textViewBad.setText(dqa.question + " ≠ " + button.getText().toString());
			this.textViewBad.setVisibility(View.VISIBLE);
			dqa.userAnswer = button.getText().toString();
			DrillCharActivity.progressHandler.numBadResponse++;
		} else {
			this.textViewBad.setText("");
			this.textViewBad.setVisibility(View.GONE);
			dqa.userAnswer = null;
		}
		this.textViewGood.setText(dqa.question + " = " + dqa.answer);

		DrillCharActivity.progressHandler.numTestDone++;
		if (DrillCharActivity.progressHandler.numTestDone < DrillCharActivity.progressHandler.numKanji) {
			this.nextQuestion();
		} else {
			DrillCharActivity.progressHandler.numTestDone--;
			this.testState = TestState.DONE;
			this.initUIResult();
		}
	}

	/**
	 * Display the next drill question.
	 */
	private void nextQuestion() {

		int kanjisCount = DrillCharActivity.progressHandler.numLoadedKanji;
		ArrayList<DrillQA> drillQA = DrillCharActivity.progressHandler.drillQA;
		int testId = 0;

		boolean alreadySee = true;
		// Don't display a question we already have display
		DrillQA dqa = null;
		while (alreadySee) {

			testId = this.randomKanji.nextInt(kanjisCount);
			dqa = drillQA.get(testId);
			if (!dqa.see) {
				alreadySee = false;
				dqa.see = true;
			}
		}
		DrillCharActivity.progressHandler.currentTest = dqa;
		this.textViewQuestion.setText(dqa.question);

		Random random = new Random();
		int goodButtonIdx = random.nextInt(6);
		Button button = this.listButtons.get(goodButtonIdx);
		button.setText(dqa.answer);
		button.setTag(dqa);
		ArrayList<Integer> usedVal = new ArrayList<Integer>(NUM_ANSWERS);
		usedVal.add(testId);

		boolean haveEnoughQuestions = (kanjisCount > NUM_ANSWERS);
		for (int i = 0; i < 6; i++) {
			if (i != goodButtonIdx) {
				DrillQA dqaError = null;
				int rand = 0;
				do {
					rand = random.nextInt(kanjisCount - 1);
				} while (usedVal.contains(rand) && haveEnoughQuestions);
				usedVal.add(rand);

				dqaError = drillQA.get(rand);
				button = this.listButtons.get(i);
				button.setText(dqaError.answer);
				button.setTag(dqa);
			}
		}

		String title = super.getResources().getQuantityString(R.plurals.drillProgress,
				DrillCharActivity.progressHandler.numBadResponse, (DrillCharActivity.progressHandler.numTestDone + 1),
				DrillCharActivity.progressHandler.numLoadedKanji, DrillCharActivity.progressHandler.numBadResponse);
		super.setTitle(title);
	}

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		// If the user want to stop this drill
		// using the back key, and the kanjis are still loading
		// stop the thread
		if (KeyEvent.KEYCODE_BACK == keyCode) {
			if (DrillCharActivity.threadLoadKanji != null) {
				DrillCharActivity.threadLoadKanji.prematureEnd();
				// wait for the thread to end.
				while (DrillCharActivity.threadLoadKanji != null) {
					try {
						Thread.sleep(100);
					} catch (InterruptedException ie) {
					}
				}
			}
			DrillCharActivity.progressHandler.clear();
			DrillCharActivity.progressHandler = null;
			System.runFinalization();
			System.gc();
		}
		return super.onKeyDown(keyCode, event);
	}

	@Override
	public void onStop() {
		if (DrillCharActivity.progressHandler != null && DrillCharActivity.progressHandler.activity != null) {
			DrillCharActivity.progressHandler.progressDialog.dismiss();
			DrillCharActivity.progressHandler.progressDialog = null;
			// Keep the progressDialog instance to know we need to redisplay it
			// in the onCreate() method
		}
		super.onStop();
	}

	/**
	 * <p>
	 * Handle device rotation.
	 * </p>
	 * {@inheritDoc}
	 * 
	 */
	@Override
	public void onSaveInstanceState(Bundle savedInstanceState) {
		super.onSaveInstanceState(savedInstanceState);
		savedInstanceState.putString(ST_TS, this.testState.name());
		savedInstanceState.putString(ST_TT, this.testType.name());
		if (testState != TestState.DONE) {
			savedInstanceState.putCharSequence(ST_TG, this.textViewGood.getText());
			savedInstanceState.putCharSequence(ST_TB, this.textViewBad.getText());
			ArrayList<CharSequence> buttonsText = new ArrayList<CharSequence>(NUM_ANSWERS);
			for (int i = 0; i < NUM_ANSWERS; i++) {
				buttonsText.add(listButtons.get(i).getText());
			}
			savedInstanceState.putCharSequenceArrayList(ST_BT, buttonsText);
		}
	}

	/**
	 * 
	 * The thread which load the kanji informations in the background.
	 * 
	 */
	private static class ThreadLoadKanji extends Thread {
		private String selection;
		private String[] selectionArgs;
		private boolean requestStop;
		private final TestType testType;

		public ThreadLoadKanji(String selection, String[] selectionArgs, TestType testType) {
			this.selection = selection;
			this.selectionArgs = selectionArgs;
			this.requestStop = false;
			this.setName("Drill-ThreadLoadKanji");
			this.testType = testType;
		}

		@Override
		public void run() {

			if (DrillCharActivity.threadLoadKanji != this) {
				// Should not happen
				Log.e(TAG, "Another loading thread is running");
				return;
			}
			// final Date start = new Date();
			final TestType _testType = this.testType;

			DrillCharActivity.threadLoadKanji = this;

			KanjiDB.KanjiDBIterator kanjiIterator = KanjiDB.getKanjiIterator(selection, selectionArgs, true, false,
					false);
			kanjiIterator.first();
			final int num = kanjiIterator.getCount();

			// show the progress bar
			DrillCharActivity.progressHandler.setDrillCount(num);

			try {
				for (int i = 0; i < num; i++) {
					final Kanji kanji = kanjiIterator.get(i);

					String meaning = null;
					if (null != kanji.meanings) {
						// TODO: duplicate code in LearnWritingActivity
						ArrayList<String> langs = Preferences.getLangsPriorities(Database.kanji);

						for (String lang : langs) {
							meaning = kanji.meanings.get(lang);
							if (null != meaning) {
								break;
							}
						}

					}
					if (null == meaning) {
						DrillCharActivity.progressHandler.recount();
						continue;
					}

					final String nihongo = new String(Character.toChars(kanji.codepoint));

					synchronized (TAG) {
						if (this.requestStop) {
							i = num;
						} else if (DrillCharActivity.progressHandler != null) {
							// Log.d(TAG, "addKanji " + question);
							if (_testType == TestType.KANJI_ROMANJI) {
								DrillCharActivity.progressHandler.addKanji(nihongo, meaning);
							} else {
								DrillCharActivity.progressHandler.addKanji(meaning, nihongo);
							}
						}
					}
				}
			} finally {
				kanjiIterator.close();

				// hide the progress bar
				synchronized (TAG) {
					if (null != DrillCharActivity.progressHandler && this.requestStop == false) {
						DrillCharActivity.progressHandler.loadFinish();
					}
					DrillCharActivity.threadLoadKanji = null;
				}
			}
			// final Date end = new Date();
			// final long loadingTime = (end.getTime() - start.getTime()) /
			// 1000;
			// Log.i(TAG, "Load kanji time: " + loadingTime + " seconds");
		}

		public void prematureEnd() {
			synchronized (TAG) {
				this.requestStop = true;
			}
		}

	}

	/**
	 * Used to retrieve data from the loader thread.
	 * 
	 */
	private static class ProgressHandler extends Handler {
		private final static String MSG_DRILL_COUNT = "dc";
		private final static String MSG_QUESTION = "q";
		private final static String MSG_ANSWER = "a";
		private final static String MSG_RECOUNT = "r";
		private final static String MSG_DONE = "d";

		private ProgressDialog progressDialog;
		private ProgressBar progressBar;
		private ArrayList<DrillQA> drillQA;
		private int numKanji;
		private int numLoadedKanji;
		private int numTestDone;
		private int numBadResponse;
		private DrillQA currentTest;
		private DrillCharActivity activity;

		public ProgressHandler() {
			this.drillQA = new ArrayList<DrillCharActivity.DrillQA>();
			this.clear();
		}

		public void clear() {
			this.progressBar = null;
			this.drillQA.clear();
			this.numKanji = 0;
			this.numLoadedKanji = 0;
			this.numTestDone = 0;
			this.numBadResponse = 0;
			this.currentTest = null;
			this.activity = null;
			if (null != this.progressDialog) {
				this.progressDialog.dismiss();
				this.progressDialog = null;
			}
		}

		public void setProgressBar(ProgressBar progressBar) {
			this.progressBar = progressBar;
			if (null != progressBar) {
				progressBar.setMax(this.numKanji);
				progressBar.setProgress(this.numLoadedKanji);
			}
		}

		public void setActivity(DrillCharActivity activity) {
			this.activity = activity;
		}

		public void setDrillCount(int size) {
			Message msg = this.obtainMessage();
			Bundle b = msg.getData();
			b.clear();
			b.putInt(MSG_DRILL_COUNT, size);
			this.sendMessage(msg);
		}

		public void addKanji(String question, String answer) {
			Message msg = this.obtainMessage();
			Bundle b = msg.getData();
			b.clear();
			b.putString(MSG_QUESTION, question);
			b.putString(MSG_ANSWER, answer);
			this.sendMessage(msg);
		}

		/**
		 * Remove one kanji to the total (no description available for this
		 * kanji)
		 */
		public void recount() {
			Message msg = this.obtainMessage();
			Bundle b = msg.getData();
			b.clear();
			b.putString(MSG_RECOUNT, MSG_RECOUNT);
			this.sendMessage(msg);
		}

		public void loadFinish() {
			Message msg = this.obtainMessage();
			Bundle b = msg.getData();
			b.clear();
			b.putBoolean(MSG_DONE, true);
			this.sendMessage(msg);
		}

		@Override
		public void handleMessage(Message msg) {
			Bundle bundle = msg.getData();
			if (bundle.containsKey(MSG_QUESTION)) {
				String question = bundle.getString(MSG_QUESTION);
				String answer = bundle.getString(MSG_ANSWER);
				this.numLoadedKanji++;
				DrillQA dqa = new DrillQA();
				dqa.question = question;
				dqa.answer = answer;
				this.drillQA.add(dqa);
				if (null != this.progressBar) {
					this.progressBar.setProgress(this.numLoadedKanji);
				}
				if (this.activity != null && this.numLoadedKanji == (NUM_ANSWERS + 1)) {
					if (this.progressDialog != null) {
						this.progressDialog.dismiss();
						this.progressDialog = null;
					}
					this.activity.nextQuestion();
					this.activity = null;
				}
				// Log.d(TAG, "handle new kanji " + question);
			} else if (bundle.containsKey(MSG_RECOUNT)) {
				this.numKanji--;
				if (null != this.progressBar) {
					this.progressBar.setMax(this.numKanji);
				}
			} else if (bundle.containsKey(MSG_DRILL_COUNT)) {
				int size = bundle.getInt(MSG_DRILL_COUNT);
				this.drillQA.ensureCapacity(size);
				this.drillQA.clear();
				this.numKanji = size;
				if (null != this.progressBar) {
					this.progressBar.setMax(size);
					this.progressBar.setVisibility(View.VISIBLE);
				}

			} else if (bundle.containsKey(MSG_DONE)) {
				if (null != this.progressBar) {
					this.progressBar.setVisibility(View.GONE);
				}
				System.gc();
			}
		}
	}

	/**
	 * A simple class to handle one drill questions/answer
	 * 
	 */
	private static class DrillQA implements Serializable {
		/**
		 * serialVersionUID.
		 */
		private static final long serialVersionUID = -6144763261950047681L;

		String question = null;
		String answer = null;
		String userAnswer = null;
		boolean see = false;
	}
}
