/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.kanji;

import java.util.ArrayList;
import java.util.TreeSet;

import org.vono.narau.Common;
import org.vono.narau.R;
import org.vono.narau.db.KanjiDB;
import org.vono.narau.models.kanji.Kanji;

import android.app.Activity;
import android.app.ProgressDialog;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.os.Parcel;
import android.os.Parcelable;
import android.text.Html;
import android.text.Layout;
import android.util.Log;
import android.util.SparseIntArray;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnLongClickListener;
import android.view.View.OnTouchListener;
import android.widget.ScrollView;
import android.widget.TextView;

/**
 * Display a list of kanjis pass as an extra with the TAG key, and return the
 * position of the character which had a long click on it.
 * 
 */
public class KanjiListSelectActivity extends Activity implements OnTouchListener, OnLongClickListener {
	public static final String TAG = KanjiListSelectActivity.class.getSimpleName();
	private static final String KEY_KANJI_LIST = "KL";
	private static final String KEY_RADICAL_LIST = "RL";
	private static final String KEY_RADICAL_NUM_SELECT = "RNS";
	private static final int NUM_KANJI_THRESHOLD = 500;

	private static ActivityHandler handler;
	private static RadicalLoaderThread thread;

	private ScrollView scrollViewRadicals;
	private TextView textViewKanji;
	private TextView textViewRadicals;
	private ProgressDialog progressDialog;
	private int position;
	private ArrayList<Kanji> kanjiList;
	private ArrayList<Radical> radicalsList;
	/**
	 * When kanjis are filtered, store the display position and the absolute
	 * kanji position, to be able to return the good value to the parent
	 * activity.
	 */
	private SparseIntArray filterPositions;
	private int numRadSelected;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		this.position = -1;
		this.numRadSelected = 0;
		this.filterPositions = new SparseIntArray();
		this.progressDialog = null;

		Bundle extra = this.getIntent().getExtras();

		if (null != savedInstanceState) {
			this.numRadSelected = savedInstanceState.getInt(KEY_RADICAL_NUM_SELECT);
			this.kanjiList = savedInstanceState.getParcelableArrayList(KEY_KANJI_LIST);
			this.radicalsList = savedInstanceState.getParcelableArrayList(KEY_RADICAL_LIST);
			if (null != KanjiListSelectActivity.handler) {
				// device rotation while loading
				this.progressDialog = ProgressDialog.show(this, Common.EMPTY_STRING, super.getString(R.string.loading),
						true);
				KanjiListSelectActivity.handler.activity = this;
			} else {
				this.initUI();
				this.updateKanjiList();
			}
		} else if (null != extra) {

			this.loadRadicals(extra.getIntArray(TAG));

		} else {
			Log.e(TAG, "no text to display");
			super.finish();
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.kanji_list_select, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		if (View.GONE == this.scrollViewRadicals.getVisibility()) {
			this.scrollViewRadicals.setVisibility(View.VISIBLE);
		} else {
			this.scrollViewRadicals.setVisibility(View.GONE);
		}
		return true;
	}

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {

		if (KeyEvent.KEYCODE_BACK == keyCode) {
			// Quit this activity without choosing a kanji
			super.setResult(-1);
			super.finish();
			return true;
		}
		return false;
	}

	/**
	 * <p>
	 * Handle device rotation.
	 * </p>
	 * {@inheritDoc}
	 * 
	 */
	@Override
	public void onSaveInstanceState(Bundle savedInstanceState) {
		super.onSaveInstanceState(savedInstanceState);

		savedInstanceState.putInt(KEY_RADICAL_NUM_SELECT, this.numRadSelected);
		savedInstanceState.putParcelableArrayList(KEY_KANJI_LIST, this.kanjiList);
		savedInstanceState.putParcelableArrayList(KEY_RADICAL_LIST, this.radicalsList);

		if (null != this.progressDialog) {
			this.progressDialog.dismiss();
			this.progressDialog = null;
		}
	}

	public boolean onTouch(View view, MotionEvent mEvent) {
		// original code: http://stackoverflow.com/questions/8637603/
		Layout layout = ((TextView) view).getLayout();
		int x = (int) mEvent.getX();
		int y = (int) mEvent.getY();

		if (layout != null) {
			int line = layout.getLineForVertical(y);
			this.position = layout.getOffsetForHorizontal(line, x) / 2;
		}

		if (view.getId() == R.id.kanjiListSelectTextViewRadicals && mEvent.getAction() == MotionEvent.ACTION_UP) {
			int pos = this.position + 1;
			this.position = -1;
			if (pos < this.radicalsList.size()) {
				Radical r = this.radicalsList.get(pos);
				r.selected = !r.selected;
				if (r.selected) {
					this.numRadSelected++;
				} else if (this.numRadSelected > 0) {
					this.numRadSelected--;
				} else {
					this.numRadSelected = 0;
				}
				this.updateKanjiList();
			}
			return true;
		}

		return false;
	}

	public boolean onLongClick(View v) {
		if (R.id.kanjiListSelectTextViewKanji == v.getId()) {
			if (this.numRadSelected == 0) {
				super.setResult(this.position);
			} else {
				super.setResult(this.filterPositions.get(this.position));
			}
			super.finish();

			return true;
		}

		return false;
	}

	private void loadRadicals(int[] codepoints) {
		final RadicalLoaderThread rlt = new RadicalLoaderThread(codepoints);

		if (codepoints.length < NUM_KANJI_THRESHOLD) {
			rlt.loadRadicals();
			this.kanjiList = rlt.tKanjiList;
			this.radicalsList = rlt.tRadicalsList;

			this.initUI();
			this.updateKanjiList();
		} else {
			this.progressDialog = ProgressDialog.show(this, Common.EMPTY_STRING, super.getString(R.string.loading),
					true);
			KanjiListSelectActivity.thread = rlt;
			KanjiListSelectActivity.handler = new ActivityHandler(this);
			rlt.start();
		}
	}

	private void initUI() {
		super.setContentView(R.layout.kanji_list_select);
		this.textViewKanji = (TextView) super.findViewById(R.id.kanjiListSelectTextViewKanji);
		this.textViewRadicals = (TextView) super.findViewById(R.id.kanjiListSelectTextViewRadicals);
		this.scrollViewRadicals = (ScrollView) super.findViewById(R.id.kanjiListSelectScrollViewRadicals);

		this.textViewKanji.setOnTouchListener(this);
		this.textViewKanji.setOnLongClickListener(this);
		this.textViewRadicals.setOnTouchListener(this);
		this.textViewRadicals.setOnLongClickListener(this);

		super.setTitle(R.string.kanjiList);
	}

	private void updateKanjiList() {
		final ArrayList<Kanji> lKanjiList = this.kanjiList;
		final int len = lKanjiList.size();

		final StringBuilder sb = new StringBuilder(len * 2);
		int numRS = this.numRadSelected;
		this.filterPositions.clear();

		if (0 == numRS) {
			for (int i = 0; i < len; i++) {
				final Kanji kanji = lKanjiList.get(i);
				sb.append(Character.toChars(kanji.codepoint));
				sb.append(" ");
			}

			// Update the "noMatch" flags for radicals
			// And the radical list
			final ArrayList<Radical> lRadicalsList = this.radicalsList;
			final int numRads = lRadicalsList.size();
			final StringBuilder sbRad = new StringBuilder();
			sbRad.append("<html><body>");

			for (int i = 0; i < numRads; i++) {
				final Radical radical = lRadicalsList.get(i);
				radical.noMatch = false;
				sbRad.append(radical.toString());
				sbRad.append(" ");
			}
			sbRad.append("</body></html>");
			this.textViewRadicals.setText(Html.fromHtml(sbRad.toString()));
			
		} else {
			// Build the list of selected radicals
			final ArrayList<Radical> lRadicalsList = this.radicalsList;
			final int numRads = lRadicalsList.size();
			final ArrayList<Integer> selectRads = new ArrayList<Integer>(numRS);
			for (int i = 0; i < numRads; i++) {
				final Radical r = lRadicalsList.get(i);
				if (r.selected) {
					selectRads.add(r.codepoint);

				}
			}
			// Check coherence:
			if (numRS != selectRads.size()) {
				this.numRadSelected = numRS = selectRads.size();
			}

			// Filter the kanjis list
			int selectPos = 0;
			final TreeSet<Integer> kanjisRads = new TreeSet<Integer>();
			for (int i = 0; i < len; i++) {
				final Kanji kanji = lKanjiList.get(i);
				if (kanji.radicals.containsAll(selectRads)) {
					sb.append(Character.toChars(kanji.codepoint));
					sb.append(" ");
					this.filterPositions.append(selectPos, i);
					selectPos++;
					kanjisRads.addAll(kanji.radicals);
				}
			}

			// Update the "noMatch" flags for radicals
			// And the radical list
			final StringBuilder sbRad = new StringBuilder();
			sbRad.append("<html><body>");

			for (int i = 0; i < numRads; i++) {
				final Radical radical = lRadicalsList.get(i);
				if (kanjisRads.contains(radical.codepoint)) {
					radical.noMatch = false;
				} else {
					radical.noMatch = true;
				}
				sbRad.append(radical.toString());
				sbRad.append(" ");
			}
			sbRad.append("</body></html>");
			this.textViewRadicals.setText(Html.fromHtml(sbRad.toString()));
		}

		this.textViewKanji.setText(sb);
	}

	public static class Radical implements Parcelable {
		boolean selected;
		boolean noMatch;
		final int codepoint;

		public Radical(int codepoint) {
			this.selected = false;
			this.noMatch = false;
			this.codepoint = codepoint;
		}

		public Radical(Parcel in) {
			this.codepoint = in.readInt();
			this.selected = in.readByte() == 1 ? true : false;
			this.noMatch = in.readByte() == 1 ? true : false;
		}

		@Override
		public String toString() {
			final String strCodepoint = new String(Character.toChars(this.codepoint));
			if (this.selected) {
				return "<font color=\"#5555FF\">" + strCodepoint + "</font>";
			} else if (this.noMatch) {
				return "<font color=\"#AAAAAA\">" + strCodepoint + "</font>";
			}
			return "<font color=\"#FFFFFF\">" + strCodepoint + "</font>";
		}

		@Override
		public boolean equals(Object o) {
			if (o instanceof Radical) {
				final Radical rO = (Radical) o;
				return this.codepoint == rO.codepoint && this.selected == rO.selected;
			} else if (o instanceof Integer) {
				final Integer cp = (Integer) o;
				return this.selected && this.codepoint == cp;
			}

			return false;
		}

		public int describeContents() {
			return 0;
		}

		public void writeToParcel(Parcel dest, int flags) {
			dest.writeInt(this.codepoint);
			dest.writeByte((byte) (this.selected ? 1 : 0));
			dest.writeByte((byte) (this.noMatch ? 1 : 0));
		}

		public static final Parcelable.Creator<Radical> CREATOR = new Parcelable.Creator<Radical>() {
			public Radical createFromParcel(Parcel in) {
				return new Radical(in);
			}

			public Radical[] newArray(int size) {
				return new Radical[size];
			}
		};
	}

	private class RadicalLoaderThread extends Thread {
		private ArrayList<Kanji> tKanjiList;
		private ArrayList<Radical> tRadicalsList;
		private int[] codepoints;

		public RadicalLoaderThread(int[] codepoints) {
			this.tKanjiList = null;
			this.tRadicalsList = null;
			this.codepoints = codepoints;
		}

		@Override
		public void run() {
			this.loadRadicals();
			ActivityHandler.sendLoadDone();
		}

		/**
		 * Fetch the radicals for each kanjis.<br/>
		 * Create the kanjis list and radicals list.
		 */
		void loadRadicals() {
			final int[] cp = this.codepoints;
			final int numKanjis = cp.length;

			this.tKanjiList = new ArrayList<Kanji>(numKanjis);
			final ArrayList<Kanji> lKanjiList = this.tKanjiList;
			final TreeSet<Integer> setRadicals = KanjiDB.getKanjiRadicals(cp);

			for (int i = 0; i < numKanjis; i++) {
				lKanjiList.add(Kanji.getKanji(cp[i]));
			}

			final int numRads = setRadicals.size();
			final Integer[] radCP = setRadicals.toArray(new Integer[0]);
			this.tRadicalsList = new ArrayList<KanjiListSelectActivity.Radical>(numRads);
			final ArrayList<KanjiListSelectActivity.Radical> lRadicalsList = this.tRadicalsList;
			for (int i = 0; i < numRads; i++) {
				final Radical rad = new Radical(radCP[i]);
				lRadicalsList.add(rad);
			}

		}
	}

	private static class ActivityHandler extends Handler {
		static final int LOAD_DONE = 4073;

		KanjiListSelectActivity activity;

		public ActivityHandler(KanjiListSelectActivity activity) {
			this.activity = activity;
		}

		static void sendLoadDone() {
			KanjiListSelectActivity.handler.sendEmptyMessage(LOAD_DONE);
		}

		@Override
		public void handleMessage(Message msg) {
			if (LOAD_DONE == msg.what && null != activity) {
				if (null != activity.progressDialog) {
					activity.progressDialog.dismiss();
					activity.progressDialog = null;
				}
				activity.kanjiList = KanjiListSelectActivity.thread.tKanjiList;
				activity.radicalsList = KanjiListSelectActivity.thread.tRadicalsList;

				activity.initUI();
				activity.updateKanjiList();

				KanjiListSelectActivity.thread = null;
				KanjiListSelectActivity.handler = null;
			}
		}
	}

}
