/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.kanji;

import org.vono.narau.Common;
import org.vono.narau.R;
import org.vono.narau.preferences.Preferences;
import org.vono.narau.preferences.SeekBarPreference;

import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.SeekBar;
import android.widget.SeekBar.OnSeekBarChangeListener;
import android.widget.TextView;

/**
 * The dialog to change the drawing speed, and the stroke width of the kanji. <br/>
 * 
 * @see SeekBarPreference
 */
public class KanjiViewDialogSeekBar extends DialogFragment implements OnSeekBarChangeListener,
		OnClickListener {
	public static final String ID_MIN = "idMin";
	public static final String ID_MAX = "idMax";
	public static final String ID_DIVIDER = "idDivider";
	public static final String ID_PREF_NAME = "idPrefName";
	public static final String ID_TITLE = "idTitle";

	private float fDivider;
	private float fMin;
	private float fMax;
	private float fValue;
	private int idTitle;
	private int idPrefName;
	private TextView textView;
	private OnDismissListener dismissListener;

	public KanjiViewDialogSeekBar() {
		this.fDivider = 0;
		this.fMin = 0;
		this.fMax = 0;
		this.fValue = 0;

		this.idTitle = 0;
		this.dismissListener = null;
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		Bundle bundle = getArguments();
		this.idTitle = bundle.getInt(ID_TITLE);
		this.idPrefName = bundle.getInt(ID_PREF_NAME);
		
		this.fMin = Common.getResourcesFloat(bundle.getInt(ID_MIN));
		this.fMax = Common.getResourcesFloat(bundle.getInt(ID_MAX));
		this.fDivider = Common.getResourcesFloat(bundle.getInt(ID_DIVIDER));
		this.fValue = Preferences.getFloat(this.idPrefName);
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		this.getDialog().setTitle(this.idTitle);

		View view = inflater.inflate(R.layout.preference_dialog_seekbar, container, false);

		SeekBar seekBar = (SeekBar) view.findViewById(R.id.prefDialogSeekBarBar);
		seekBar.setMax((int) (this.fDivider * (this.fMax - this.fMin)));
		seekBar.setProgress((int) (this.fDivider * (this.fValue - this.fMin)));
		seekBar.setOnSeekBarChangeListener(this);

		this.textView = (TextView) view.findViewById(R.id.prefDialogSeekBarText);
		this.textView.setText(String.format("%1$.1f/%2$.1f", this.fValue, this.fMax));

		Button button = (Button) view.findViewById(R.id.prefDialogSeekBarButtonOk);
		button.setOnClickListener(this);

		return view;
	}

	public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
		this.fValue = progress / this.fDivider + this.fMin;
		this.textView.setText(String.format("%1$.1f/%2$.1f", this.fValue, this.fMax));
	}

	public void onStartTrackingTouch(SeekBar seekBar) {
	}

	public void onStopTrackingTouch(SeekBar seekBar) {
	}

	public void onClick(View v) {
		this.dismiss();
		Preferences.setFloat(this.idPrefName, this.fValue);
		if (null != this.dismissListener) {
			dismissListener.onDialogDismiss(this);
		}
	}

	public void setOnDismissListener(OnDismissListener listener) {
		this.dismissListener = listener;
	}

	public static interface OnDismissListener {
		void onDialogDismiss(KanjiViewDialogSeekBar dialog);
	}

}
