/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.learn.reading;

import java.util.ArrayList;
import java.util.HashMap;

import org.vono.narau.Common;
import org.vono.narau.kanji.KanjiArrayAdapter;
import org.vono.narau.models.kanji.KanjiData;

import android.app.Activity;
import android.content.Context;
import android.graphics.Color;
import android.os.Bundle;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.LinearLayout.LayoutParams;
import android.widget.ScrollView;
import android.widget.TableLayout;
import android.widget.TableRow;
import android.widget.TextView;

/**
 * 
 * Display Hiragana/Katakana charts. TODO: rework not pretty code.
 */
public class KanaChartActivity extends Activity {
	/**
	 * Log tag
	 */
	// private static final String TAG =
	// KanaChartActivity.class.getSimpleName();

	static final int TYPE_MASK = KanjiData.KANA_CHART;
	
	private static HashMap<Common.KanaType, KanaChartActivity.TableScrollView> views;
	

	static {
		KanaChartActivity.views = new HashMap<Common.KanaType, KanaChartActivity.TableScrollView>(2);
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		Bundle extra = this.getIntent().getExtras();
		if (null == extra) {
			throw new RuntimeException("don't know what to display");
		}

		KanjiData.WritingCategory wc = (KanjiData.WritingCategory) extra
				.getSerializable(KanjiArrayAdapter.ITEM_CATEGORY);

		final TableScrollView view = this.buildTable(wc);
		if (null != view) {
			super.setContentView(view);
		}
	}

	private TableScrollView buildTable(KanjiData.WritingCategory wc) {
		TableScrollView scrollView = null;

		scrollView = KanaChartActivity.views.get(wc.type);

		if (null == scrollView) {
			ArrayList<TextView> listTV = new ArrayList<TextView>(10);

			Context context = super.getApplicationContext();
			TableLayout tableLayout = new TableLayout(context);

			String[][] kanaChartArray = null;
			switch (wc.type) {
			case HIRAGANA:
				kanaChartArray = KanjiData.HIRAGANA_CHART;
				break;
			case KATAKANA:
				kanaChartArray = KanjiData.KATAKANA_CHART;
				break;
			default:
				super.finish();
				return null;
			}

			int count = kanaChartArray.length;
			TableRow.LayoutParams lpOne = new TableRow.LayoutParams(
					TableRow.LayoutParams.MATCH_PARENT, TableRow.LayoutParams.WRAP_CONTENT);
			lpOne.span = 5;

			TableRow.LayoutParams lpSix = new TableRow.LayoutParams(
					TableRow.LayoutParams.MATCH_PARENT, TableRow.LayoutParams.WRAP_CONTENT);
			lpSix.setMargins(0, 3, 0, 3);

			for (int i = 0; i < count; i++) {
				String[] line = kanaChartArray[i];
				TableRow row = new TableRow(context);

				if (line.length == 1) {
					TextView tv = new TextView(context);
					tv.setText(line[0]);
					tv.setGravity(Gravity.CENTER);
					tv.setPadding(0, 5, 0, 0);
					tv.setTextColor(Color.BLACK);
					row.addView(tv, lpOne);
					row.setBackgroundColor(Color.WHITE);
					tableLayout.addView(row);
				} else {
					for (int j = 0; j < line.length; j++) {
						String kana = line[j];
						if (kana.length() > 0) {
							TextView tv = new TextView(context);
							// tv.setBackgroundColor(Color.BLUE);

							String romanji = Common.convertKana(line[j], wc.type,
									Common.KanaType.ROMANJI);
							tv.setTextSize(30);
							tv.setGravity(Gravity.CENTER);

							tv.setText(kana + "\n" + romanji);
							LayoutParams lp = new LinearLayout.LayoutParams(
									LayoutParams.WRAP_CONTENT, LayoutParams.MATCH_PARENT, 1.0f);
							tv.setLayoutParams(lp);

							row.addView(tv, lpSix);
							listTV.add(tv);
						} else {
							View v = new View(context);
							row.addView(v);
						}

					}
					tableLayout.addView(row);
					// setDividerDrawable() is API level 11
					// so add a fake row
					row = new TableRow(context);
					row.setBackgroundColor(Color.WHITE);
					row.setPadding(0, 0, 0, 1);
					tableLayout.addView(row);

				}
			}
			tableLayout.setPadding(0, 0, 3, 0);
			// Log.d(TAG, "maxWidth: " + maxWidth);

			scrollView = new TableScrollView(context, listTV);
			scrollView.addView(tableLayout);
			KanaChartActivity.views.put(wc.type, scrollView);
		} else {
			((ViewGroup) scrollView.getParent()).removeView(scrollView);
		}
		return scrollView;
	}

	private static class TableScrollView extends ScrollView {
		private ArrayList<TextView> listTV;

		public TableScrollView(Context context, ArrayList<TextView> listTV) {
			super(context);
			this.listTV = listTV;
		}

		@Override
		public void onSizeChanged(int w, int h, int oldw, int oldh) {
			super.onSizeChanged(w, h, oldw, oldh);

			// Resize all the text views
			// to make all the kana TextView's have the same size.
			int tvW = w / 5;
			if (w > 0) {
				for (TextView tv : listTV) {
					tv.setWidth(tvW);
				}
			}
		}
	}

}
