/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.models.kanji;

/**
 * 
 *
 */
public class ConvKanaRomanji {
	/**
	 * Source: http://www.mauvecloud.net/charsets/romaji.html
	 */
	public static final String[][] HIRAGANA_ROMANJI = { { "ぁ", "xa" }, { "あ", "a" },
			{ "ああ", "aa" }, { "ぃ", "xi" }, { "い", "i" }, { "いぃ", "yi" }, { "いぃい", "yii" },
			{ "いい", "ii" }, { "いぇ", "ye" }, { "いぇえ", "yee" }, { "ぅ", "xu" }, { "う", "u" },
			{ "うぁ", "w'a" }, { "うぁあ", "w'aa" }, { "うぃ", "w'i" }, { "うぃい", "w'ii" },
			{ "うぅ", "w'u" }, { "うぅう", "w'uu" }, { "うう", "uu" }, { "うぇ", "w'e" }, { "うぇえ", "w'ee" },
			{ "うぉ", "w'o" }, { "うぉう", "w'ou" }, { "うぉお", "w'oo" }, { "ぇ", "xe" }, { "え", "e" },
			{ "ええ", "ee" }, { "ぉ", "xo" }, { "お", "o" }, { "おう", "ou" }, { "おお", "oo" },
			{ "か", "ka" }, { "かあ", "kaa" }, { "が", "ga" }, { "があ", "gaa" }, { "き", "ki" },
			{ "きぃ", "kyi" }, { "きぃい", "kyii" }, { "きい", "kii" }, { "きぇ", "kye" },
			{ "きぇえ", "kyee" }, { "きゃ", "kya" }, { "きゃあ", "kyaa" }, { "きゅ", "kyu" },
			{ "きゅう", "kyuu" }, { "きょ", "kyo" }, { "きょう", "kyou" }, { "きょお", "kyoo" },
			{ "ぎ", "gi" }, { "ぎぃ", "gyi" }, { "ぎぃい", "gyii" }, { "ぎい", "gii" }, { "ぎぇ", "gye" },
			{ "ぎぇえ", "gyee" }, { "ぎゃ", "gya" }, { "ぎゃあ", "gyaa" }, { "ぎゅ", "gyu" },
			{ "ぎゅう", "gyuu" }, { "ぎょ", "gyo" }, { "ぎょう", "gyou" }, { "ぎょお", "gyoo" },
			{ "く", "ku" }, { "くぁ", "kwa" }, { "くぁあ", "kwaa" }, { "くぃ", "kwi" }, { "くぃい", "kwii" },
			{ "くぅ", "kwu" }, { "くぅう", "kwuu" }, { "くう", "kuu" }, { "くぇ", "kwe" },
			{ "くぇえ", "kwee" }, { "くぉ", "kwo" }, { "くぉう", "kwou" }, { "くぉお", "kwoo" },
			{ "ぐ", "gu" }, { "ぐぁ", "gwa" }, { "ぐぁあ", "gwaa" }, { "ぐぃ", "gwi" }, { "ぐぃい", "gwii" },
			{ "ぐぅ", "gwu" }, { "ぐぅう", "gwuu" }, { "ぐう", "guu" }, { "ぐぇ", "gwe" },
			{ "ぐぇえ", "gwee" }, { "ぐぉ", "gwo" }, { "ぐぉう", "gwou" }, { "ぐぉお", "gwoo" },
			{ "け", "ke" }, { "けえ", "kee" }, { "げ", "ge" }, { "げえ", "gee" }, { "こ", "ko" },
			{ "こう", "kou" }, { "こお", "koo" }, { "ご", "go" }, { "ごう", "gou" }, { "ごお", "goo" },
			{ "さ", "sa" }, { "さあ", "saa" }, { "ざ", "za" }, { "ざあ", "zaa" }, { "し", "shi" },
			{ "しい", "shii" }, { "しぇ", "she" }, { "しぇえ", "shee" }, { "しゃ", "sha" },
			{ "しゃあ", "shaa" }, { "しゅ", "shu" }, { "しゅう", "shuu" }, { "しょ", "sho" },
			{ "しょう", "shou" }, { "しょお", "shoo" }, { "じ", "ji" }, { "じぃ", "jyi" },
			{ "じぃい", "jyii" }, { "じい", "jii" }, { "じぇ", "je" }, { "じぇえ", "jee" }, { "じゃ", "ja" },
			{ "じゃあ", "jaa" }, { "じゅ", "ju" }, { "じゅう", "juu" }, { "じょ", "jo" }, { "じょう", "jou" },
			{ "じょお", "joo" }, { "す", "su" }, { "すぁ", "s'a" }, { "すぁあ", "s'aa" }, { "すぃ", "s'i" },
			{ "すぃい", "s'ii" }, { "すぅ", "s'u" }, { "すぅう", "s'uu" }, { "すう", "suu" },
			{ "すぇ", "s'e" }, { "すぇえ", "s'ee" }, { "すぉ", "s'o" }, { "すぉう", "s'ou" },
			{ "すぉお", "s'oo" }, { "ず", "zu" }, { "ずう", "zuu" }, { "せ", "se" }, { "せえ", "see" },
			{ "ぜ", "ze" }, { "ぜえ", "zee" }, { "そ", "so" }, { "そう", "sou" }, { "そお", "soo" },
			{ "ぞ", "zo" }, { "ぞう", "zou" }, { "ぞお", "zoo" }, { "た", "ta" }, { "たあ", "taa" },
			{ "だ", "da" }, { "だあ", "daa" }, { "ち", "chi" }, { "ちい", "chii" }, { "ちぇ", "che" },
			{ "ちぇえ", "chee" }, { "ちゃ", "cha" }, { "ちゃあ", "chaa" }, { "ちゅ", "chu" },
			{ "ちゅう", "chuu" }, { "ちょ", "cho" }, { "ちょう", "chou" }, { "ちょお", "choo" },
			{ "ぢ", "dji" }, { "ぢい", "djii" }, { "ぢぇ", "dje" }, { "ぢぇえ", "djee" }, { "ぢゃ", "dja" },
			{ "ぢゃあ", "djaa" }, { "ぢゅ", "dju" }, { "ぢゅう", "djuu" }, { "ぢょ", "djo" },
			{ "ぢょう", "djou" }, { "ぢょお", "djoo" }, { "っ", "xtsu" }, { "っいぃ", "yyi" },
			{ "っいぃい", "yyii" }, { "っいぇ", "yye" }, { "っいぇえ", "yyee" }, { "っうぁ", "ww'a" },
			{ "っうぁあ", "ww'aa" }, { "っうぃ", "ww'i" }, { "っうぃい", "ww'ii" }, { "っうぅ", "ww'u" },
			{ "っうぅう", "ww'uu" }, { "っうぇ", "ww'e" }, { "っうぇえ", "ww'ee" }, { "っうぉ", "ww'o" },
			{ "っうぉう", "ww'ou" }, { "っうぉお", "ww'oo" }, { "っか", "kka" }, { "っかあ", "kkaa" },
			{ "っが", "gga" }, { "っがあ", "ggaa" }, { "っき", "kki" }, { "っきぃ", "kkyi" },
			{ "っきぃい", "kkyii" }, { "っきい", "kkii" }, { "っきぇ", "kkye" }, { "っきぇえ", "kkyee" },
			{ "っきさ", "xx'a" }, { "っきさあ", "xx'aa" }, { "っきし", "xx'i" }, { "っきしい", "xx'ii" },
			{ "っきす", "xx'u" }, { "っきすう", "xx'uu" }, { "っきせ", "xx'e" }, { "っきせえ", "xx'ee" },
			{ "っきそ", "xx'o" }, { "っきそう", "xx'ou" }, { "っきそお", "xx'oo" }, { "っきゃ", "kkya" },
			{ "っきゃあ", "kkyaa" }, { "っきゅ", "kkyu" }, { "っきゅう", "kkyuu" }, { "っきょ", "kkyo" },
			{ "っきょう", "kkyou" }, { "っきょお", "kkyoo" }, { "っぎ", "ggi" }, { "っぎぃ", "ggyi" },
			{ "っぎぃい", "ggyii" }, { "っぎい", "ggii" }, { "っぎぇ", "ggye" }, { "っぎぇえ", "ggyee" },
			{ "っぎゃ", "ggya" }, { "っぎゃあ", "ggyaa" }, { "っぎゅ", "ggyu" }, { "っぎゅう", "ggyuu" },
			{ "っぎょ", "ggyo" }, { "っぎょう", "ggyou" }, { "っぎょお", "ggyoo" }, { "っく", "kku" },
			{ "っくぁ", "kkwa" }, { "っくぁあ", "kkwaa" }, { "っくぃ", "kkwi" }, { "っくぃい", "kkwii" },
			{ "っくぅ", "kkwu" }, { "っくぅう", "kkwuu" }, { "っくう", "kkuu" }, { "っくぇ", "kkwe" },
			{ "っくぇえ", "kkwee" }, { "っくぉ", "kkwo" }, { "っくぉう", "kkwou" }, { "っくぉお", "kkwoo" },
			{ "っぐ", "ggu" }, { "っぐぁ", "ggwa" }, { "っぐぁあ", "ggwaa" }, { "っぐぃ", "ggwi" },
			{ "っぐぃい", "ggwii" }, { "っぐぅ", "ggwu" }, { "っぐぅう", "ggwuu" }, { "っぐう", "gguu" },
			{ "っぐぇ", "ggwe" }, { "っぐぇえ", "ggwee" }, { "っぐぉ", "ggwo" }, { "っぐぉう", "ggwou" },
			{ "っぐぉお", "ggwoo" }, { "っけ", "kke" }, { "っけえ", "kkee" }, { "っげ", "gge" },
			{ "っげえ", "ggee" }, { "っこ", "kko" }, { "っこう", "kkou" }, { "っこお", "kkoo" },
			{ "っご", "ggo" }, { "っごう", "ggou" }, { "っごお", "ggoo" }, { "っさ", "ssa" },
			{ "っさあ", "ssaa" }, { "っざ", "zza" }, { "っざあ", "zzaa" }, { "っし", "sshi" },
			{ "っしい", "sshii" }, { "っしぇ", "sshe" }, { "っしぇえ", "sshee" }, { "っしゃ", "ssha" },
			{ "っしゃあ", "sshaa" }, { "っしゅ", "sshu" }, { "っしゅう", "sshuu" }, { "っしょ", "ssho" },
			{ "っしょう", "sshou" }, { "っじ", "jji" }, { "っじぃ", "jjyi" }, { "っじぃい", "jjyii" },
			{ "っじい", "jjii" }, { "っじぇ", "jje" }, { "っじぇえ", "jjee" }, { "っじゃ", "jja" },
			{ "っじゃあ", "jjaa" }, { "っじゅ", "jju" }, { "っじゅう", "jjuu" }, { "っじょ", "jjo" },
			{ "っじょう", "jjou" }, { "っじょお", "jjoo" }, { "っす", "ssu" }, { "っすぁ", "ss'a" },
			{ "っすぁあ", "ss'aa" }, { "っすぃ", "ss'i" }, { "っすぃい", "ss'ii" }, { "っすぅ", "ss'u" },
			{ "っすぅう", "ss'uu" }, { "っすう", "ssuu" }, { "っすぇ", "ss'e" }, { "っすぇえ", "ss'ee" },
			{ "っすぉ", "ss'o" }, { "っすぉう", "ss'ou" }, { "っすぉお", "ss'oo" }, { "っず", "zzu" },
			{ "っずう", "zzuu" }, { "っせ", "sse" }, { "っせえ", "ssee" }, { "っぜ", "zze" },
			{ "っぜえ", "zzee" }, { "っそ", "sso" }, { "っそう", "ssou" }, { "っそお", "ssoo" },
			{ "っぞ", "zzo" }, { "っぞう", "zzou" }, { "っぞお", "zzoo" }, { "った", "tta" },
			{ "ったあ", "ttaa" }, { "っだ", "dda" }, { "っだあ", "ddaa" }, { "っち", "cchi" },
			{ "っちい", "cchii" }, { "っちぇ", "cche" }, { "っちぇえ", "cchee" }, { "っちゃ", "ccha" },
			{ "っちゃあ", "cchaa" }, { "っちゅ", "cchu" }, { "っちゅう", "cchuu" }, { "っちょ", "ccho" },
			{ "っちょう", "cchou" }, { "っちょお", "cchoo" }, { "っぢ", "ddji" }, { "っぢい", "ddjii" },
			{ "っぢぇ", "ddje" }, { "っぢぇえ", "ddjee" }, { "っぢゃ", "ddja" }, { "っぢゃあ", "ddjaa" },
			{ "っぢゅ", "ddju" }, { "っぢゅう", "ddjuu" }, { "っぢょ", "ddjo" }, { "っぢょう", "ddjou" },
			{ "っぢょお", "ddjoo" }, { "っつ", "ttsu" }, { "っつぁ", "ttsa" }, { "っつぁあ", "ttsaa" },
			{ "っつぃ", "ttsi" }, { "っつぃい", "ttsii" }, { "っつう", "ttsuu" }, { "っつぇ", "ttse" },
			{ "っつぇえ", "ttsee" }, { "っつぉ", "ttso" }, { "っつぉう", "ttsou" }, { "っつぉお", "ttsoo" },
			{ "っづ", "ddzu" }, { "って", "tte" }, { "ってぃ", "tt'i" }, { "ってぃい", "tt'ii" },
			{ "ってえ", "ttee" }, { "っで", "dde" }, { "っでえ", "ddee" }, { "っと", "tto" },
			{ "っとぁ", "tt'a" }, { "っとぁあ", "tt'aa" }, { "っとぅ", "tt'u" }, { "っとう", "ttou" },
			{ "っとぇ", "tt'e" }, { "っとぇえ", "tt'ee" }, { "っとぉ", "tt'o" }, { "っとぉう", "tt'ou" },
			{ "っとぉお", "tt'oo" }, { "っとお", "ttoo" }, { "っど", "ddo" }, { "っどう", "ddou" },
			{ "っどお", "ddoo" }, { "っは", "hha" }, { "っはあ", "hhaa" }, { "っば", "bba" },
			{ "っばあ", "bbaa" }, { "っぱ", "ppa" }, { "っぱあ", "ppaa" }, { "っひ", "hhi" },
			{ "っひぃ", "hhyi" }, { "っひぃい", "hhyii" }, { "っひい", "hhii" }, { "っひぇ", "hhye" },
			{ "っひぇえ", "hhyee" }, { "っひゃ", "hhya" }, { "っひゃあ", "hhyaa" }, { "っひゅ", "hhyu" },
			{ "っひゅう", "hhyuu" }, { "っひょ", "hhyo" }, { "っひょう", "hhyou" }, { "っひょお", "hhyoo" },
			{ "っび", "bbi" }, { "っびぃ", "bbyi" }, { "っびぃい", "bbyii" }, { "っびい", "bbii" },
			{ "っびぇ", "bbye" }, { "っびぇえ", "bbyee" }, { "っびゃ", "bbya" }, { "っびゃあ", "bbyaa" },
			{ "っびゅ", "bbyu" }, { "っびゅう", "bbyuu" }, { "っびょ", "bbyo" }, { "っびょう", "bbyou" },
			{ "っびょお", "bbyoo" }, { "っぴ", "ppi" }, { "っぴぃ", "ppyi" }, { "っぴぃい", "ppyii" },
			{ "っぴい", "ppii" }, { "っぴぇ", "ppye" }, { "っぴぇえ", "ppyee" }, { "っぴゃ", "ppya" },
			{ "っぴゃあ", "ppyaa" }, { "っぴゅ", "ppyu" }, { "っぴゅう", "ppyuu" }, { "っぴょ", "ppyo" },
			{ "っぴょう", "ppyou" }, { "っぴょお", "ppyoo" }, { "っふ", "ffu" }, { "っふぁ", "ffa" },
			{ "っふぁあ", "ffaa" }, { "っふぃ", "ffi" }, { "っふぃい", "ffii" }, { "っふう", "ffuu" },
			{ "っふぇ", "ffe" }, { "っふぇえ", "ffee" }, { "っふぉ", "ffo" }, { "っふぉう", "ffou" },
			{ "っふぉお", "ffoo" }, { "っふゃ", "ffya" }, { "っふゃあ", "ffyaa" }, { "っふゅ", "ffyu" },
			{ "っふゅう", "ffyuu" }, { "っふょ", "ffyo" }, { "っふょう", "ffyou" }, { "っふょお", "ffyoo" },
			{ "っぶ", "bbu" }, { "っぶう", "bbuu" }, { "っぷ", "ppu" }, { "っぷう", "ppuu" },
			{ "っへ", "hhe" }, { "っへえ", "hhee" }, { "っべ", "bbe" }, { "っべえ", "bbee" },
			{ "っぺ", "ppe" }, { "っぺえ", "ppee" }, { "っほ", "hho" }, { "っほう", "hhou" },
			{ "っほお", "hhoo" }, { "っぼ", "bbo" }, { "っぼう", "bbou" }, { "っぼお", "bboo" },
			{ "っぽ", "ppo" }, { "っぽう", "ppou" }, { "っぽお", "ppoo" }, { "っや", "yya" },
			{ "っやあ", "yyaa" }, { "っゆ", "yyu" }, { "っゆう", "yyuu" }, { "っよ", "yyo" },
			{ "っよう", "yyou" }, { "っよお", "yyoo" }, { "っら", "rra" }, { "っらあ", "rraa" },
			{ "っり", "rri" }, { "っりぃ", "rryi" }, { "っりぃい", "rryii" }, { "っりい", "rrii" },
			{ "っりぇ", "rrye" }, { "っりぇえ", "rryee" }, { "っりゃ", "rrya" }, { "っりゃあ", "rryaa" },
			{ "っりゅ", "rryu" }, { "っりゅう", "rryuu" }, { "っりょ", "rryo" }, { "っりょう", "rryou" },
			{ "っりょお", "rryoo" }, { "っる", "rru" }, { "っるう", "rruu" }, { "っれ", "rre" },
			{ "っれえ", "rree" }, { "っろ", "rro" }, { "っろう", "rrou" }, { "っろお", "rroo" },
			{ "っわ", "wwa" }, { "っわあ", "wwaa" }, { "っゐ", "wwi" }, { "っゐい", "wwii" },
			{ "っゑ", "wwe" }, { "っゑえ", "wwee" }, { "っを", "wwo" }, { "っをう", "wwou" },
			{ "っをお", "wwoo" }, { "っゔ", "vvu" }, { "っゔぁ", "vva" }, { "っゔぁあ", "vvaa" },
			{ "っゔぃ", "vvi" }, { "っゔぃい", "vvii" }, { "っゔう", "vvuu" }, { "っゔぇ", "vve" },
			{ "っゔぇえ", "vvee" }, { "っゔぉ", "vvo" }, { "っゔぉう", "vvou" }, { "っゔぉお", "vvoo" },
			{ "つ", "tsu" }, { "つぁ", "tsa" }, { "つぁあ", "tsaa" }, { "つぃ", "tsi" }, { "つぃい", "tsii" },
			{ "つう", "tsuu" }, { "つぇ", "tse" }, { "つぇえ", "tsee" }, { "つぉ", "tso" },
			{ "つぉう", "tsou" }, { "つぉお", "tsoo" }, { "づ", "dzu" }, { "づう", "dzuu" }, { "て", "te" },
			{ "てぃ", "t'i" }, { "てぃい", "t'ii" }, { "てえ", "tee" }, { "で", "de" }, { "でぃ", "di" },
			{ "でぃい", "dii" }, { "でえ", "dee" }, { "と", "to" }, { "とぁ", "t'a" }, { "とぁあ", "t'aa" },
			{ "とぅ", "t'u" }, { "とう", "tou" }, { "とぇ", "t'e" }, { "とぇえ", "t'ee" }, { "とぉ", "t'o" },
			{ "とぉう", "t'ou" }, { "とぉお", "t'oo" }, { "とお", "too" }, { "ど", "do" }, { "どぅ", "du" },
			{ "どぅう", "duu" }, { "どう", "dou" }, { "どお", "doo" }, { "な", "na" }, { "なあ", "naa" },
			{ "に", "ni" }, { "にぃ", "nyi" }, { "にぃい", "nyii" }, { "にい", "nii" }, { "にぇ", "nye" },
			{ "にぇえ", "nyee" }, { "にゃ", "nya" }, { "にゃあ", "nyaa" }, { "にゅ", "nyu" },
			{ "にゅう", "nyuu" }, { "にょ", "nyo" }, { "にょう", "nyou" }, { "にょお", "nyoo" },
			{ "ぬ", "nu" }, { "ぬう", "nuu" }, { "ね", "ne" }, { "ねえ", "nee" }, { "の", "no" },
			{ "のう", "nou" }, { "のお", "noo" }, { "は", "ha" }, { "はあ", "haa" }, { "ば", "ba" },
			{ "ぱ", "pa" }, { "ぱあ", "paa" }, { "ひ", "hi" }, { "ひぃ", "hyi" }, { "ひぃい", "hyii" },
			{ "ひい", "hii" }, { "ひぇ", "hye" }, { "ひぇえ", "hyee" }, { "ひゃ", "hya" },
			{ "ひゃあ", "hyaa" }, { "ひゅ", "hyu" }, { "ひゅう", "hyuu" }, { "ひょ", "hyo" },
			{ "ひょう", "hyou" }, { "ひょお", "hyoo" }, { "び", "bi" }, { "びぃ", "byi" },
			{ "びぃい", "byii" }, { "びい", "bii" }, { "びぇ", "bye" }, { "びぇえ", "byee" },
			{ "びゃ", "bya" }, { "びゃあ", "byaa" }, { "びゅ", "byu" }, { "びゅう", "byuu" },
			{ "びょ", "byo" }, { "びょう", "byou" }, { "びょお", "byoo" }, { "ぴ", "pi" }, { "ぴぃ", "pyi" },
			{ "ぴぃい", "pyii" }, { "ぴい", "pii" }, { "ぴぇ", "pye" }, { "ぴぇえ", "pyee" },
			{ "ぴゃ", "pya" }, { "ぴゃあ", "pyaa" }, { "ぴゅ", "pyu" }, { "ぴゅう", "pyuu" },
			{ "ぴょ", "pyo" }, { "ぴょう", "pyou" }, { "ぴょお", "pyoo" }, { "ふ", "fu" }, { "ふぁ", "fa" },
			{ "ふぁあ", "faa" }, { "ふぃ", "fi" }, { "ふぃい", "fii" }, { "ふう", "fuu" }, { "ふぇ", "fe" },
			{ "ふぇえ", "fee" }, { "ふぉ", "fo" }, { "ふぉう", "fou" }, { "ふぉお", "foo" }, { "ふゃ", "fya" },
			{ "ふゃあ", "fyaa" }, { "ふゅ", "fyu" }, { "ふゅう", "fyuu" }, { "ふょ", "fyo" },
			{ "ふょう", "fyou" }, { "ふょお", "fyoo" }, { "ぶ", "bu" }, { "ぶう", "buu" }, { "ぷ", "pu" },
			{ "ぷう", "puu" }, { "へ", "he" }, { "へえ", "hee" }, { "べ", "be" }, { "べえ", "bee" },
			{ "ぺ", "pe" }, { "ぺえ", "pee" }, { "ほ", "ho" }, { "ほう", "hou" }, { "ほお", "hoo" },
			{ "ぼ", "bo" }, { "ぼう", "bou" }, { "ぼお", "boo" }, { "ぽ", "po" }, { "ぽう", "pou" },
			{ "ぽお", "poo" }, { "ま", "ma" }, { "まあ", "maa" }, { "み", "mi" }, { "みぃ", "myi" },
			{ "みぃい", "myii" }, { "みい", "mii" }, { "みぇ", "mye" }, { "みぇえ", "myee" },
			{ "みゃ", "mya" }, { "みゃあ", "myaa" }, { "みゅ", "myu" }, { "みゅう", "myuu" },
			{ "みょ", "myo" }, { "みょう", "myou" }, { "みょお", "myoo" }, { "む", "mu" }, { "むう", "muu" },
			{ "め", "me" }, { "めえ", "mee" }, { "も", "mo" }, { "もう", "mou" }, { "もお", "moo" },
			{ "ゃ", "xya" }, { "や", "ya" }, { "やあ", "yaa" }, { "ゅ", "xyu" }, { "ゆ", "yu" },
			{ "ゆう", "yuu" }, { "ょ", "xyo" }, { "よ", "yo" }, { "よう", "you" }, { "よお", "yoo" },
			{ "ら", "ra" }, { "らあ", "raa" }, { "り", "ri" }, { "りぃ", "ryi" }, { "りぃい", "ryii" },
			{ "りい", "rii" }, { "りぇ", "rye" }, { "りぇえ", "ryee" }, { "りゃ", "rya" },
			{ "りゃあ", "ryaa" }, { "りゅ", "ryu" }, { "りゅう", "ryuu" }, { "りょ", "ryo" },
			{ "りょう", "ryou" }, { "りょお", "ryoo" }, { "る", "ru" }, { "るう", "ruu" }, { "れ", "re" },
			{ "れえ", "ree" }, { "ろ", "ro" }, { "ろう", "rou" }, { "ろお", "roo" }, { "わ", "wa" },
			{ "わあ", "waa" }, { "ゐ", "wi" }, { "ゐい", "wii" }, { "ゑ", "we" }, { "ゑえ", "wee" },
			{ "を", "wo" }, { "をう", "wou" }, { "をお", "woo" }, { "ん", "n" }, { "んば", "mba" },
			{ "んび", "mbi" }, { "んびぃ", "mbyi" }, { "んびぃい", "mbyii" }, { "んびい", "mbii" },
			{ "んびぇ", "mbye" }, { "んびぇえ", "mbyee" }, { "んびゃ", "mbya" }, { "んびゃあ", "mbyaa" },
			{ "んびゅ", "mbyu" }, { "んびゅう", "mbyuu" }, { "んびょ", "mbyo" }, { "んびょう", "mbyou" },
			{ "んびょお", "mbyoo" }, { "んぶ", "mbu" }, { "んぶう", "mbuu" }, { "んべ", "mbe" },
			{ "んべえ", "mbee" }, { "んぼ", "mbo" }, { "んぼう", "mbou" }, { "んぼお", "mboo" },
			{ "んま", "mma" }, { "んまあ", "mmaa" }, { "んみ", "mmi" }, { "んみぃ", "mmyi" },
			{ "んみぃい", "mmyii" }, { "んみい", "mmii" }, { "んみぇ", "mmye" }, { "んみぇえ", "mmyee" },
			{ "んみゃ", "mmya" }, { "んみゃあ", "mmyaa" }, { "んみゅ", "mmyu" }, { "んみゅう", "mmyuu" },
			{ "んみょ", "mmyo" }, { "んみょう", "mmyou" }, { "んみょお", "mmyoo" }, { "んむ", "mmu" },
			{ "んむう", "mmuu" }, { "んめ", "mme" }, { "んめえ", "mmee" }, { "んも", "mmo" },
			{ "んもう", "mmou" }, { "んもお", "mmoo" }, { "ゔ", "vu" }, { "ゔぁ", "va" }, { "ゔぁあ", "vaa" },
			{ "ゔぃ", "vi" }, { "ゔぃい", "vii" }, { "ゔう", "vuu" }, { "ゔぇ", "ve" }, { "ゔぇえ", "vee" },
			{ "ゔぉ", "vo" }, { "ゔぉう", "vou" }, { "ゔぉお", "voo" }, { "ー", "-" }, };

	/**
	 * Source: http://www.mauvecloud.net/charsets/romaji.html
	 */
	public static final String[][] KATAKANA_ROMANJI = { { "ァ", "xa" }, { "ア", "a" }, { "ィ", "xi" },
			{ "イ", "i" }, { "イィ", "yi" }, { "イイ", "ii" }, { "イェ", "ye" }, { "ゥ", "xu" },
			{ "ウ", "u" }, { "ウァ", "w'a" }, { "ウィ", "w'i" }, { "ウゥ", "w'u" }, { "ウェ", "w'e" },
			{ "ウォ", "w'o" }, { "ェ", "xe" }, { "エ", "e" }, { "ォ", "xo" }, { "オ", "o" },
			{ "カ", "ka" }, { "ガ", "ga" }, { "キ", "ki" }, { "キィ", "kyi" }, { "キェ", "kye" },
			{ "キャ", "kya" }, { "キュ", "kyu" }, { "キョ", "kyo" }, { "ギ", "gi" }, { "ギィ", "gyi" },
			{ "ギェ", "gye" }, { "ギャ", "gya" }, { "ギュ", "gyu" }, { "ギョ", "gyo" }, { "ク", "ku" },
			{ "クァ", "kwa" }, { "クィ", "kwi" }, { "クゥ", "kwu" }, { "クェ", "kwe" }, { "クォ", "kwo" },
			{ "グ", "gu" }, { "グァ", "gwa" }, { "グィ", "gwi" }, { "グゥ", "gwu" }, { "グェ", "gwe" },
			{ "グォ", "gwo" }, { "ケ", "ke" }, { "ゲ", "ge" }, { "コ", "ko" }, { "ゴ", "go" },
			{ "サ", "sa" }, { "ザ", "za" }, { "シ", "shi" }, { "シェ", "she" }, { "シャ", "sha" },
			{ "シュ", "shu" }, { "ショ", "sho" }, { "ジ", "ji" }, { "ジィ", "jyi" }, { "ジェ", "je" },
			{ "ジャ", "ja" }, { "ジュ", "ju" }, { "ジョ", "jo" }, { "ス", "su" }, { "スァ", "s'a" },
			{ "スィ", "s'i" }, { "スゥ", "s'u" }, { "スェ", "s'e" }, { "スォ", "s'o" }, { "ズ", "zu" },
			{ "セ", "se" }, { "ゼ", "ze" }, { "ソ", "so" }, { "ゾ", "zo" }, { "タ", "ta" },
			{ "ダ", "da" }, { "チ", "chi" }, { "チェ", "che" }, { "チェエ", "chee" }, { "チャ", "cha" },
			{ "チュ", "chu" }, { "チョ", "cho" }, { "ヂ", "dji" }, { "ヂェ", "dje" }, { "ヂャ", "dja" },
			{ "ヂャア", "djaa" }, { "ヂュ", "dju" }, { "ヂョ", "djo" }, { "ッ", "xtsu" }, { "ッイィ", "yyi" },
			{ "ッイェ", "yye" }, { "ッウァ", "ww'a" }, { "ッウィ", "ww'i" }, { "ッウゥ", "ww'u" },
			{ "ッウェ", "ww'e" }, { "ッウォ", "ww'o" }, { "ッカ", "kka" }, { "ッガ", "gga" },
			{ "ッキ", "kki" }, { "ッキィ", "kkyi" }, { "ッキェ", "kkye" }, { "ッキャ", "kkya" },
			{ "ッキュ", "kkyu" }, { "ッキョ", "kkyo" }, { "ッギ", "ggi" }, { "ッギィ", "ggyi" },
			{ "ッギェ", "ggye" }, { "ッギャ", "ggya" }, { "ッギュ", "ggyu" }, { "ッギョ", "ggyo" },
			{ "ック", "kku" }, { "ックァ", "kkwa" }, { "ックィ", "kkwi" }, { "ックゥ", "kkwu" },
			{ "ックェ", "kkwe" }, { "ックォ", "kkwo" }, { "ッグ", "ggu" }, { "ッグァ", "ggwa" },
			{ "ッグィ", "ggwi" }, { "ッグゥ", "ggwu" }, { "ッグェ", "ggwe" }, { "ッグォ", "ggwo" },
			{ "ッケ", "kke" }, { "ッゲ", "gge" }, { "ッコ", "kko" }, { "ッゴ", "ggo" }, { "ッサ", "ssa" },
			{ "ッザ", "zza" }, { "ッシ", "sshi" }, { "ッシェ", "sshe" }, { "ッシャ", "ssha" },
			{ "ッシュ", "sshu" }, { "ッショ", "ssho" }, { "ッジ", "jji" }, { "ッジィ", "jjyi" },
			{ "ッジェ", "jje" }, { "ッジャ", "jja" }, { "ッジュ", "jju" }, { "ッジョ", "jjo" },
			{ "ッス", "ssu" }, { "ッスぃ", "ss'i" }, { "ッスァ", "ss'a" }, { "ッスゥ", "ss'u" },
			{ "ッスェ", "ss'e" }, { "ッスォ", "ss'o" }, { "ッズ", "zzu" }, { "ッセ", "sse" },
			{ "ッゼ", "zze" }, { "ッソ", "sso" }, { "ッゾ", "zzo" }, { "ッタ", "tta" }, { "ッダ", "dda" },
			{ "ッチ", "cchi" }, { "ッチェ", "cche" }, { "ッチェエ", "cchee" }, { "ッチャ", "ccha" },
			{ "ッチュ", "cchu" }, { "ッチョ", "ccho" }, { "ッヂ", "ddji" }, { "ッヂェ", "ddje" },
			{ "ッヂャ", "ddja" }, { "ッヂュ", "ddju" }, { "ッヂョ", "ddjo" }, { "ッツ", "ttsu" },
			{ "ッツァ", "ttsa" }, { "ッツィ", "ttsi" }, { "ッツェ", "ttse" }, { "ッツォ", "ttso" },
			{ "ッヅ", "ddzu" }, { "ッテ", "tte" }, { "ッティ", "tt'i" }, { "ッデ", "dde" }, { "ット", "tto" },
			{ "ットァ", "tt'a" }, { "ットゥ", "tt'u" }, { "ットェ", "tt'e" }, { "ットォ", "tt'o" },
			{ "ッド", "ddo" }, { "ッハ", "hha" }, { "ッバ", "bba" }, { "ッパ", "ppa" }, { "ッヒ", "hhi" },
			{ "ッヒィ", "hhyi" }, { "ッヒェ", "hhye" }, { "ッヒャ", "hhya" }, { "ッヒュ", "hhyu" },
			{ "ッヒョ", "hhyo" }, { "ッビ", "bbi" }, { "ッビィ", "bbyi" }, { "ッビェ", "bbye" },
			{ "ッビャ", "bbya" }, { "ッビュ", "bbyu" }, { "ッビョ", "bbyo" }, { "ッピ", "ppi" },
			{ "ッピィ", "ppyi" }, { "ッピェ", "ppye" }, { "ッピャ", "ppya" }, { "ッピュ", "ppyu" },
			{ "ッピョ", "ppyo" }, { "ッフ", "ffu" }, { "ッファ", "ffa" }, { "ッフィ", "ffi" },
			{ "ッフェ", "ffe" }, { "ッフォ", "ffo" }, { "ッフャ", "ffya" }, { "ッフュ", "ffyu" },
			{ "ッフョ", "ffyo" }, { "ッブ", "bbu" }, { "ップ", "ppu" }, { "ッヘ", "hhe" }, { "ッベ", "bbe" },
			{ "ッペ", "ppe" }, { "ッホ", "hho" }, { "ッボ", "bbo" }, { "ッポ", "ppo" }, { "ッヤ", "yya" },
			{ "ッユ", "yyu" }, { "ッヨ", "yyo" }, { "ッラ", "rra" }, { "ッリ", "rri" }, { "ッリィ", "rryi" },
			{ "ッリェ", "rrye" }, { "ッリャ", "rrya" }, { "ッリュ", "rryu" }, { "ッリョ", "rryo" },
			{ "ッル", "rru" }, { "ッレ", "rre" }, { "ッロ", "rro" }, { "ッワ", "wwa" }, { "ッヰ", "wwi" },
			{ "ッヱ", "wwe" }, { "ッヲ", "wwo" }, { "ッヴ", "vvu" }, { "ッヴァ", "vva" }, { "ッヴィ", "vvi" },
			{ "ッヴェ", "vve" }, { "ッヴォ", "vvo" }, { "ツ", "tsu" }, { "ツァ", "tsa" }, { "ツィ", "tsi" },
			{ "ツェ", "tse" }, { "ツォ", "tso" }, { "ヅ", "dzu" }, { "テ", "te" }, { "ティ", "t'i" },
			{ "デ", "de" }, { "ディ", "di" }, { "ト", "to" }, { "トァ", "t'a" }, { "トゥ", "t'u" },
			{ "トェ", "t'e" }, { "トォ", "t'o" }, { "ド", "do" }, { "ドゥ", "du" }, { "ナ", "na" },
			{ "ニ", "ni" }, { "ニィ", "nyi" }, { "ニェ", "nye" }, { "ニャ", "nya" }, { "ニュ", "nyu" },
			{ "ニョ", "nyo" }, { "ヌ", "nu" }, { "ネ", "ne" }, { "ノ", "no" }, { "ハ", "ha" },
			{ "バ", "ba" }, { "パ", "pa" }, { "ヒ", "hi" }, { "ヒィ", "hyi" }, { "ヒェ", "hye" },
			{ "ヒャ", "hya" }, { "ヒュ", "hyu" }, { "ヒョ", "hyo" }, { "ビ", "bi" }, { "ビィ", "byi" },
			{ "ビェ", "bye" }, { "ビャ", "bya" }, { "ビュ", "byu" }, { "ビョ", "byo" }, { "ピ", "pi" },
			{ "ピィ", "pyi" }, { "ピェ", "pye" }, { "ピャ", "pya" }, { "ピュ", "pyu" }, { "ピョ", "pyo" },
			{ "フ", "fu" }, { "ファ", "fa" }, { "フィ", "fi" }, { "フェ", "fe" }, { "フォ", "fo" },
			{ "フャ", "fya" }, { "フュ", "fyu" }, { "フョ", "fyo" }, { "ブ", "bu" }, { "プ", "pu" },
			{ "ヘ", "he" }, { "ベ", "be" }, { "ペ", "pe" }, { "ホ", "ho" }, { "ボ", "bo" },
			{ "ポ", "po" }, { "マ", "ma" }, { "ミ", "mi" }, { "ミィ", "myi" }, { "ミェ", "mye" },
			{ "ミャ", "mya" }, { "ミュ", "myu" }, { "ミョ", "myo" }, { "ム", "mu" }, { "メ", "me" },
			{ "モ", "mo" }, { "ャ", "xya" }, { "ヤ", "ya" }, { "ュ", "xyu" }, { "ユ", "yu" },
			{ "ョ", "xyo" }, { "ヨ", "yo" }, { "ラ", "ra" }, { "リ", "ri" }, { "リィ", "ryi" },
			{ "リェ", "rye" }, { "リャ", "rya" }, { "リュ", "ryu" }, { "リョ", "ryo" }, { "ル", "ru" },
			{ "レ", "re" }, { "ロ", "ro" }, { "ワ", "wa" }, { "ヰ", "wi" }, { "ヱ", "we" },
			{ "ヲ", "wo" }, { "ン", "n" }, { "ンバ", "mba" }, { "ンビ", "mbi" }, { "ンビィ", "mbyi" },
			{ "ンビェ", "mbye" }, { "ンビャ", "mbya" }, { "ンビュ", "mbyu" }, { "ンビョ", "mbyo" },
			{ "ンブ", "mbu" }, { "ンベ", "mbe" }, { "ンボ", "mbo" }, { "ンマ", "mma" }, { "ンミ", "mmi" },
			{ "ンミィ", "mmyi" }, { "ンミェ", "mmye" }, { "ンミャ", "mmya" }, { "ンミュ", "mmyu" },
			{ "ンミョ", "mmyo" }, { "ンム", "mmu" }, { "ンメ", "mme" }, { "ンモ", "mmo" }, { "ヴ", "vu" },
			{ "ヴァ", "va" }, { "ヴィ", "vi" }, { "ヴェ", "ve" }, { "ヴォ", "vo" }, { "ヵ", "xka" },
			{ "ヶ", "xke" },  };
}
