package org.vono.narau.utils;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Paint.Align;
import android.graphics.Rect;
import android.util.AttributeSet;
import android.widget.TextView;

/**
 * Code from <a href=
 * "http://stackoverflow.com/questions/2617266/how-to-adjust-text-font-size-to-fit-textview"
 * >stackoverflow.com</a>
 * 
 * @author unSinn
 * 
 */
public class FontFitTextView extends TextView {

	// How much of the available space should be used in percent.
	private static final float MARGINHEIGHT = 0.8f;
	private static final float MARGINWIDTH = 0.8f;

	private Paint paint;
	private int viewWidth;
	private int viewHeight;
	private float textHeight = 0.0f;

	public FontFitTextView(Context c) {
		this(c, null);
	}

	public FontFitTextView(Context c, AttributeSet attrs) {
		super(c, attrs);
		initComponent();
	}

	// Default constructor override
	public FontFitTextView(Context c, AttributeSet attrs, int defStyle) {
		super(c, attrs, defStyle);
		initComponent();
	}

	/**
	 * 
	 */
	private void initComponent() {
		// Context c, AttributeSet attrs, int defStyle
		paint = new Paint();
		paint.setTextSize(10);
		paint.setTextAlign(Align.CENTER);
		paint.setAntiAlias(true);

		// FIXME: Get background/foreground colors from style

		// From class TextView
		// final Resources.Theme theme = super.getContext().getTheme();

		/*
		 * Look the appearance up without checking first if it exists because
		 * almost every TextView has one and it greatly simplifies the logic
		 * to be able to parse the appearance first and then let specific tags
		 * for this View override it.
		 */
		/*
		TypedArray a = theme.obtainStyledAttributes(attrs,
				com.android.internal.R.styleable.TextViewAppearance, defStyle, 0);
		TypedArray appearance = null;
		int ap = a.getResourceId(
				com.android.internal.R.styleable.TextViewAppearance_textAppearance, -1);
		a.recycle();
		if (ap != -1) {
			appearance = theme.obtainStyledAttributes(ap,
					com.android.internal.R.styleable.TextAppearance);
		}
		if (appearance != null) {
			int n = appearance.getIndexCount();
			for (int i = 0; i < n; i++) {
				int attr = appearance.getIndex(i);

				// R.style.
				switch (attr) {

				case com.android.internal.R.styleable.TextAppearance_textColor:
					textColor = appearance.getColorStateList(attr);
					break;

				}
			}

			appearance.recycle();
		}*/
		// int[] styles = new int[1];
		// styles[0] = android.R.attr.textColor;
		// TypedArray a = theme.obtainStyledAttributes(attrs, styles, defStyle,
		// 0);

		paint.setColor(Color.WHITE);
	}

	public void setFontColor(int c) {
		// TODO use style to get the text color
		paint.setColor(c);
	}

	/**
	 * Compute the text size to fit in this text view
	 * 
	 * @param s
	 *            The string to display
	 * @param c
	 *            The canvas where the text is draw
	 */
	private void calcTextSize(String s, Canvas c) {

		float availableWidth = this.viewWidth * MARGINWIDTH;
		float availableHeight = this.viewHeight * MARGINHEIGHT;

		// This value scales the old font up or down to match the available
		// space.
		// float scale = 1.0f;

		// Rectangle for measuring the text dimensions
		Rect rect = new Rect();
		// final float oldFontSize = paint.getTextSize();
		// final float oldFontSize = 10.0f;
		float stepSize = 5.0f;
		// final float initFontSize = 8.0f;

		float txtWidth = 0.0f;
		float txtHeight = 0.0f;
		boolean gotIt = false;

		// First calc something close to the final size
		final float wSizePerPix = 0.84f;
		// final float hSizePerPix = 0.88f;
		float fontSize = wSizePerPix * availableWidth;
		// DisplayMetrics metrics =
		// super.getContext().getResources().getDisplayMetrics();

		paint.setTextSize(fontSize);
		paint.getTextBounds(s, 0, s.length(), rect);
		txtWidth = rect.width();
		txtHeight = rect.height();
		if (txtWidth > availableWidth || txtHeight > availableHeight) {
			stepSize = -stepSize;
		}

		while (!gotIt) {
			paint.setTextSize(fontSize);
			paint.getTextBounds(s, 0, s.length(), rect);
			txtWidth = rect.width();
			txtHeight = rect.height();

			// Log.d("FontFitTextView", "textSize=" + textWidth + "/" +
			// textHeight + " scale="
			// + fontSize);
			if (stepSize > 0) {
				if (txtWidth < availableWidth && txtHeight < availableHeight) {
					fontSize += stepSize;
				} else {
					fontSize -= stepSize;
					gotIt = true;
				}
			} else {
				if (txtWidth > availableWidth || txtHeight > availableHeight) {
					fontSize += stepSize;
				} else {
					fontSize -= stepSize;
					gotIt = true;
				}
			}
		}
		// Log.d("FontFitTextView", "fontSize=" + fontSize);
		this.textHeight = txtHeight;

		/*
				// find scale-value to fit the text horizontally
				float scaleWidth = 1f;
				if (textWidth > 0.0f) {
					scaleWidth = (availableWidth) / textWidth * MARGINWIDTH;
				}
				Log.d("FontFitTextView", "calcTextSize: scaleWidth " + scaleWidth);

				// find scale-value to fit the text vertically
				float scaleHeight = 1f;
				if (textHeight > 0.0f) {
					scaleHeight = (availableHeight) / textHeight * MARGINHEIGHT;
				}
				Log.d("FontFitTextView", "calcTextSize: scaleHeight " + scaleHeight);

				// We are always limited by the smaller one
				if (scaleWidth < scaleHeight) {
					scale = scaleWidth;
				} else {
					scale = scaleHeight;
				}
				

				// We apply the scale to the old font size to make it bigger or smaller
				final float newFontSize = (oldFontSize * scale);
				paint.setTextSize(newFontSize);
		*/
		// Log.d("FontFitTextView", "calcTextSize " + textWidth + "/" +
		// textHeight);
	}

	/**
	 * Calculates the origin on the Y-Axis (width) for the text in this view.
	 * 
	 * @return
	 */
	private float calcStartDrawingPosX() {
		float left = getMeasuredWidth();
		float centerX = left - (this.viewWidth / 2);
		// Log.d("FontFitTextView", "calcStartDrawingPosX " + centerX);
		return centerX;
	}

	/**
	 * Calculates the origin on the Y-Axis (height) for the text in this view.
	 * 
	 * @return
	 */
	private float calcStartDrawingPosY() {
		// float bottom = getMeasuredHeight();
		// The paint only centers horizontally, origin on the Y-Axis stays at
		// the bottom, thus we have to lift the origin additionally by the
		// height of the font.
		// float centerY = bottom - (this.viewHeight / 2) + (this.textHeight /
		// 2);
		float centerY = (this.textHeight + this.viewHeight) / 2;
		// Log.d("FontFitTextView", "calcStartDrawingPosX " + centerY);
		return centerY;
	}

	@Override
	protected void onDraw(Canvas canvas) {
		// Log.d("FontFitTextView", "onDraw");
		String text = getText().toString();
		if (text.length() > 0) {
			calcTextSize(text, canvas);
			canvas.drawText(text, calcStartDrawingPosX(), calcStartDrawingPosY(), paint);
		}
	};

	@Override
	protected void onSizeChanged(int w, int h, int oldw, int oldh) {
		// Log.d("FontFitTextView", "onSizeChanged " + w + " " + h + " old:" +
		// oldw + " " + oldh);
		this.viewWidth = w;
		this.viewHeight = h;
		super.onSizeChanged(w, h, oldw, oldh);
	}

	// @Override
	// protected void onVisibilityChanged(View changedView, int visibility) {
	// super.onVisibilityChanged(changedView, visibility);
	// Log.d("FontFitTextView", "onVisibilityChanged " + visibility);
	// if (visibility == VISIBLE) {
	// super.invalidate();
	// }
	// }

	// @Override
	// protected void onWindowVisibilityChanged(int visibility) {
	// Log.d("FontFitTextView", "onWindowVisibilityChanged " + visibility);
	// }
	//
	// @Override
	// protected void onLayout(boolean changed, int left, int top, int right,
	// int bottom) {
	// super.onLayout(changed, left, top, right, bottom);
	// Log.d("FontFitTextView", "onLayout " + left + " " + top + " " + right +
	// " " + bottom);
	// }
}
