/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau;

import java.io.IOException;
import java.io.InputStream;

import android.app.Activity;
import android.content.res.AssetManager;
import android.os.Bundle;
import android.text.Html;
import android.text.Spanned;
import android.text.method.LinkMovementMethod;
import android.util.Log;
import android.widget.TabHost;
import android.widget.TabHost.TabSpec;
import android.widget.TextView;

/**
 * Display the GPLv3 license and the tanks texts.
 * 
 */
public class LicenseActivity extends Activity {
	private static final String TAG = LicenseActivity.class.getSimpleName();
	private static final int FILE_GPL_SIZE = 37363;
	private static final String FILE_GPL = "gpl.html";
	private static final String FILE_THANKS = "thanks.html";

	private static final String SMALL_GPL = "Narau  Copyright (C) 2012 Yvon TANGUY<br/>\n"
			+ "<br/>\n"
			+ "This program is free software: you can redistribute it and/or modify "
			+ "it under the terms of the GNU General Public License as published by "
			+ "the Free Software Foundation, either version 3 of the License, or "
			+ "(at your option) any later version.<br/>\n"
			+ "<br/>\n"
			+ "This program is distributed in the hope that it will be useful, "
			+ "but WITHOUT ANY WARRANTY; without even the implied warranty of "
			+ "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the "
			+ "GNU General Public License for more details.<br/>\n"
			+ "<br/>\n"
			+ "You should have received a copy of the GNU General Public License "
			+ "along with this program.  If not, see <a href=\"http://www.gnu.org/licenses\">http://www.gnu.org/licenses</a>.";

	private static final String SMALL_THANKS = "Thanks the world";

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		super.setContentView(R.layout.license);

		TabHost mTabHost = (TabHost) super.findViewById(android.R.id.tabhost);
		mTabHost.setup();

		TabSpec mTabSpec = mTabHost.newTabSpec("tab_license");
		mTabSpec.setIndicator(this.getString(R.string.license),
				super.getResources().getDrawable(R.drawable.ic_gpl_v3_logo));
		mTabSpec.setContent(R.id.licenseTextviewGPL);
		mTabHost.addTab(mTabSpec);

		mTabSpec = mTabHost.newTabSpec("tab_thanks");
		mTabSpec.setIndicator(this.getString(R.string.thanks),
				super.getResources().getDrawable(android.R.drawable.btn_star));
		mTabSpec.setContent(R.id.licenseTextviewThanks);
		mTabHost.addTab(mTabSpec);

		TextView mTextView = (TextView) super.findViewById(R.id.licenseTextviewGPL);
		mTextView.setMovementMethod(LinkMovementMethod.getInstance());
		mTextView.setText(this.getGPLHtml(FILE_GPL, SMALL_GPL));

		mTextView = (TextView) super.findViewById(R.id.licenseTextviewThanks);
		mTextView.setMovementMethod(LinkMovementMethod.getInstance());
		mTextView.setText(this.getGPLHtml(FILE_THANKS, SMALL_THANKS));

		mTabHost.setCurrentTab(0);
	}

	/**
	 * Read the file "gpl.html" in the assets directory.
	 * 
	 * @return The HTML file.
	 */
	private Spanned getGPLHtml(String filename, String defaultText) {
		Spanned htmlText = null;
		AssetManager mAssetManager = super.getAssets();

		InputStream mInputStream = null;
		try {
			mInputStream = mAssetManager.open(filename);
			byte[] buffer = new byte[FILE_GPL_SIZE];
			int readCount = 0;
			readCount = mInputStream.read(buffer);
			String fileContent = new String(buffer, 0, readCount);
			htmlText = Html.fromHtml(fileContent);

		} catch (IOException ioe) {
			Log.e(TAG, ioe.getMessage(), ioe);

		} finally {
			if (null != mInputStream) {
				try {
					mInputStream.close();
				} catch (IOException ioe) {
					Log.e(TAG, ioe.getMessage(), ioe);
				}
			}

			if (null == htmlText) {
				htmlText = Html.fromHtml(defaultText);
			}

		}
		return htmlText;
	}

}
