/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.dictionary;

import java.util.ArrayList;

import org.vono.narau.Common;
import org.vono.narau.Common.KanaType;
import org.vono.narau.R;
import org.vono.narau.dictionary.jmdict.Entry;
import org.vono.narau.preferences.Preferences;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.text.Html;
import android.text.method.LinkMovementMethod;
import android.util.Log;
import android.util.TypedValue;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.inputmethod.EditorInfo;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.TextView.OnEditorActionListener;
import android.widget.Toast;
import android.widget.ZoomButton;

public class DictionaryActivity extends Activity implements OnEditorActionListener, OnClickListener {
	private static final String TAG = DictionaryActivity.class.getSimpleName();
	public static final String KEY_MESSAGE = "INFO";
	public static final String KEY_RESULT_AVAIL = "RA";

	private static final String KEY_TXT_SEARCH = "search";
	private static final String KEY_TXT_RESULT = "result";
	private static final String KEY_BUTTON_CONF = "conv";

	public static final String A_ROMANJI = "a";
	public static final String A_HIRAGANA = "あ";
	public static final String A_KATAKANA = "ア";

	/**
	 * Put the results.
	 */
	private TextView textViewResult;

	/**
	 * Contains the word(s) to search
	 */
	private EditText editTextSearch;

	private Button buttonSearch;
	private Button buttonConvRomanji;
	private Common.KanaType buttonConvstate;
	private Button buttonDeleteText;

	private HandlerEdictThread handler;

	private String dictSearch;
	private String dictSearchLangs;

	public DictionaryActivity() {
		this.textViewResult = null;
		this.editTextSearch = null;
		this.buttonSearch = null;
		this.buttonConvRomanji = null;
		this.buttonConvstate = KanaType.ROMANJI;
		this.buttonDeleteText = null;
		this.handler = null;

		this.dictSearch = Common.getString(R.string.dictSearch);
		this.dictSearchLangs = Common.getString(R.string.dictSearchLangs);

	}

	public static boolean haveDatabase() {
		return DictionaryDB.isReady();
	}

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		if (DictionaryDB.isReady()) {

			this.handler = new HandlerEdictThread(this);

			super.setContentView(R.layout.dictionary);
			this.buttonSearch = (Button) findViewById(R.id.dictionaryButtonSearch);

			this.buttonSearch = (Button) super.findViewById(R.id.dictionaryButtonSearch);
			this.buttonSearch.setOnClickListener(this);
			this.buttonConvRomanji = (Button) super.findViewById(R.id.dictionaryButtonConvRomanji);
			this.buttonConvRomanji.setOnClickListener(this);
			if (this.buttonConvRomanji.getText().length() == 0) {
				this.buttonConvRomanji.setText(A_ROMANJI);
				this.buttonConvstate = Common.KanaType.ROMANJI;
			}
			this.buttonDeleteText = (Button) super.findViewById(R.id.dictionaryButtonDeleteSearchText);
			this.buttonDeleteText.setOnClickListener(this);
			ZoomButton button = (ZoomButton) super.findViewById(R.id.dictionaryZoomButtonIn);
			button.setOnClickListener(this);
			button = (ZoomButton) super.findViewById(R.id.dictionaryZoomButtonOut);
			button.setOnClickListener(this);

			this.textViewResult = (TextView) super.findViewById(R.id.dictionaryTextViewResult);
			this.textViewResult.setMovementMethod(LinkMovementMethod.getInstance());

			float zoomValue = Preferences.getFloat(R.string.prefNameTextZoom);
			if (0f == zoomValue) {
				// First time here
				Preferences.setFloat(R.string.prefNameTextZoom, this.textViewResult.getTextSize());
			} else {
				this.textViewResult.setTextSize(TypedValue.COMPLEX_UNIT_PX, zoomValue);
			}

			this.editTextSearch = (EditText) super.findViewById(R.id.dictionaryEditTextSearch);
			this.editTextSearch.setOnEditorActionListener(this);

			DictionaryThread.init(this);
		} else {
			Log.e(TAG, "DictionaryDB not available");
			super.finish();
		}
	}

	/**
	 * Used by the search thread to update the ui
	 * 
	 * @param key
	 *            Type of message
	 * @param message
	 *            the message
	 */
	public void sendMessage(String key, String message) {
		if (null != this.handler) {
			Message msg = Message.obtain();
			Bundle data = new Bundle();
			data.putString(key, message);
			msg.setData(data);

			this.handler.sendMessage(msg);
		}

	}

	private void setLangSearchId(int id) {
		int _id = id;
		if (0 == _id) {
			this.buttonSearch.setText(dictSearch);
		} else if (_id > 0) {
			ArrayList<String> langPriorities = Preferences.getLangsPriorities(Common.Database.dictonary);
			// check max value:
			_id = (_id >= langPriorities.size()) ? 0 : _id;
			String lang = langPriorities.get(_id);
			String langName = Common.getLocale(lang).getDisplayLanguage();
			String text = String.format(this.dictSearchLangs, langName, Common.JPN_LANG_NAME);
			this.buttonSearch.setText(text);
		} else if (_id < 0) {
			ArrayList<String> langPriorities = Preferences.getLangsPriorities(Common.Database.dictonary);
			// check max value:
			_id = (langPriorities.size() < -_id) ? -1 : _id;
			String lang = langPriorities.get(-_id - 1);
			String langName = Common.getLocale(lang).getDisplayLanguage();
			String text = String.format(this.dictSearchLangs, Common.JPN_LANG_NAME, langName);
			this.buttonSearch.setText(text);
		}

		DictionaryThread.setLangSearchId(_id);
	}

	private void displayResult() {
		ArrayList<Entry> lastSearch = DictionaryThread.getLastSearch();
		if (null != lastSearch && lastSearch.size() > 0) {
			int num = lastSearch.size();
			int langId = DictionaryThread.getLangSearchId();

			String displayLang = null;
			int nextLangId = langId;
			if (langId >= 0) {
				displayLang = Preferences.getLangsPriorities(Common.Database.dictonary).get(langId);
				nextLangId++;
			} else {
				displayLang = Preferences.getLangsPriorities(Common.Database.dictonary).get(-1 - langId);
				nextLangId--;
			}

			StringBuilder sb = new StringBuilder();
			sb.append("<html><body>");

			for (int i = 0; i < num; i++) {
				Entry entry = lastSearch.get(i);
				sb.append(entry.toString(displayLang));
			}
			sb.append("</body></html>");

			this.textViewResult.setText(Html.fromHtml(sb.toString()));
			this.setLangSearchId(nextLangId);

			Toast.makeText(this, String.format(this.getString(R.string.dictNumResult), num), Toast.LENGTH_LONG).show();

			// Set the focus to the text view result, to prevent
			// when clicking on a link, to display the search text view.
			// This is annoying when the scroll is far from the beginning.
			this.textViewResult.requestFocus();
		} else {
			StringBuilder sb = new StringBuilder();

			sb.append("<html><body>");
			sb.append(super.getString(R.string.dictNoResult));

			sb.append("<br/><br/><a href=\"http://translate.google.com/m/translate?source=mog\">");
			sb.append(super.getString(R.string.dictTryGoogle));
			sb.append("</a><br/><br/>\n");

			sb.append("<a href=\"http://www.microsofttranslator.com/\">");
			sb.append(super.getString(R.string.dictTryBing));
			sb.append("</a><br/><br/>\n");

			// http://www.microsofttranslator.com/
			sb.append("</body></html>");

			this.textViewResult.setText(Html.fromHtml(sb.toString()));
			this.setLangSearchId(0);
			// show the keyboard
			InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
			imm.showSoftInput(this.editTextSearch, 0);
		}
		this.buttonSearch.setEnabled(true);
		this.buttonConvRomanji.setEnabled(true);
		this.buttonDeleteText.setEnabled(true);
		this.editTextSearch.setEnabled(true);
	}

	/**
	 * Launch the search. Click on dictionaryButtonSearch.
	 * 
	 * @param v
	 *            View
	 */
	public void launchSearch(View v) {
		String searchText = this.editTextSearch.getText().toString().trim();

		if (searchText.length() == 0) {
			this.textViewResult.setText(R.string.dictNothingToSearch);
		} else {
			// Disable the ui during search
			this.buttonSearch.setEnabled(false);
			this.buttonConvRomanji.setEnabled(false);
			this.buttonDeleteText.setEnabled(false);
			this.editTextSearch.setEnabled(false);

			// Hide the keyboard
			InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
			imm.hideSoftInputFromWindow(this.editTextSearch.getWindowToken(), 0);

			// Start to search
			DictionaryThread.setSearchText(searchText);
		}

	}

	public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {
		if (EditorInfo.IME_ACTION_SEARCH == actionId && R.id.dictionaryEditTextSearch == v.getId()) {
			this.launchSearch(v);
			return true;
		}
		return false;
	}

	/**
	 * Click on dictionaryButtonDeleteSearchText
	 * 
	 * @param v
	 */
	public void deleteText(View v) {
		this.editTextSearch.setText(Common.EMPTY_STRING);
		this.setLangSearchId(0);
		this.buttonConvRomanji.setText(A_ROMANJI);
		this.buttonConvstate = Common.KanaType.ROMANJI;

		this.editTextSearch.requestFocus();

		// show the keyboard
		InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
		imm.showSoftInput(this.editTextSearch, 0);
	}

	/**
	 * Click on dictionaryButtonConvRomanji
	 * 
	 * @param v
	 */
	public void convCharacters(View v) {
		String searchText = this.editTextSearch.getText().toString();
		String newText = null;
		switch (buttonConvstate) {
		case ROMANJI:
			this.buttonConvRomanji.setText(A_HIRAGANA);
			this.buttonConvstate = Common.KanaType.HIRAGANA;
			newText = Common.convertKana(searchText, Common.KanaType.ROMANJI, Common.KanaType.HIRAGANA);
			this.setLangSearchId(-1);
			break;
		case HIRAGANA:
			this.buttonConvRomanji.setText(A_KATAKANA);
			this.buttonConvstate = Common.KanaType.KATAKANA;
			newText = Common.convertKana(searchText, Common.KanaType.HIRAGANA, Common.KanaType.KATAKANA);
			this.setLangSearchId(-1);
			break;

		case KATAKANA:
			this.buttonConvRomanji.setText(A_ROMANJI);
			this.buttonConvstate = Common.KanaType.ROMANJI;
			newText = Common.convertKana(searchText, Common.KanaType.KATAKANA, Common.KanaType.ROMANJI);
			this.setLangSearchId(0);

			break;
		default:
			throw new RuntimeException("invalid convertion");
		}
		this.editTextSearch.setText(newText);
	}

	/**
	 * Zoom in the text view result.
	 * 
	 * @param v
	 *            Button zoom in
	 */
	public void zoomIn(View v) {
		float txtSize = this.textViewResult.getTextSize() * 1.1f;
		this.textViewResult.setTextSize(TypedValue.COMPLEX_UNIT_PX, txtSize);
		Preferences.setFloat(R.string.prefNameTextZoom, txtSize);
	}

	/**
	 * Zoom out the text view result.
	 * 
	 * @param v
	 *            Button zoom out
	 */
	public void zoomOut(View v) {
		float txtSize = this.textViewResult.getTextSize() * 0.9f;
		this.textViewResult.setTextSize(TypedValue.COMPLEX_UNIT_PX, txtSize);
		Preferences.setFloat(R.string.prefNameTextZoom, txtSize);
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.menuDictionaryShowControls:
			LinearLayout zoomControls = (LinearLayout) super.findViewById(R.id.dictionaryZoomControls);
			if (View.VISIBLE == zoomControls.getVisibility()) {
				zoomControls.setVisibility(View.GONE);
				item.setTitle(R.string.dictMenuShowControls);
			} else {
				zoomControls.setVisibility(View.VISIBLE);
				item.setTitle(R.string.dictMenuHideControls);
			}
			break;
		default:
			return super.onOptionsItemSelected(item);
		}
		return true;
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.dictionary, menu);
		return true;
	}

	@Override
	public void onNewIntent(Intent intent) {
		// Log.d(TAG, "onNewIntent");
		super.onNewIntent(intent);
		super.setIntent(intent);
	}

	@Override
	protected void onSaveInstanceState(Bundle outState) {
		super.onSaveInstanceState(outState);
		// Log.d(TAG, "onSaveInstanceState");
		outState.putCharSequence(KEY_TXT_RESULT, this.textViewResult.getText());
		outState.putCharSequence(KEY_TXT_SEARCH, this.editTextSearch.getText());
		outState.putString(KEY_BUTTON_CONF, this.buttonConvstate.name());

	}

	@Override
	protected void onRestoreInstanceState(Bundle savedInstanceState) {
		super.onSaveInstanceState(savedInstanceState);
		// Log.d(TAG, "onRestoreInstanceState");

		this.textViewResult.setText(savedInstanceState.getCharSequence(KEY_TXT_RESULT));
		this.editTextSearch.setText(savedInstanceState.getCharSequence(KEY_TXT_SEARCH));
		this.buttonConvstate = Common.KanaType.valueOf(savedInstanceState.getString(KEY_BUTTON_CONF));
		switch (this.buttonConvstate) {
		case HIRAGANA:
			this.buttonConvRomanji.setText(A_HIRAGANA);
			break;
		case KATAKANA:
			this.buttonConvRomanji.setText(A_KATAKANA);
			break;
		case ROMANJI:
			this.buttonConvRomanji.setText(A_ROMANJI);
			break;
		default:
			throw new RuntimeException("invalid convertion");
		}
	}

	@Override
	public void onDestroy() {
		// Log.d(TAG, "onDestroy");

		this.buttonConvRomanji = null;
		this.buttonDeleteText = null;
		this.buttonSearch = null;
		this.textViewResult = null;

		if (null != this.editTextSearch) {
			this.editTextSearch.setOnEditorActionListener(null);
			this.editTextSearch = null;
		}

		if (null != this.handler) {
			this.handler.da = null;
			this.handler = null;
		}

		super.onDestroy();
	}

	private static class HandlerEdictThread extends Handler {
		private DictionaryActivity da;

		public HandlerEdictThread(DictionaryActivity da) {
			this.da = da;
		}

		@Override
		public void handleMessage(Message msg) {
			if (null != da) {
				Bundle data = msg.getData();
				if (data.containsKey(KEY_MESSAGE)) {
					String text = data.getString(KEY_MESSAGE);
					this.da.textViewResult.setText(text);
				} else if (data.containsKey(KEY_RESULT_AVAIL)) {
					this.da.displayResult();
				}
			}
		}
	}

	public void onClick(View v) {
		switch (v.getId()) {
		case R.id.dictionaryButtonSearch:
			this.launchSearch(v);
			break;
		case R.id.dictionaryButtonConvRomanji:
			this.convCharacters(v);
			break;
		case R.id.dictionaryButtonDeleteSearchText:
			this.deleteText(v);
			break;
		case R.id.dictionaryZoomButtonOut:
			this.zoomOut(v);
			break;
		case R.id.dictionaryZoomButtonIn:
			this.zoomIn(v);
			break;
		default:
			Log.e(TAG, "Unknown button " + v.getId());
			break;
		}

	}
}
