/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.dictionary;

import org.vono.narau.R;
import org.vono.narau.dictionary.jmdict.Entry;
import org.vono.narau.preferences.Preferences;

import android.app.Activity;
import android.net.Uri;
import android.os.Bundle;
import android.text.Html;
import android.text.method.LinkMovementMethod;
import android.util.Log;
import android.util.TypedValue;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

/**
 * Display the detail of one entry, when click on a link in the textViewResult
 * on the DictionaryActivity, or display a small pop-up for kanji/reading
 * informations.
 */
public class EntryActivity extends Activity implements OnClickListener {
	public static final String TAG = EntryActivity.class.getSimpleName();

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		final Uri uri = super.getIntent().getData();

		if (null != uri) {

			String host = uri.getHost();
			String lastPath = uri.getLastPathSegment();

			if (Entry.HOST_ENTRY.equals(host)) {

				super.setContentView(R.layout.dictionary_entry);

				try {
					int sequence = Integer.parseInt(lastPath);
					final Entry entry = DictionaryThread.getEntry(sequence);
					this.displayEntry(entry);

				} catch (NumberFormatException nfe) {
					Log.e(TAG, nfe.getMessage(), nfe);
				}
			} else if (Entry.HOST_INF.equals(host)) {
				String text = Informations.getInfText(lastPath);

				if (null != text) {
					Toast.makeText(this, text, Toast.LENGTH_LONG).show();
				}
				finish();
			}
		} else if (null != super.getIntent().getExtras()) {
			super.setContentView(R.layout.dictionary_entry);
			final int sequence = super.getIntent().getExtras().getInt(TAG);
			final Entry entry = DictionaryDB.fetchdEntry(sequence);
			this.displayEntry(entry);
		} else {
			super.finish();
		}
	}

	private void displayEntry(Entry entry) {
		final TextView tv = (TextView) super.findViewById(R.id.dictEntryTextView);
		tv.setMovementMethod(LinkMovementMethod.getInstance());
		final Button button = (Button) super.findViewById(R.id.dictEntryClose);
		button.setOnClickListener(this);

		if (null != entry) {

			StringBuilder sb = new StringBuilder();
			sb.append("<html><body>\n");
			sb.append(entry.toString());
			sb.append("</body></html>");
			tv.setText(Html.fromHtml(sb.toString()));

			final float zoomValue = Preferences.getFloat(R.string.prefNameTextZoom);
			if (0f != zoomValue) {
				tv.setTextSize(TypedValue.COMPLEX_UNIT_PX, zoomValue);
			}
		}
	}

	/**
	 * @param v
	 *            An unused view
	 */
	public void close(View v) {
		// Click from the close button
		super.finish();
	}

	public void onClick(View v) {
		switch (v.getId()) {
		case R.id.dictEntryClose:
			this.close(v);
			break;
		default:
			Log.e(TAG, "Unknown button " + v.getId());
		}

	}

}
