/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.dictionary.jmdict;

import java.util.HashMap;

/**
 * The sense element will record the translational equivalent of the Japanese
 * word, plus other related information. Where there are several distinctly
 * different meanings of the word, multiple sense elements will be employed.
 * 
 */
public class Sense {

	/**
	 * These elements, if present, indicate that the sense is restricted to the
	 * lexeme represented by the keb.
	 */
	public String stagk;

	/**
	 * These elements, if present, indicate that the sense is restricted to the
	 * lexeme represented by the reb.
	 */
	public String stagr;

	/**
	 * Part-of-speech information about the entry/sense. Should use appropriate
	 * entity codes. In general where there are multiple senses in an entry, the
	 * part-of-speech of an earlier sense will apply to later senses unless
	 * there is a new part-of-speech indicated.
	 */
	public String pos;

	/**
	 * This element is used to indicate a cross-reference to another entry with
	 * a similar or related meaning or sense. The content of this element is
	 * typically a keb or reb element in another entry. In some cases a keb will
	 * be followed by a reb and/or a sense number to provide a precise target
	 * for the cross-reference. Where this happens, a JIS "centre-dot" (0x2126)
	 * is placed between the components of the cross-reference.
	 */
	public String xref;

	/**
	 * Within each sense will be one or more "glosses", i.e. target-language
	 * words or phrases which are equivalents to the Japanese word. This element
	 * would normally be present, however it may be omitted in entries which are
	 * purely for a cross-reference.
	 */
	public HashMap<String, String> glosses;

	public Sense() {
		this.glosses = null;
		this.pos = null;
		this.stagk = null;
		this.stagr = null;
		this.xref = null;
	}
}
