/**
 * Copyright (C) 2012 Yvon TANGUY
 * 
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.download;

import java.net.URL;
import java.util.Date;

import org.vono.narau.Common;
import org.vono.narau.R;
import org.vono.narau.dictionary.DictionaryDB;
import org.vono.narau.kanji.KanjiDB;
import org.vono.narau.preferences.Preferences;

import android.widget.CheckBox;

/**
 * filename size file_time sha512 db_version download_url
 * 
 */
public class DBInfos {

	/**
	 * gzip filename.
	 */
	public final String name;
	/**
	 * sqlite db filename.
	 */
	public final String dbName;
	public final int strDBNameId;
	public final int expectedDBVersion;

	public long gzSize;
	public long dbSize;
	private Date creationDate;
	public String md5;
	public int dbVersion;
	public URL[] url;
	public boolean infosLoaded;
	public boolean selected;
	private boolean _haveLatest;
	public CheckBox checkBox;

	public static final DBInfos[] arrayDBInfos = new DBInfos[2];

	static {
		DBInfos infos = new DBInfos(Common.DB_DICT_NAME, R.string.dbDictionary, DictionaryDB.VALUE_DB_VERSION);
		arrayDBInfos[0] = infos;
		infos = new DBInfos(Common.DB_KANJI_NAME, R.string.dbKanjiKana, KanjiDB.VALUE_DB_VERSION);
		arrayDBInfos[1] = infos;
	}

	private DBInfos(String name, int dbNameId, int expectedDBVersion) {
		this.dbName = name;
		this.name = name + Common.GZ_EXT;
		this.strDBNameId = dbNameId;
		this.expectedDBVersion = expectedDBVersion;
		this.infosLoaded = false;
		this.dbSize = 0;
		this.gzSize = 0;
		this.creationDate = null;
		this.selected = true;
		this._haveLatest = false;
		this.checkBox = null;
	}

	public void setCreationDate(Long value) {
		this.creationDate = new Date(value);
		final long oldValue = Preferences.getSP().getLong(Preferences.PREF_DB_CD + this.dbName, 0);
		if (oldValue < value) {
			this.selected = true;
			this._haveLatest = false;
		} else {
			this.selected = false;
			this._haveLatest = true;
		}
	}

	public Date getCreationDate() {
		return this.creationDate;
	}

	public boolean haveLatest() {
		return this._haveLatest;
	}

	public void saveToPref() {
		Preferences.getSP().edit().putLong(Preferences.PREF_DB_CD + this.dbName, this.creationDate.getTime()).commit();
	}
}
