/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.exercise;

import org.vono.narau.Common;
import org.vono.narau.kanji.KanjiArrayAdapter;
import org.vono.narau.kanji.KanjiDB;
import org.vono.narau.kanji.KanjiData;

import android.app.ListActivity;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ListView;

public abstract class DrillMenuActivity extends ListActivity implements OnItemClickListener {
	// private static final String TAG =
	// DrillMenuActivity.class.getSimpleName();
	public static final String DRILL_DIRECTION = "dir";
	protected static final String DRILL_DIRECTION_TO = "to";
	protected static final String DRILL_DIRECTION_FROM = "from";

	protected final String direction;

	public DrillMenuActivity(String dir) {
		this.direction = dir;
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		final boolean dbAvail = Common.haveAllDBAvail(Common.DB_MASK_KANJI);
		int disabledTypeMask = 0;
		if (!dbAvail) {
			disabledTypeMask = KanjiData.KANJI;
		}

		ListView listView = super.getListView();

		KanjiArrayAdapter kaa = new KanjiArrayAdapter(this.getApplicationContext(), DrillCharActivity.TYPE_MASK,
				disabledTypeMask);

		listView.setAdapter(kaa);
		listView.setOnItemClickListener(this);
	}

	public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
		Object tag = view.getTag();

		if (tag instanceof KanjiData.WritingCategory) {
			KanjiData.WritingCategory wc = (KanjiData.WritingCategory) tag;

			Intent intent = null;
			if (DrillMenuActivity.DRILL_DIRECTION_FROM.equals(this.direction)) {
				intent = new Intent(this.getApplicationContext(), DrillCharFromActivity.class);
			} else {
				intent = new Intent(this.getApplicationContext(), DrillCharToActivity.class);
			}
			intent.putExtra(KanjiArrayAdapter.ITEM_CATEGORY, wc);
			intent.putExtra(DrillMenuActivity.DRILL_DIRECTION, this.direction);

			super.startActivity(intent);

		}
	}

	public static boolean haveDatabase() {
		return KanjiDB.isReady();
	}
}
