/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.kanji;

import java.util.ArrayList;

import org.vono.narau.Common;
import org.vono.narau.R;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.TextView;

/**
 * 
 * 
 */
public class KanjiArrayAdapter extends ArrayAdapter<KanjiData.CharacterCatalog> implements
		OnCheckedChangeListener {

	public static final String ITEM_CATEGORY = "ITEM_CATEGORY";

	private ArrayList<View> views;

	/**
	 * 
	 * @param context
	 *            the application context
	 * @param typeMask
	 *            The type of KanjiData.WritingCategory to display
	 * @param disableTypeMask
	 *            The type of KanjiData.WritingCategory to display disabled
	 */
	public KanjiArrayAdapter(Context context, int typeMask, int disableTypeMask) {
		super(context, R.layout.kanji_line_main_caterory);
		final LayoutInflater inflator = (LayoutInflater) super.getContext().getSystemService(
				Context.LAYOUT_INFLATER_SERVICE);

		final int numWType = KanjiData.writingType.length;
		final ArrayList<View> lviews = new ArrayList<View>(KanjiData.NUM_UI_LINES);

		for (int i = 0; i < numWType; i++) {
			final KanjiData.WritingType wt = KanjiData.writingType[i];

			final ArrayList<KanjiData.WritingCategory> alWC = wt.writingCategories;
			final int numCat = alWC.size();
			int addTypePosition = lviews.size();
			int numCatAdd = 0;
			for (int j = 0; j < numCat; j++) {
				final KanjiData.WritingCategory wc = alWC.get(j);
				final ArrayList<KanjiData.CharacterCatalog> alCC = wc.catalogList;
				final int numCC = alCC.size();
				int addCatPosition = lviews.size();
				int numCharAdd = 0;
				boolean haveDisabledItem = false;
				for (int k = 0; k < numCC; k++) {
					KanjiData.CharacterCatalog cc = alCC.get(k);
					if (((cc.typeMask & typeMask) != cc.typeMask)
							|| !Common.haveAllDBAvail(cc.dbMask))
						continue;
					numCharAdd++;
					haveDisabledItem = haveDisabledItem || ((cc.typeMask & disableTypeMask) != 0);
					if (cc.visible) {
						final View v = this.makeViewCharCatalog(cc, context, inflator,
								haveDisabledItem);
						lviews.add(v);
					}
				}
				if (numCharAdd > 0) {
					numCatAdd++;
					lviews.add(addCatPosition, makeViewCategory(wc, inflator, haveDisabledItem));
				}
			}
			if (numCatAdd > 0) {
				final View viewType = inflator.inflate(R.layout.kanji_line_main_caterory, null);
				final TextView tv = (TextView) viewType.findViewById(R.id.kanjiLineMainCategoryTextView);
				tv.setText(wt.stringId);
				lviews.add(addTypePosition, viewType);
			}
		}

		this.views = lviews;
	}

	private View makeViewCharCatalog(KanjiData.CharacterCatalog cc, Context context,
			LayoutInflater inflator, boolean disabled) {
		final View viewChar = inflator.inflate(R.layout.kanji_line_item_category, null);
		final CheckBox cb = (CheckBox) viewChar.findViewById(R.id.kanjiLineItemCheckBox);
		// tag for KanjiArrayAdapter.onCheckedChanged()
		cb.setTag(cc);
		cb.setOnCheckedChangeListener(this);
		cb.setChecked(cc.isSelected());
		final TextView tv = (TextView) viewChar.findViewById(R.id.kanjiLineItemTextView);
		if (cc.level > 0) {
			tv.setText(context.getString(cc.stringId, cc.level));
		} else {
			tv.setText(cc.stringId);
		}
		viewChar.setTag(cc);
		if (disabled) {
			cb.setEnabled(false);
			tv.setEnabled(false);
			viewChar.setEnabled(false);
		}
		return viewChar;
	}

	private static View makeViewCategory(KanjiData.WritingCategory wc, LayoutInflater inflator,
			boolean disabled) {
		final View viewCat = inflator.inflate(R.layout.kanji_line_sub_category, null);
		final TextView tv = (TextView) viewCat.findViewById(R.id.kanjiLineSubCategoryTextView);
		tv.setText(wc.stringId);
		// tag use by the activity which use this ArrayAdapter
		// DrillMenuActivity & LearnWritingMenuActivity
		viewCat.setTag(wc);
		if (disabled) {
			tv.setEnabled(false);
			viewCat.setEnabled(false);
		}

		return viewCat;
	}

	@Override
	public int getCount() {
		return this.views.size();
	}

	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		return this.views.get(position);
	}

	/**
	 * Change the selection of KanjiData.CharacterCatalog. {@inheritDoc}
	 */
	public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
		KanjiData.CharacterCatalog cc = (KanjiData.CharacterCatalog) buttonView.getTag();
		cc.setSelection(isChecked);
	}

}
