/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.kanji;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;

import org.vono.narau.Common;
import org.vono.narau.R;
import org.vono.narau.preferences.Preferences;

/**
 * I don't like the name of this class, but I didn't find a better one.
 * 
 */
public class KanjiData {

	public static final String[] HIRAGANA_BASE = { "あ", "い", "う", "え", "お", "か", "き", "く", "け",
			"こ", "さ", "し", "す", "せ", "そ", "た", "ち", "つ", "て", "と", "な", "に", "ぬ", "ね", "の", "は",
			"ひ", "ふ", "へ", "ほ", "ま", "み", "む", "め", "も", "や", "ゆ", "よ", "ら", "り", "る", "れ", "ろ",
			"わ", "を", "ん" };
	public static final String[] HIRAGANA_SUP = { "が", "ぎ", "ぐ", "げ", "ご", "ざ", "じ", "ず", "ぜ", "ぞ",
			"だ", "ぢ", "づ", "で", "ど", "ば", "び", "ぶ", "べ", "ぼ", "ぱ", "ぴ", "ぷ", "ぺ", "ぽ" };
	public static final String[] HIRAGANA_DEPRECATED = { "ゐ", "ゑ" };
	public static final String[] HIRAGANA_COMBINE = { "きゃ", "ぎゃ", "きゅ", "ぎゅ", "きょ", "ぎょ", "しゃ",
			"じゃ", "しゅ", "じゅ", "しょ", "じょ", "ちゃ", "ぢゃ", "ちゅ", "ぢゅ", "ちょ", "ぢょ", "にゃ", "にゅ", "にょ",
			"ひゃ", "びゃ", "ぴゃ", "ひゅ", "びゅ", "ぴゅ", "ひょ", "びょ", "ぴょ", "みゃ", "みゅ", "みょ", "りゃ", "りゅ",
			"りょ", };

	public static final String[] KATAKANA_BASE = { "ア", "イ", "ウ", "エ", "オ", "カ", "キ", "ク", "ケ",
			"コ", "サ", "シ", "ス", "セ", "ソ", "タ", "チ", "ツ", "テ", "ト", "ナ", "ニ", "ヌ", "ネ", "ノ", "ハ",
			"ヒ", "フ", "ヘ", "ホ", "マ", "ミ", "ム", "メ", "モ", "ヤ", "ユ", "ヨ", "ラ", "リ", "ル", "レ", "ロ",
			"ワ", "ヲ", "ン" };
	public static final String[] KATAKANA_SUP = { "ガ", "ギ", "グ", "ゲ", "ゴ", "ザ", "ジ", "ズ", "ゼ", "ゾ",
			"ダ", "ヂ", "ヅ", "デ", "ド", "バ", "ビ", "ブ", "ベ", "ボ", "パ", "ピ", "プ", "ペ", "ポ" };
	public static final String[] KATAKANA_DEPRECATED = { "ヰ", "ヱ" };
	public static final String[] KATAKANA_COMBINE = { "キャ", "ギャ", "キュ", "ギュ", "キョ", "ギョ", "シャ",
			"ジャ", "シュ", "ジュ", "ショ", "ジョ", "チャ", "ヂャ", "チュ", "ヂュ", "チョ", "ヂョ", "ニャ", "ニュ", "ニョ",
			"ヒャ", "ビャ", "ピャ", "ヒュ", "ビュ", "ピュ", "ヒョ", "ビョ", "ピョ", "ミャ", "ミュ", "ミョ", "リャ", "リュ",
			"リョ", };

	public static final String[][] HIRAGANA_CHART = { { "Gojūon" }, { "あ", "い", "う", "え", "お" },
			{ "か", "き", "く", "け", "こ" }, { "さ", "し", "す", "せ", "そ" }, { "た", "ち", "つ", "て", "と" },
			{ "な", "に", "ぬ", "ね", "の" }, { "は", "ひ", "ふ", "へ", "ほ" }, { "ま", "み", "む", "め", "も" },
			{ "や", "", "ゆ", "", "よ" }, { "ら", "り", "る", "れ", "ろ" }, { "わ", "", "", "", "を" },
			{ "ん", "" }, { "Dakuon" }, { "が", "ぎ", "ぐ", "げ", "ご" }, { "ざ", "じ", "ず", "ぜ", "ぞ" },
			{ "だ", "ぢ", "づ", "で", "ど" }, { "ば", "び", "ぶ", "べ", "ぼ" }, { "Handakuon" },
			{ "ぱ", "ぴ", "ぷ", "ぺ", "ぽ" }, { "Yōon" }, { "きゃ", "", "きゅ", "", "きょ" },
			{ "しゃ", "", "しゅ", "", "しょ" }, { "ちゃ", "", "ちゅ", "", "ちょ" },
			{ "にゃ", "", "にゅ", "", "にょ" }, { "ひゃ", "", "ひゅ", "", "ひょ" },
			{ "みゃ", "", "みゅ", "", "みょ" }, { "りゃ", "", "りゅ", "", "りょ" },
			{ "ぎゃ", "", "ぎゅ", "", "ぎょ" }, { "じゃ", "", "じゅ", "", "じょ" },
			{ "ぢゃ", "", "ぢゅ", "", "ぢょ" }, { "びゃ", "", "びゅ", "", "びょ" },
			{ "ぴゃ", "", "ぴゅ", "", "ぴょ" } };

	public static final String[][] KATAKANA_CHART = { { "Gojūon" }, { "ア", "イ", "ウ", "エ", "オ" },
			{ "カ", "キ", "ク", "ケ", "コ" }, { "サ", "シ", "ス", "セ", "ソ" }, { "タ", "チ", "ツ", "テ", "ト" },
			{ "ナ", "ニ", "ヌ", "ネ", "ノ" }, { "ハ", "ヒ", "フ", "ヘ", "ホ" }, { "マ", "ミ", "ム", "メ", "モ" },
			{ "ヤ", "", "ユ", "", "ヨ" }, { "ラ", "リ", "ル", "レ", "ロ" }, { "ワ", "", "", "", "ヲ", },
			{ "ン", "" }, { "Dakuon" }, { "ガ", "ギ", "グ", "ゲ", "ゴ" }, { "ザ", "ジ", "ズ", "ゼ", "ゾ" },
			{ "ダ", "ヂ", "ヅ", "デ", "ド" }, { "バ", "ビ", "ブ", "ベ", "ボ" }, { "Handakuon" },
			{ "パ", "ピ", "プ", "ペ", "ポ" }, { "Yōon" }, { "キャ", "", "キュ", "", "キョ" },
			{ "シャ", "", "シュ", "", "ショ" }, { "チャ", "", "チュ", "", "チョ" },
			{ "ニャ", "", "ニュ", "", "ニョ" }, { "ヒャ", "", "ヒュ", "", "ヒョ" },
			{ "ミャ", "", "ミュ", "", "ミョ" }, { "リャ", "", "リュ", "", "リョ" },
			{ "ギャ", "", "ギュ", "", "ギョ" }, { "ジャ", "", "ジュ", "", "ジョ" },
			{ "ヂャ", "", "ヂュ", "", "ヂョ" }, { "ビャ", "", "ビュ", "", "ビョ" },
			{ "ピャ", "", "ピュ", "", "ピョ" } };

	public static int KANA = 1 << 0;
	public static int KANJI = 1 << 1;
	public static int KANA_CHAR = KANA | 1 << 2;
	public static int KANA_STRING = KANA | 1 << 3;
	public static int KANA_CHART = KANA | 1 << 4;
	public static int KANJI_JLPT = KANJI | 1 << 5;
	public static int KANJI_GRADE = KANJI | 1 << 6;
	public static int KANJI_NAME = KANJI | 1 << 7;
	public static int KANJI_OTHER = KANJI | 1 << 8;

	/**
	 * Kana Hiragana: Base/Sup/Combine/Obsolete<br/>
	 * Kana Katakana: Base/Sup/Combine/Obsolete<br/>
	 * Kanji JLPT: 1, 2, 3, 4<br/>
	 * Kanji school: 1, 2, 3, 4, 5, 6, 8, 9, 10<br/>
	 * Kanji other
	 */
	public static class CharacterCatalog {
		/**
		 * KANA|KANJI|CHAR|STRING|CHART
		 */
		public final int typeMask;
		/**
		 * The database need to display this category. See Common.isDBAvail()
		 */
		public final int dbMask;
		public final int stringId;
		public final boolean visible;
		private final String prefName;
		public final String[] charCatalog;
		public final String[][] chart;

		public int level;
		private boolean selected;

		/**
		 * Where clause
		 */
		public String selection;
		/**
		 * Where clause parameters values.
		 */
		public ArrayList<String> selectionArgs;

		private CharacterCatalog(int typeMask, int dbMask, int stringId, String prefName,
				boolean visible, Object[] data) {
			this.typeMask = typeMask;
			this.dbMask = dbMask;
			this.stringId = stringId;
			this.prefName = prefName;
			this.visible = visible;

			if (data instanceof String[]) {
				this.charCatalog = (String[]) data;
				this.chart = null;
			} else {
				this.charCatalog = null;
				this.chart = (String[][]) data;
			}

			this.level = -1;
			this.selection = null;
			this.selectionArgs = null;

			this.selected = Preferences.getSP().getBoolean(prefName, false);
		}

		public boolean isSelected() {
			return this.selected;
		}

		public void setSelection(boolean value) {
			this.selected = value;
			Preferences.getSP().edit().putBoolean(prefName, value).commit();
		}

		@Override
		public String toString() {
			return CharacterCatalog.class.getSimpleName() + ": " + this.prefName;
		}
	}

	/**
	 * 
	 * Hiragana/Katakana/Kanji JLPT/Kanji grade...
	 * 
	 */
	public static class WritingCategory implements Serializable {
		/**
		 * serialVersionUID, writeObject() and readObject() are needed to pass
		 * as a Serializable class. Used as an "extra" parameter from
		 * activities.
		 */
		private static final long serialVersionUID = -1786260883906604134L;
		public int stringId;
		public ArrayList<KanjiData.CharacterCatalog> catalogList;
		public Common.KanaType type;

		public WritingCategory(int stringId, Common.KanaType type) {
			this.stringId = stringId;
			this.type = type;
			this.catalogList = new ArrayList<KanjiData.CharacterCatalog>();
		}

		/**
		 * Serializable method.
		 * 
		 * @param out
		 * @throws IOException
		 */
		private void writeObject(java.io.ObjectOutputStream out) throws IOException {
			out.writeInt(this.stringId);
			out.writeInt(this.type.ordinal());
		}

		/**
		 * Serializable method.
		 * 
		 * @param in
		 * @throws IOException
		 * @throws ClassNotFoundException
		 */
		private void readObject(java.io.ObjectInputStream in) throws IOException,
				ClassNotFoundException {
			this.stringId = in.readInt();
			final int ordinal = in.readInt();
			Common.KanaType[] vals = Common.KanaType.values();
			for (Common.KanaType val : vals) {
				if (val.ordinal() == ordinal) {
					this.type = val;
					break;
				}
			}

			for (WritingType wt : writingType) {
				ArrayList<KanjiData.WritingCategory> alWC = wt.writingCategories;
				for (KanjiData.WritingCategory wc : alWC) {
					if (wc.stringId == this.stringId) {
						this.catalogList = wc.catalogList;
						break;
					}
				}
			}

		}
	}

	/**
	 * Kanji/Kana
	 */
	public static class WritingType {
		public final int stringId;
		public final ArrayList<KanjiData.WritingCategory> writingCategories;

		public WritingType(int stringId) {
			this.stringId = stringId;
			this.writingCategories = new ArrayList<KanjiData.WritingCategory>();
		}
	}

	public static final WritingType[] writingType = new WritingType[2];
	public static final int NUM_UI_LINES = 31;

	/**
	 * Build the selection (where clause) and args, for the query in the
	 * kanji.db.
	 * 
	 * @see KanjiDB
	 * @param cc
	 */
	private static void makeKanaSQLSelection(CharacterCatalog cc) {
		final String[] catalog = cc.charCatalog;
		final int catalogLength = catalog.length;
		final StringBuilder sbSelection = new StringBuilder(catalogLength
				* (KanjiDB.WHERE_CODEPOINT.length() + 4));
		final ArrayList<String> alSelectionArgs = new ArrayList<String>(catalogLength);
		sbSelection.append(KanjiDB.WHERE_CODEPOINT);
		alSelectionArgs.add(Integer.toString(catalog[0].codePointAt(0)));
		for (int i = 1; i < catalogLength; i++) {
			sbSelection.append(" or ");
			sbSelection.append(KanjiDB.WHERE_CODEPOINT);
			alSelectionArgs.add(Integer.toString(catalog[i].codePointAt(0)));
		}

		cc.selection = sbSelection.toString();
		cc.selectionArgs = alSelectionArgs;
	}

	static {

		// Kana
		WritingType wt = new WritingType(R.string.kana);
		KanjiData.writingType[0] = wt;
		// Kana: Hiragana
		WritingCategory wc = new WritingCategory(R.string.hiragana, Common.KanaType.HIRAGANA);
		wt.writingCategories.add(wc);
		CharacterCatalog cc = new CharacterCatalog(KANA_CHAR, Common.DB_MASK_KANJI, R.string.base,
				"hiragana.base", true, HIRAGANA_BASE);
		KanjiData.makeKanaSQLSelection(cc);
		wc.catalogList.add(cc);
		cc = new CharacterCatalog(KANA_CHAR, Common.DB_MASK_KANJI, R.string.supplementary,
				"hiragana.supplementary", true, HIRAGANA_SUP);
		KanjiData.makeKanaSQLSelection(cc);
		wc.catalogList.add(cc);
		cc = new CharacterCatalog(KANA_STRING, Common.DB_MASK_KANJI, R.string.combine,
				"hiragana.combine", true, HIRAGANA_COMBINE);
		wc.catalogList.add(cc);
		cc = new CharacterCatalog(KANA_CHAR, Common.DB_MASK_KANJI, R.string.deprecated,
				"hiragana.deprecated", true, HIRAGANA_DEPRECATED);
		KanjiData.makeKanaSQLSelection(cc);
		wc.catalogList.add(cc);
		cc = new CharacterCatalog(KANA_CHART, 0, R.string.learnHiraganaChart, "hiragana.chart",
				false, HIRAGANA_CHART);
		cc.selected = true;
		wc.catalogList.add(cc);

		// Kana: Katakana
		wc = new WritingCategory(R.string.katakana, Common.KanaType.KATAKANA);
		wt.writingCategories.add(wc);
		cc = new CharacterCatalog(KANA_CHAR, Common.DB_MASK_KANJI, R.string.base, "katakana.base",
				true, KATAKANA_BASE);
		KanjiData.makeKanaSQLSelection(cc);
		wc.catalogList.add(cc);
		cc = new CharacterCatalog(KANA_CHAR, Common.DB_MASK_KANJI, R.string.supplementary,
				"katakana.supplementary", true, KATAKANA_SUP);
		KanjiData.makeKanaSQLSelection(cc);
		wc.catalogList.add(cc);
		cc = new CharacterCatalog(KANA_STRING, Common.DB_MASK_KANJI, R.string.combine,
				"katakana.combine", true, KATAKANA_COMBINE);
		wc.catalogList.add(cc);
		cc = new CharacterCatalog(KANA_CHAR, Common.DB_MASK_KANJI, R.string.deprecated,
				"katakana.deprecated", true, KATAKANA_DEPRECATED);
		KanjiData.makeKanaSQLSelection(cc);
		wc.catalogList.add(cc);
		cc = new CharacterCatalog(KANA_CHART, 0, R.string.learnKatakanaChart, "katakana.chart",
				false, KATAKANA_CHART);
		cc.selected = true;
		wc.catalogList.add(cc);

		// Kanji
		wt = new WritingType(R.string.kanji);
		KanjiData.writingType[1] = wt;
		// Kanji JLPT
		wc = new WritingCategory(R.string.jlpt, Common.KanaType.KANJI);
		wt.writingCategories.add(wc);
		for (int i = 1; i <= 4; i++) {
			cc = new CharacterCatalog(KANJI_JLPT, Common.DB_MASK_KANJI, R.string.level, "kanji.jlpt."
					+ i, true, null);
			cc.level = i;
			cc.selection = "jlpt=?";
			cc.selectionArgs = new ArrayList<String>(1);
			cc.selectionArgs.add(Integer.toString(i));
			wc.catalogList.add(cc);
		}
		// Kanji Grade
		// The kanji grade level. 1 through 6 indicates a Kyouiku kanji
		// and the grade in which the kanji is taught in Japanese schools.
		// 8 indicates it is one of the remaining Jouyou Kanji to be learned
		// in junior high school, and 9 or 10 indicates it is a Jinmeiyou (for
		// use in names) kanji.
		wc = new WritingCategory(R.string.kanjiKyouiku, Common.KanaType.KANJI);
		wt.writingCategories.add(wc);
		for (int i = 1; i <= 6; i++) {
			cc = new CharacterCatalog(KANJI_GRADE, Common.DB_MASK_KANJI,
					R.string.kanjiKyouikuGrade, "kanji.kyouiku." + i, true, null);
			cc.level = i;
			cc.selection = "grade=?";
			cc.selectionArgs = new ArrayList<String>(1);
			cc.selectionArgs.add(Integer.toString(i));
			wc.catalogList.add(cc);
		}
		// Jouyou Kanji
		wc = new WritingCategory(R.string.kanjiJouyou, Common.KanaType.KANJI);
		wt.writingCategories.add(wc);
		cc = new CharacterCatalog(KANJI_GRADE, Common.DB_MASK_KANJI, R.string.kanjiJouyouGrade,
				"kanji.jouyou", true, null);
		cc.selection = "grade=?";
		cc.selectionArgs = new ArrayList<String>(1);
		cc.selectionArgs.add("8");
		wc.catalogList.add(cc);
		// Jinmeiyou
		wc = new WritingCategory(R.string.kanjiJinmeiyou, Common.KanaType.KANJI);
		wt.writingCategories.add(wc);
		cc = new CharacterCatalog(KANJI_NAME, Common.DB_MASK_KANJI, R.string.kanjiJinmeiyouGrade,
				"kanji.jinmeiyou.1", true, null);
		cc.level = 1;
		cc.selection = "grade=?";
		cc.selectionArgs = new ArrayList<String>(1);
		cc.selectionArgs.add("9");
		wc.catalogList.add(cc);
		cc = new CharacterCatalog(KANJI_NAME, Common.DB_MASK_KANJI, R.string.kanjiJinmeiyouGrade,
				"kanji.jinmeiyou.2", true, null);
		cc.level = 2;
		cc.selection = "grade=?";
		cc.selectionArgs = new ArrayList<String>(1);
		cc.selectionArgs.add("10");
		wc.catalogList.add(cc);

		// Kanji other
		wc = new WritingCategory(R.string.kanjiOther, Common.KanaType.KANJI);
		wt.writingCategories.add(wc);
		// No category
		cc = new CharacterCatalog(KANJI_OTHER, Common.DB_MASK_KANJI, R.string.kanjiNoCategory,
				"kanji.no_category", true, null);
		cc.selection = "jlpt is null and grade is null";
		cc.selectionArgs = new ArrayList<String>(0);
		wc.catalogList.add(cc);
		// All available kanji
		cc = new CharacterCatalog(KANJI_OTHER, Common.DB_MASK_KANJI, R.string.kanjiAll, "kanji.all",
				true, null);
		cc.selection = Common.EMPTY_STRING;
		cc.selectionArgs = new ArrayList<String>(0);
		wc.catalogList.add(cc);
	}

	public static boolean isHiragana(int codepoint) {
		return (0x3041 <= codepoint && codepoint <= 0x309F);
	}

	public static boolean isKatakana(int codepoint) {
		return (0x30A1 <= codepoint && codepoint <= 0x30FA);
	}
}
