/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.learn.reading;

import org.vono.narau.Common;
import org.vono.narau.kanji.KanjiArrayAdapter;
import org.vono.narau.kanji.KanjiData;

import android.app.Activity;
import android.content.Context;
import android.graphics.Color;
import android.os.Bundle;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ScrollView;
import android.widget.TableLayout;
import android.widget.TableRow;
import android.widget.TextView;

/**
 * 
 * Display Hiragana/Katakana charts.<br/>
 * I can't make the display as expected on this Cupcake version. I don't want to
 * spend more time on this.
 */
public class KanaChartActivity extends Activity {

	static final int TYPE_MASK = KanjiData.KANA_CHART;

	private static final ScrollView[] scrollViews = new ScrollView[Common.KanaType.values().length];

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		Bundle extra = this.getIntent().getExtras();
		if (null == extra) {
			throw new RuntimeException("don't know what to display");
		}

		KanjiData.WritingCategory wc = (KanjiData.WritingCategory) extra
				.getSerializable(KanjiArrayAdapter.ITEM_CATEGORY);

		final ScrollView view = this.buildTable(wc);
		if (null != view) {
			super.setContentView(view);
		} else {
			super.finish();
		}
	}

	private ScrollView buildTable(KanjiData.WritingCategory wc) {
		ScrollView scrollView = null;

		scrollView = KanaChartActivity.scrollViews[wc.type.ordinal()];

		if (null == scrollView) {
			Context context = super.getApplicationContext();
			TableLayout tableLayout = new TableLayout(context);

			String[][] kanaChartArray = null;
			switch (wc.type) {
			case HIRAGANA:
				kanaChartArray = KanjiData.HIRAGANA_CHART;
				break;
			case KATAKANA:
				kanaChartArray = KanjiData.KATAKANA_CHART;
				break;
			default:
				return null;
			}

			final int count = kanaChartArray.length;
			for (int i = 0; i < count; i++) {
				final String[] line = kanaChartArray[i];
				final int lineLen = line.length;
				final TableRow row = new TableRow(context);

				if (lineLen == 1) {
					final TextView tv = new TextView(context);
					tv.setText(line[0]);
					tv.setGravity(Gravity.CENTER);
					tv.setPadding(0, 5, 0, 0);
					tv.setTextColor(Color.BLACK);
					row.addView(tv);
					row.setBackgroundColor(Color.WHITE);
					tableLayout.addView(row);
				} else {
					for (int j = 0; j < lineLen; j++) {
						final String kana = line[j];
						if (kana.length() > 0) {

							final String romanji = Common.convertKana(line[j], wc.type,
									Common.KanaType.ROMANJI);

							final TextView tv = new TextView(context);
							tv.setTextSize(30);
							tv.setGravity(Gravity.CENTER);
							tv.setText(kana + "\n" + romanji);
							row.addView(tv);
						} else {
							View v = new View(context);
							row.addView(v);
						}
					}
					tableLayout.addView(row);
					// setDividerDrawable() is API level 11
					// so add a fake row
					final TableRow rowSep = new TableRow(context);
					rowSep.setBackgroundColor(Color.WHITE);
					rowSep.setPadding(0, 0, 0, 2);
					tableLayout.addView(rowSep);

				}
			}

			scrollView = new ScrollView(context);
			scrollView.addView(tableLayout);
			KanaChartActivity.scrollViews[wc.type.ordinal()] = scrollView;
		} else {
			((ViewGroup) scrollView.getParent()).removeView(scrollView);
		}
		return scrollView;
	}

}
