/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.learn.writing;

import java.util.ArrayList;

import org.vono.narau.Common;
import org.vono.narau.R;
import org.vono.narau.kanji.Kanji;
import org.vono.narau.kanji.KanjiArrayAdapter;
import org.vono.narau.kanji.KanjiDB;
import org.vono.narau.kanji.KanjiData;
import org.vono.narau.kanji.KanjiListSelectActivity;
import org.vono.narau.kanji.KanjiView;
import org.vono.narau.kanji.KanjiViewDialogSeekBar;
import org.vono.narau.preferences.Preferences;

import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.support.v4.app.FragmentActivity;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;

/**
 * 
 *
 */
public class LearnWritingActivity extends FragmentActivity implements KanjiViewDialogSeekBar.OnDismissListener,
		OnClickListener {
	private static final String TAG = LearnWritingActivity.class.getSimpleName();
	private static final String CURRENT_POSITION = "CP";
	static final int TYPE_MASK = KanjiData.KANA_CHAR | KanjiData.KANJI_JLPT | KanjiData.KANJI_GRADE
			| KanjiData.KANJI_NAME | KanjiData.KANJI_OTHER;
	private static final int ALL_KANJI_CODE = 456;

	private static ArrayList<Kanji> listKanji = null;
	private static ActivityHandler handler = null;
	private static LoadKanjiListThread thread = null;

	private KanjiView kanjiView;
	private int currentPosition;
	private ProgressDialog progressDialog;
	private Button buttonPrevious;
	private Button buttonNext;

	public LearnWritingActivity() {
		if (null == LearnWritingActivity.listKanji) {
			LearnWritingActivity.listKanji = new ArrayList<Kanji>();
		}

		this.currentPosition = 0;
		this.progressDialog = null;
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		Bundle extra = this.getIntent().getExtras();
		if (null == extra) {
			throw new RuntimeException("don't know what to display");
		}

		if (null == LearnWritingActivity.handler) {
			LearnWritingActivity.handler = new ActivityHandler(this);
		} else {
			LearnWritingActivity.handler.activity = this;
		}

		KanjiData.WritingCategory wc = (KanjiData.WritingCategory) extra
				.getSerializable(KanjiArrayAdapter.ITEM_CATEGORY);

		final ArrayList<KanjiData.CharacterCatalog> alCC = wc.catalogList;
		final int numCC = alCC.size();

		StringBuilder sbSelection = new StringBuilder();
		ArrayList<String> selectionArgsList = new ArrayList<String>(numCC);

		boolean haveCatalogSelected = false;

		for (int i = 0; i < numCC; i++) {
			final KanjiData.CharacterCatalog cc = alCC.get(i);
			if (cc.isSelected() && (cc.typeMask & TYPE_MASK) == cc.typeMask) {
				haveCatalogSelected = true;
				if (sbSelection.length() > 0) {
					sbSelection.append(" or ");
				}
				sbSelection.append(cc.selection);
				selectionArgsList.addAll(cc.selectionArgs);
			}
		}

		if (haveCatalogSelected) {

			super.setContentView(R.layout.learn_writing);
			this.kanjiView = (KanjiView) super.findViewById(R.id.learnWritingKanjiView);
			this.kanjiView.setShowGrid(Preferences.getBoolean(R.string.prefNameKanjiShowGrid));
			this.kanjiView.setShowStrokesNumbers(Preferences.getBoolean(R.string.prefNameKanjiShowStrokesNumber));
			this.buttonNext = (Button) super.findViewById(R.id.learnWritingButtonNext);
			this.buttonPrevious = (Button) super.findViewById(R.id.learnWritingButtonPrevious);
			this.buttonNext.setOnClickListener(this);
			this.buttonPrevious.setOnClickListener(this);
			Button button = (Button) super.findViewById(R.id.learnWritingButtonClear);
			button.setOnClickListener(this);
			button = (Button) super.findViewById(R.id.learnWritingButtonPlay);
			button.setOnClickListener(this);

			String selection = sbSelection.toString();
			String[] selectionArgs = selectionArgsList.toArray(new String[0]);

			if (null == savedInstanceState) {
				this.progressDialog = ProgressDialog.show(this, Common.EMPTY_STRING,
						Common.getString(R.string.loading), true);
				LearnWritingActivity.thread = new LoadKanjiListThread(selection, selectionArgs);
				thread.start();
			} else {
				this.currentPosition = savedInstanceState.getInt(CURRENT_POSITION);
				this.updateCurrentPosition();
			}
		} else {
			Log.e(TAG, "No kanji to display");
			this.finish();
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.learn_writing, menu);

		MenuItem item = menu.findItem(R.id.menuLearnWritingShowGrid);
		if (Preferences.getBoolean(R.string.prefNameKanjiShowGrid)) {
			item.setTitle(R.string.prefKanjiHideGrid);
		} else {
			item.setTitle(R.string.prefKanjiShowGrid);
		}
		item = menu.findItem(R.id.menuLearnWritingShowStrokeNumber);
		if (Preferences.getBoolean(R.string.prefNameKanjiShowStrokesNumber)) {
			item.setTitle(R.string.prefKanjiHideStrokeNumber);
		} else {
			item.setTitle(R.string.prefKanjiShowStrokeNumber);
		}

		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.menuLearnWritingDrawSpeed:
			KanjiViewDialogSeekBar dialogFragment = new KanjiViewDialogSeekBar();
			Bundle args = new Bundle();
			args.putInt(KanjiViewDialogSeekBar.ID_PREF_NAME, R.string.prefNameKanjiDrawSpeed);
			args.putInt(KanjiViewDialogSeekBar.ID_MIN, R.string.prefValueKanjiStrokeMinDrawSpeed);
			args.putInt(KanjiViewDialogSeekBar.ID_MAX, R.string.prefValueKanjiStrokeMaxDrawSpeed);
			args.putInt(KanjiViewDialogSeekBar.ID_DIVIDER, R.string.prefValueKanjiStrokeDividerDrawSpeed);
			args.putInt(KanjiViewDialogSeekBar.ID_TITLE, R.string.prefKanjiDrawingSpeed);
			dialogFragment.setArguments(args);
			dialogFragment.show(super.getSupportFragmentManager(), "Speed");
			break;
		case R.id.menuLearnWritingStrokeWidth:
			dialogFragment = new KanjiViewDialogSeekBar();
			args = new Bundle();
			args.putInt(KanjiViewDialogSeekBar.ID_PREF_NAME, R.string.prefNameKanjiStrokesWidth);
			args.putInt(KanjiViewDialogSeekBar.ID_MIN, R.string.prefValueKanjiStrokeMinWidth);
			args.putInt(KanjiViewDialogSeekBar.ID_MAX, R.string.prefValueKanjiStrokeMaxWidth);
			args.putInt(KanjiViewDialogSeekBar.ID_DIVIDER, R.string.prefValueKanjiStrokeDividerWidth);
			args.putInt(KanjiViewDialogSeekBar.ID_TITLE, R.string.prefKanjiStrokeWidth);
			dialogFragment.setArguments(args);
			dialogFragment.setOnDismissListener(this);
			dialogFragment.show(super.getSupportFragmentManager(), "Width");
			break;
		case R.id.menuLearnWritingShowGrid:
			final boolean showGrid = !Preferences.getBoolean(R.string.prefNameKanjiShowGrid);
			Preferences.setBoolean(R.string.prefNameKanjiShowGrid, showGrid);
			this.kanjiView.setShowGrid(showGrid);
			if (showGrid) {
				item.setTitle(R.string.prefKanjiHideGrid);
			} else {
				item.setTitle(R.string.prefKanjiShowGrid);
			}
			break;
		case R.id.menuLearnWritingShowStrokeNumber:
			final boolean showNumbers = !Preferences.getBoolean(R.string.prefNameKanjiShowStrokesNumber);
			Preferences.setBoolean(R.string.prefNameKanjiShowStrokesNumber, showNumbers);
			this.kanjiView.setShowStrokesNumbers(showNumbers);
			if (showNumbers) {
				item.setTitle(R.string.prefKanjiHideStrokeNumber);
			} else {
				item.setTitle(R.string.prefKanjiShowStrokeNumber);
			}
			break;
		case R.id.menuLearnWritingShowAll:
			this.showAllKanji();
			break;
		default:
			return super.onOptionsItemSelected(item);
		}
		return true;

	}

	@Override
	public void onDestroy() {
		super.onDestroy();

		if (null != this.kanjiView) {
			this.kanjiView.setKanji(null);
			this.kanjiView = null;
		}

		if (null != this.progressDialog) {
			this.progressDialog.dismiss();
			this.progressDialog = null;
		}
		if (null != LearnWritingActivity.handler) {
			LearnWritingActivity.handler.activity = null;
		}
	}

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		boolean ret = this.kanjiView.onKeyDown(keyCode, event);

		if (!ret && KeyEvent.KEYCODE_BACK == keyCode) {
			if (null != LearnWritingActivity.thread) {
				LearnWritingActivity.thread.setPrematureEnd();
				while (LearnWritingActivity.thread.isAlive()) {
					try {
						Thread.sleep(50);
					} catch (InterruptedException ie) {
					}
				}
				LearnWritingActivity.thread = null;
			}
			LearnWritingActivity.handler = null;
			LearnWritingActivity.listKanji.clear();
			LearnWritingActivity.listKanji = null;
			super.finish();

			System.runFinalization();
			System.gc();
			ret = true;
		} else if (!ret) {
			ret = super.onKeyDown(keyCode, event);
		}
		return ret;
	}

	/**
	 * <p>
	 * Handle device rotation.
	 * </p>
	 * {@inheritDoc}
	 * 
	 */
	@Override
	public void onSaveInstanceState(Bundle savedInstanceState) {
		savedInstanceState.putInt(CURRENT_POSITION, this.currentPosition);
	}

	/**
	 * Show the list of kanjis, and get the result in onActivityResult()
	 */
	private void showAllKanji() {
		final ArrayList<Kanji> lListKanji = LearnWritingActivity.listKanji;
		final int numKanji = lListKanji.size();
		final int[] arrayCodepoint = new int[numKanji];

		for (int i = 0; i < numKanji; i++) {
			arrayCodepoint[i] = lListKanji.get(i).codepoint;
		}
		Intent intent = new Intent(this, KanjiListSelectActivity.class);
		intent.putExtra(KanjiListSelectActivity.TAG, arrayCodepoint);
		// intent.put
		super.startActivityForResult(intent, LearnWritingActivity.ALL_KANJI_CODE);
	}

	/**
	 * Result from the all kanjis activity
	 */
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		if (LearnWritingActivity.ALL_KANJI_CODE == requestCode && resultCode >= 0) {
			this.currentPosition = resultCode;

			final int numKanji = LearnWritingActivity.listKanji.size();
			if (this.currentPosition >= numKanji - 1) {
				this.currentPosition = numKanji - 1;
			}

			this.updateCurrentPosition();
		}
	}

	public void onClick(View v) {
		switch (v.getId()) {
		case R.id.learnWritingButtonPrevious:
			this.previousKanji(v);
			break;
		case R.id.learnWritingButtonPlay:
			this.playKanji(v);
			break;
		case R.id.learnWritingButtonClear:
			this.clearUserDrawing(v);
			break;
		case R.id.learnWritingButtonNext:
			this.nextKanji(v);
			break;
		}

	}

	/**
	 * Draw the kanji (playing)
	 * 
	 * @param v
	 *            The button
	 */
	public void playKanji(View v) {
		this.kanjiView.playKanji();
	}

	/**
	 * Show next kanji.
	 * 
	 * @param v
	 *            The button
	 */
	public void nextKanji(View v) {

		if (this.currentPosition + 1 < LearnWritingActivity.listKanji.size()) {
			this.currentPosition++;
			this.updateCurrentPosition();
		}
	}

	/**
	 * Show previous kanji
	 * 
	 * @param v
	 *            The previous button
	 */
	public void previousKanji(View v) {

		if (0 < this.currentPosition) {
			this.currentPosition--;
			this.updateCurrentPosition();
		}
	}

	private void updateButtonVisibility() {
		final int numKanjis = LearnWritingActivity.listKanji.size();

		if (0 == this.currentPosition) {
			this.buttonPrevious.setVisibility(View.INVISIBLE);
			this.buttonNext.setVisibility(View.VISIBLE);
		} else if (this.currentPosition + 1 == numKanjis) {
			this.buttonPrevious.setVisibility(View.VISIBLE);
			this.buttonNext.setVisibility(View.INVISIBLE);
		} else {
			this.buttonPrevious.setVisibility(View.VISIBLE);
			this.buttonNext.setVisibility(View.VISIBLE);
		}
	}

	private void updateCurrentPosition() {
		final Kanji kanji = LearnWritingActivity.listKanji.get(this.currentPosition);
		KanjiDB.getKanji(kanji);
		this.updateKanji(kanji);
		this.updateButtonVisibility();
	}

	/**
	 * Clear the user's drawing part
	 * 
	 * @param v
	 *            The play button.
	 */
	public void clearUserDrawing(View v) {
		this.kanjiView.clearUserDrawing();
	}

	private void updateKanji(Kanji kanji) {
		if (null != kanji) {
			this.kanjiView.setKanji(kanji);

			TextView tv = (TextView) super.findViewById(R.id.learnWritingTextView);
			tv.setText(kanjiToString(kanji));
		}
	}

	private String kanjiToString(Kanji kanji) {
		StringBuilder sb = new StringBuilder();

		// TODO Improve this
		if (null != kanji) {

			String txtKanji = "" + (char) kanji.codepoint;
			sb.append((this.currentPosition + 1) + "/" + LearnWritingActivity.listKanji.size());
			sb.append("\t");
			sb.append(txtKanji);
			sb.append("\t(");
			if (null != kanji.meanings && !kanji.meanings.isEmpty()) {
				// FIXME: duplicate code in CharDrillActivity
				ArrayList<String> langs = Preferences.getLangsPriorities(Common.Database.kanji);
				String meaning = null;
				for (String lang : langs) {
					meaning = kanji.meanings.get(lang);
					if (null != meaning) {
						break;
					}
				}
				if (null == meaning) {
					meaning = super.getString(R.string.noDescriptionAvailForSelectLangs);
				}
				sb.append(meaning);
			} else {
				if (KanjiData.isHiragana(kanji.codepoint)) {
					sb.append(Common.convertKana(txtKanji, Common.KanaType.HIRAGANA, Common.KanaType.ROMANJI));
				} else if (KanjiData.isKatakana(kanji.codepoint)) {
					sb.append(Common.convertKana(txtKanji, Common.KanaType.KATAKANA, Common.KanaType.ROMANJI));
				} else {
					sb.append("?? TODO codepoint " + kanji.codepoint + " ??");
				}

			}
			sb.append(")");
		}
		return sb.toString();
	}

	/**
	 * Refesh the kanji view after we change the stroke width.
	 */
	public void onDialogDismiss(KanjiViewDialogSeekBar dialog) {
		this.kanjiView.invalidate();
	}

	private static class LoadKanjiListThread extends Thread {
		private final String selection;
		private final String[] selectionArgs;

		private boolean prematureEnd;

		public LoadKanjiListThread(String selection, String[] selectionArgs) {
			this.selection = selection;
			this.selectionArgs = selectionArgs;
			this.prematureEnd = false;
			super.setName("LoadKanjiListThread");
		}

		@Override
		public void run() {
			final KanjiDB.KanjiSVGIterator kanjiIteratorUnicode = KanjiDB.getKanjiSVGIterator(selection, selectionArgs);

			int codepoint = kanjiIteratorUnicode.first();
			while (codepoint != -1) {
				Kanji kanji = Kanji.getKanji(codepoint);
				LearnWritingActivity.listKanji.add(kanji);

				synchronized (this) {
					if (this.prematureEnd) {
						codepoint = -1;
					} else {
						codepoint = kanjiIteratorUnicode.next();
					}
				}
			}

			kanjiIteratorUnicode.close();
			ActivityHandler.sendLoadDone();
		}

		public synchronized void setPrematureEnd() {
			this.prematureEnd = true;
		}
	}

	private static class ActivityHandler extends Handler {
		static final int LOAD_DONE = 4073;

		LearnWritingActivity activity;

		public ActivityHandler(LearnWritingActivity activity) {
			this.activity = activity;
		}

		static void sendLoadDone() {
			LearnWritingActivity.handler.sendEmptyMessage(LOAD_DONE);
		}

		@Override
		public void handleMessage(Message msg) {
			if (LOAD_DONE == msg.what && null != activity) {
				if (null != activity.progressDialog) {
					activity.progressDialog.dismiss();
					activity.progressDialog = null;
				}
				activity.currentPosition = 0;
				activity.updateCurrentPosition();
			}
		}
	}

}
