/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.preferences;

import java.util.ArrayList;

import org.vono.narau.R;

import android.content.Context;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.TextView;

/**
 * 
 */
public class LangArrayAdapter extends ArrayAdapter<LangLineModel> implements
		OnCheckedChangeListener, OnClickListener {
	private static final String TAG = LangArrayAdapter.class.getSimpleName();

	private ArrayList<LangLineModel> list;
	private final Context context;

	public LangArrayAdapter(Context context, ArrayList<LangLineModel> list) {
		super(context, R.layout.preference_lang_line, list);
		this.context = context;
		this.list = list;
	}

	static class ViewHolder {
		protected TextView textView;
		protected CheckBox checkBox;
		protected LangLineModel model;
		protected int position;

		protected void setModel(LangLineModel model, int position) {
			this.model = model;
			this.checkBox.setTag(this);
			this.textView.setTag(this);
			this.textView.setText(model.toString());
			this.checkBox.setChecked(model.isSelected());
			this.position = position;
		}
	}

	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		View view = null;

		if (null == convertView) {
			LayoutInflater inflator = (LayoutInflater) this.context
					.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
			view = inflator.inflate(R.layout.preference_lang_line, null);
			final ViewHolder viewHolder = new ViewHolder();
			viewHolder.textView = (TextView) view.findViewById(R.id.langLineTextView);
			viewHolder.checkBox = (CheckBox) view.findViewById(R.id.langLineCheckBox);

			viewHolder.checkBox.setOnCheckedChangeListener(this);
			viewHolder.textView.setOnClickListener(this);
			view.setTag(viewHolder);

		} else {
			view = convertView;
		}

		ViewHolder holder = (ViewHolder) view.getTag();
		holder.setModel(this.list.get(position), position);

		return view;
	}

	/**
	 * A minimum of 1 lang must be selected
	 */
	private void checkNumLangSelected() {
		int numLangs = this.list.size();
		int countSelected = 0;
		ArrayList<LangLineModel> llmList = this.list;
		for (int i = 0; i < numLangs; i++) {
			LangLineModel llm = llmList.get(i);
			if (llm.isSelected()) {
				countSelected++;
			}
		}

		if (countSelected == 0) {
			llmList.get(0).setSelected(true);
			super.notifyDataSetChanged();
		}
	}

	public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
		ViewHolder holder = (ViewHolder) buttonView.getTag();
		holder.model.setSelected(isChecked);

		this.checkNumLangSelected();
	}

	/**
	 * Swap two languages if it is not the first language.
	 */
	public void onClick(View v) {
		ViewHolder holder = (ViewHolder) v.getTag();
		int currentPos = holder.position;

		if (currentPos > 0) {
			int prevPos = currentPos - 1;
			LangLineModel prevModel = this.list.get(prevPos);
			Log.i(TAG, "swap langs " + holder.model.toString() + " with " + prevModel.toString());
			// swap the position
			this.list.set(prevPos, holder.model);
			this.list.set(currentPos, prevModel);

			super.notifyDataSetChanged();
		}

	}

	public ArrayList<LangLineModel> getList() {
		return this.list;
	}

}
