/*
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau;

import java.util.Calendar;

import org.tmatesoft.sqljet.core.SqlJetException;
import org.tmatesoft.sqljet.core.SqlJetTransactionMode;
import org.tmatesoft.sqljet.core.table.ISqlJetTable;
import org.tmatesoft.sqljet.core.table.SqlJetDb;

public class Common {

	private static final String KEY_DB_VERSION = "version";
	private static final String KEY_DB_CREATION_DATE = "creation_date";
	private static final String VALUE_CREATION_DATE = String.format("%1$tY%1$tm%1$td", Calendar.getInstance());
	private static final String KEY_DB_CREATION_TIME = "creation_time";
	private static final String KEY_DB_LANG = "lang";

	private static final String TABLE_DBINFO = "db_info";
	private static final String CREATE_TABLE_DB_INFO = "CREATE TABLE " + TABLE_DBINFO
			+ " (key TEXT PRIMARY KEY, value_str TEXT, value_int INTEGER);";

	public static final String EMPTY_STRING = "";

	/**
	 * 1=current tag reader<br/>
	 * 2=tag name<br/>
	 * 3=line number
	 */
	public static final String ERR_UNEXPECTED_START_TAG = "%1$s: Unexpected start tag %2$s, at line %3$d";
	/**
	 * 1=current tag reader<br/>
	 * 2=tag name<br/>
	 * 3=line number
	 */
	public static final String ERR_UNEXPECTED_END_TAG = "%1$s: Unexpected end tag %2$s, at line %3$d";
	/**
	 * 1=current tag reader<br/>
	 * 2=event id<br/>
	 * 3=line number
	 */
	public static final String ERR_UNKNOWN_EVENTTYPE = "%1$s: Unknown event type %2$d, at line %3$d";
	/**
	 * 1=current tag reader<br/>
	 * 2=attribute name<br/>
	 * 3=line number
	 */
	public static final String ERR_UNKNOWN_ATTRIBUTE = "%1$s: Unknown attribute %2$s, at line %3$d";

	/**
	 * Default language is English.
	 */
	public static String LANG_DEFAULT = "eng";

	/**
	 * Japanese language code.
	 */
	public static String LANG_JAPANESE = "jpn";

	/**
	 * Based on http://www.sil.org/iso639-3/iso-639-3_20120228.tab
	 */
	public static final String[][] LANGS_ARRAY = new String[][] { { "aar", "aa", "Afar" },
			{ "abk", "ab", "Abkhazian" }, { "afr", "af", "Afrikaans" }, { "aka", "ak", "Akan" },
			{ "amh", "am", "Amharic" }, { "ara", "ar", "Arabic" }, { "arg", "an", "Aragonese" },
			{ "asm", "as", "Assamese" }, { "ava", "av", "Avaric" }, { "ave", "ae", "Avestan" },
			{ "aym", "ay", "Aymara" }, { "aze", "az", "Azerbaijani" }, { "bak", "ba", "Bashkir" },
			{ "bam", "bm", "Bambara" }, { "bel", "be", "Belarusian" }, { "ben", "bn", "Bengali" },
			{ "bis", "bi", "Bislama" }, { "bod", "bo", "Tibetan" }, { "bos", "bs", "Bosnian" },
			{ "bre", "br", "Breton" }, { "bul", "bg", "Bulgarian" }, { "cat", "ca", "Catalan" },
			{ "ces", "cs", "Czech" }, { "cha", "ch", "Chamorro" }, { "che", "ce", "Chechen" },
			{ "chu", "cu", "Church Slavic" }, { "chv", "cv", "Chuvash" }, { "cor", "kw", "Cornish" },
			{ "cos", "co", "Corsican" }, { "cre", "cr", "Cree" }, { "cym", "cy", "Welsh" }, { "dan", "da", "Danish" },
			{ "deu", "de", "German" }, { "div", "dv", "Dhivehi" }, { "dzo", "dz", "Dzongkha" },
			{ "ell", "el", "Modern Greek (1453-)" }, { "eng", "en", "English" }, { "epo", "eo", "Esperanto" },
			{ "est", "et", "Estonian" }, { "eus", "eu", "Basque" }, { "ewe", "ee", "Ewe" }, { "fao", "fo", "Faroese" },
			{ "fas", "fa", "Persian" }, { "fij", "fj", "Fijian" }, { "fin", "fi", "Finnish" },
			{ "fra", "fr", "French" }, { "fry", "fy", "Western Frisian" }, { "ful", "ff", "Fulah" },
			{ "gla", "gd", "Scottish Gaelic" }, { "gle", "ga", "Irish" }, { "glg", "gl", "Galician" },
			{ "glv", "gv", "Manx" }, { "grn", "gn", "Guarani" }, { "guj", "gu", "Gujarati" },
			{ "hat", "ht", "Haitian" }, { "hau", "ha", "Hausa" }, { "heb", "he", "Hebrew" }, { "her", "hz", "Herero" },
			{ "hin", "hi", "Hindi" }, { "hmo", "ho", "Hiri Motu" }, { "hrv", "hr", "Croatian" },
			{ "hun", "hu", "Hungarian" }, { "hye", "hy", "Armenian" }, { "ibo", "ig", "Igbo" }, { "ido", "io", "Ido" },
			{ "iii", "ii", "Sichuan Yi" }, { "iku", "iu", "Inuktitut" }, { "ile", "ie", "Interlingue" },
			{ "ina", "ia", "Interlingua (International Auxiliary Language Association)" },
			{ "ind", "id", "Indonesian" }, { "ipk", "ik", "Inupiaq" }, { "isl", "is", "Icelandic" },
			{ "ita", "it", "Italian" }, { "jav", "jv", "Javanese" }, { "jpn", "ja", "Japanese" },
			{ "kal", "kl", "Kalaallisut" }, { "kan", "kn", "Kannada" }, { "kas", "ks", "Kashmiri" },
			{ "kat", "ka", "Georgian" }, { "kau", "kr", "Kanuri" }, { "kaz", "kk", "Kazakh" },
			{ "khm", "km", "Central Khmer" }, { "kik", "ki", "Kikuyu" }, { "kin", "rw", "Kinyarwanda" },
			{ "kir", "ky", "Kirghiz" }, { "kom", "kv", "Komi" }, { "kon", "kg", "Kongo" }, { "kor", "ko", "Korean" },
			{ "kua", "kj", "Kuanyama" }, { "kur", "ku", "Kurdish" }, { "lao", "lo", "Lao" }, { "lat", "la", "Latin" },
			{ "lav", "lv", "Latvian" }, { "lim", "li", "Limburgan" }, { "lin", "ln", "Lingala" },
			{ "lit", "lt", "Lithuanian" }, { "ltz", "lb", "Luxembourgish" }, { "lub", "lu", "Luba-Katanga" },
			{ "lug", "lg", "Ganda" }, { "mah", "mh", "Marshallese" }, { "mal", "ml", "Malayalam" },
			{ "mar", "mr", "Marathi" }, { "mkd", "mk", "Macedonian" }, { "mlg", "mg", "Malagasy" },
			{ "mlt", "mt", "Maltese" }, { "mon", "mn", "Mongolian" }, { "mri", "mi", "Maori" },
			{ "msa", "ms", "Malay (macrolanguage)" }, { "mya", "my", "Burmese" }, { "nau", "na", "Nauru" },
			{ "nav", "nv", "Navajo" }, { "nbl", "nr", "South Ndebele" }, { "nde", "nd", "North Ndebele" },
			{ "ndo", "ng", "Ndonga" }, { "nep", "ne", "Nepali macrolanguage" }, { "nld", "nl", "Dutch" },
			{ "nno", "nn", "Norwegian Nynorsk" }, { "nob", "nb", "Norwegian Bokmål" }, { "nor", "no", "Norwegian" },
			{ "nya", "ny", "Nyanja" }, { "oci", "oc", "Occitan (post 1500)" }, { "oji", "oj", "Ojibwa" },
			{ "ori", "or", "Oriya macrolanguage" }, { "orm", "om", "Oromo" }, { "oss", "os", "Ossetian" },
			{ "pan", "pa", "Panjabi" }, { "pli", "pi", "Pali" }, { "pol", "pl", "Polish" },
			{ "por", "pt", "Portuguese" }, { "pus", "ps", "Pushto" }, { "que", "qu", "Quechua" },
			{ "roh", "rm", "Romansh" }, { "ron", "ro", "Romanian" }, { "run", "rn", "Rundi" },
			{ "rus", "ru", "Russian" }, { "sag", "sg", "Sango" }, { "san", "sa", "Sanskrit" },
			{ "sin", "si", "Sinhala" }, { "slk", "sk", "Slovak" }, { "slv", "sl", "Slovenian" },
			{ "sme", "se", "Northern Sami" }, { "smo", "sm", "Samoan" }, { "sna", "sn", "Shona" },
			{ "snd", "sd", "Sindhi" }, { "som", "so", "Somali" }, { "sot", "st", "Southern Sotho" },
			{ "spa", "es", "Spanish" }, { "sqi", "sq", "Albanian" }, { "srd", "sc", "Sardinian" },
			{ "srp", "sr", "Serbian" }, { "ssw", "ss", "Swati" }, { "sun", "su", "Sundanese" },
			{ "swa", "sw", "Swahili (macrolanguage)" }, { "swe", "sv", "Swedish" }, { "tah", "ty", "Tahitian" },
			{ "tam", "ta", "Tamil" }, { "tat", "tt", "Tatar" }, { "tel", "te", "Telugu" }, { "tgk", "tg", "Tajik" },
			{ "tgl", "tl", "Tagalog" }, { "tha", "th", "Thai" }, { "tir", "ti", "Tigrinya" },
			{ "ton", "to", "Tonga (Tonga Islands)" }, { "tsn", "tn", "Tswana" }, { "tso", "ts", "Tsonga" },
			{ "tuk", "tk", "Turkmen" }, { "tur", "tr", "Turkish" }, { "twi", "tw", "Twi" }, { "uig", "ug", "Uighur" },
			{ "ukr", "uk", "Ukrainian" }, { "urd", "ur", "Urdu" }, { "uzb", "uz", "Uzbek" }, { "ven", "ve", "Venda" },
			{ "vie", "vi", "Vietnamese" }, { "vol", "vo", "Volapük" }, { "wln", "wa", "Walloon" },
			{ "wol", "wo", "Wolof" }, { "xho", "xh", "Xhosa" }, { "yid", "yi", "Yiddish" }, { "yor", "yo", "Yoruba" },
			{ "zha", "za", "Zhuang" }, { "zho", "zh", "Chinese" }, { "zul", "zu", "Zulu" }, };

	// /*following added for jmdict */{ "ger", "de", "German" }, { "fre", "fr",
	// "French" },

	/**
	 * Lang code fix for Android (iso lang can have two identifier, JMdict can
	 * use one, and Android the other)
	 */
	public static final String[][] LANGS_FIX = { { "fre", "fra" }, { "ger", "deu" }, { "dut", "nld" } };

	public static String fixLang(String lang) {
		int size = LANGS_FIX.length;
		for (int i = 0; i < size; i++) {
			if (LANGS_FIX[i][0].equals(lang)) {
				return LANGS_FIX[i][1];
			}
		}
		return lang;
	}

	public static String getLangName(String threeLetters) {
		int size = LANGS_ARRAY.length;
		for (int i = 0; i < size; i++) {
			if (LANGS_ARRAY[i][0].equals(threeLetters)) {
				return LANGS_ARRAY[i][2];
			}
		}

		return null;
	}

	/**
	 * 
	 * @param twoLetters
	 *            the lang in two letter format: "en"
	 * @return the three letter format: "eng"
	 */
	public static String getLang3(String twoLetters) {
		int size = LANGS_ARRAY.length;
		for (int i = 0; i < size; i++) {
			if (LANGS_ARRAY[i][1].equals(twoLetters)) {
				return Common.fixLang(LANGS_ARRAY[i][0]);
			}
		}

		return twoLetters;
	}

	public static void createTablesCommon(SqlJetDb db, int version, String lang) throws SqlJetException {
		db.createTable(CREATE_TABLE_DB_INFO);

		db.beginTransaction(SqlJetTransactionMode.WRITE);
		ISqlJetTable tableDB = db.getTable(TABLE_DBINFO);
		// string key, string value, int value

		tableDB.insert(KEY_DB_LANG, fixLang(lang), null);
		tableDB.insert(KEY_DB_CREATION_DATE, VALUE_CREATION_DATE, null);
		tableDB.insert(KEY_DB_CREATION_TIME, null, Calendar.getInstance().getTimeInMillis() / 1000L);
		tableDB.insert(KEY_DB_VERSION, null, version);

		db.commit();
	}
}
