/*
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.dictdb;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;

import org.vono.narau.Log;
import org.vono.narau.dictdb.entry.Entry;
import org.vono.narau.dictdb.entry.KanjiElement;
import org.vono.narau.dictdb.entry.ReadingElement;

public class Dictionary {
	private static final String TAG = Dictionary.class.getSimpleName();

	private static final int NUM_ENTRIES = 162098;
	private static final boolean FIRST_MATCH_ONLY = true;

	private final HashMap<Integer, Entry> entries;
	private int maxEntryId;

	public Dictionary() {
		this.entries = new HashMap<Integer, Entry>(NUM_ENTRIES);
		this.maxEntryId = -1;
	}

	public void addEntry(Entry entry) {
		if (null != entry) {
			if (this.entries.containsKey(entry.sequence)) {

			} else {
				this.entries.put(entry.sequence, entry);
				if (this.maxEntryId < entry.sequence) {
					this.maxEntryId = entry.sequence;
				}
			}
		}
	}

	public Collection<Entry> getEntries() {
		return this.entries.values();
	}

	public int getMaxEntryId() {
		return this.maxEntryId;
	}

	/**
	 * Find an Entry. Search first for kanji elements before reading element.<br/>
	 * If there is more than one match, return null, an display a warning.
	 * 
	 * @param elem
	 *            Japanese text.
	 * @return The entry or null if not found
	 */
	public Entry findEntryFromJpn(String elem) {
		final Collection<Entry> collection = this.entries.values();

		final ArrayList<Entry> entriesFoundKanji = new ArrayList<Entry>();
		final ArrayList<Entry> entriesFoundKana = new ArrayList<Entry>();

		for (Entry entry : collection) {
			boolean foundEntry = false;

			final ArrayList<KanjiElement> k_ele = entry.k_ele;
			if (null != k_ele) {
				final int sKE = k_ele.size();
				for (int index = 0; index < sKE; index++) {
					KanjiElement ke = k_ele.get(index);
					if (elem.equals(ke.keb)) {
						entriesFoundKanji.add(entry);
						index = sKE;
						foundEntry = true;
					}
				}
			}
			if (!foundEntry && null != entry.r_ele) {
				// Search reading element
				final ArrayList<ReadingElement> r_ele = entry.r_ele;
				final int sRE = r_ele.size();

				for (int index = 0; index < sRE; index++) {
					final ReadingElement re = r_ele.get(index);
					if (elem.equals(re.reb)) {
						entriesFoundKana.add(entry);
						index = sRE;
						foundEntry = true;
					}
				}
			}

			if (foundEntry && FIRST_MATCH_ONLY) {
				break;
			}
		}

		// Return the best match entry
		// null if there is no entries or more than one

		final int numEntriesKanji = entriesFoundKanji.size();
		final int numEntriesKana = entriesFoundKana.size();

		Entry entry = null;

		if (numEntriesKanji > 1) {
			Log.w(TAG, "Many kanji entries found for '" + elem + "'");
			entry = entriesFoundKanji.get(0);
		} else if (numEntriesKanji == 1) {
			entry = entriesFoundKanji.get(0);
		} else if (numEntriesKana > 1) {
			Log.w(TAG, "Many kana entries found for '" + elem + "'");
			entry = entriesFoundKana.get(0);
		} else if (numEntriesKana == 1) {
			entry = entriesFoundKana.get(0);
		}

		return entry;
	}
}
