/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.dictdb.entry;

import java.util.ArrayList;

/**
 * <p>
 * entry (ent_seq, k_ele*, r_ele+, info?, sense+)
 * </p>
 * Entries consist of kanji elements, reading elements, general information and
 * sense elements. Each entry must have at least one reading element and one
 * sense element. Others are optional.
 * 
 */
public class Entry {

	/**
	 * A unique numeric sequence number for each entry
	 */
	public final int sequence;

	/**
	 * The kanji element, or in its absence, the reading element.
	 */
	public ArrayList<KanjiElement> k_ele;

	/**
	 * reading element
	 */
	public ArrayList<ReadingElement> r_ele;

	/**
	 * general information
	 */
	public Info info;

	/**
	 * sense elements
	 */
	public ArrayList<Sense> senses;

	public Entry(int seq) {
		this.sequence = seq;
		this.info = null;
		this.k_ele = new ArrayList<KanjiElement>();
		this.r_ele = new ArrayList<ReadingElement>(1);
		this.senses = new ArrayList<Sense>(1);
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof Entry) {
			final Entry e = (Entry) obj;
			return this.sequence == e.sequence;
		}
		return false;
	}

	@Override
	public String toString() {
		final StringBuilder sb = new StringBuilder();

		sb.append(this.sequence);
		sb.append(' ');

		final boolean haveKE = this.k_ele.size() > 0;

		if (haveKE) {
			for (KanjiElement ke : this.k_ele) {
				sb.append(ke);
				sb.append(";");
			}
			sb.append(" [");
		}
		for (ReadingElement re : this.r_ele) {
			sb.append(re);
			sb.append(";");
		}
		if (haveKE) {
			sb.append(']');
		}
		
		for (Sense s : this.senses) {
			sb.append('/');
			sb.append(s);
		}

		return sb.toString();
	}

}
