/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.dictdb.entry;

import java.util.ArrayList;

/**
 * <p>
 * r_ele (reb, re_nokanji?, re_restr*, re_inf*, re_pri*)
 * </p>
 * The reading element typically contains the valid readings of the word(s) in
 * the kanji element using modern kanadzukai. Where there are multiple reading
 * elements, they will typically be alternative readings of the kanji element.
 * In the absence of a kanji element, i.e. in the case of a word or phrase
 * written entirely in kana, these elements will define the entry.
 * 
 */
public class ReadingElement {
	/**
	 * 
	 * this element content is restricted to kana and related characters such as
	 * chouon and kurikaeshi. Kana usage will be consistent between the keb and
	 * reb elements; e.g. if the keb contains katakana, so too will the reb.
	 */
	public String reb;

	/**
	 * This element, which will usually have a null value, indicates that the
	 * reb, while associated with the keb, cannot be regarded as a true reading
	 * of the kanji. It is typically used for words such as foreign place names,
	 * gairaigo which can be in kanji or katakana, etc.
	 */
	public String re_nokanji;

	/**
	 * This element is used to indicate when the reading only applies to a
	 * subset of the keb elements in the entry. In its absence, all readings
	 * apply to all kanji elements. The contents of this element must exactly
	 * match those of one of the keb elements.
	 */
	public ArrayList<String> re_restr;

	/**
	 * General coded information pertaining to the specific reading. Typically
	 * it will be used to indicate some unusual aspect of the reading.
	 */
	public ArrayList<String> re_inf;

	/**
	 * See the comment on ke_pri above.
	 */
	public ArrayList<String> re_pri;

	@Override
	public String toString() {
		return this.reb;
	}

}