/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.dictdb.entry;

import org.vono.narau.Common;

/**
 * <p>
 * gloss (#PCDATA | pri)*
 * </p>
 * Within each sense will be one or more "glosses", i.e. target-language words
 * or phrases which are equivalents to the Japanese word. This element would
 * normally be present, however it may be omitted in entries which are purely
 * for a cross-reference.
 * 
 */
public class SenseGloss {

	public String text = null;

	/**
	 * The xml:lang attribute defines the target language of the gloss. It will
	 * be coded using the three-letter language code from the ISO 639 standard.
	 * When absent, the value "eng" (i.e. English) is the default value.
	 */
	public String lang = Common.LANG_DEFAULT;

	/**
	 * The g_gend attribute defines the gender of the gloss (typically a noun in
	 * the target language. When absent, the gender is either not relevant or
	 * has yet to be provided.
	 */
	public String g_gend = null;

	/**
	 * <p>
	 * ??? not use in file, not sure of the format</> These elements highlight
	 * particular target-language words which are strongly associated with the
	 * Japanese word. The purpose is to establish a set of target-language words
	 * which can effectively be used as head-words in a reverse
	 * target-language/Japanese relationship.
	 */
	public boolean pri = false;

	@Override
	public String toString() {
		return this.lang + ":" + this.text;
	}
}
