/**
 *  Copyright (C) 2012 Yvon TANGUY

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.dictdb.entry;

import org.vono.narau.Common;

/**
 * This element records the information about the source language(s) of a
 * loan-word/gairaigo. If the source language is other than English, the
 * language is indicated by the xml:lang attribute. The element value (if any)
 * is the source word or phrase.
 * 
 */
public class SenseLangSource {

	public static String TYPE_FULL = "full";
	public static String TYPE_PARTIAL = "part";

	public String text = null;

	/**
	 * The ls_wasei attribute indicates that the Japanese word has been
	 * constructed from words in the source language, and not from an actual
	 * phrase in that language. Most commonly used to indicate "waseieigo".
	 */
	public boolean wasei = false;

	/**
	 * The xml:lang attribute defines the language(s) from which a loanword is
	 * drawn. It will be coded using the three-letter language code from the ISO
	 * 639-2 standard. When absent, the value "eng" (i.e. English) is the
	 * default value. The bibliographic (B) codes are used.
	 */
	public String lang = Common.LANG_DEFAULT;

	/**
	 * The ls_type attribute indicates whether the lsource element fully or
	 * partially describes the source word or phrase of the loanword. If absent,
	 * it will have the implied value of "full". Otherwise it will contain
	 * "part".
	 */
	public String type = TYPE_FULL;

	public String toXML() {
		StringBuilder sb = new StringBuilder();
		sb.append("<lsource");
		if (this.wasei) {
			sb.append(" ls_wasei=\"y\"");
		}
		if (!Common.LANG_DEFAULT.equals(lang)) {
			sb.append(" xml:lang=\"");
			sb.append(this.lang);
			sb.append("\"");
		}

		if (!TYPE_FULL.equals(this.type)) {
			sb.append(" ls_type=\"");
			sb.append(TYPE_PARTIAL);
			sb.append("\"");
		}

		if (null == this.text) {
			sb.append("/>");
		} else {
			sb.append(">");
			sb.append(this.text);
			sb.append("</lsource>");
		}
		return sb.toString();
	}
}
